import * as apps from 'lib/apps';
import * as functions from 'lib/functions';
import { visualsPageFunctions } from 'lib/visuals-page/visuals-page-functions';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * Verify that the number of dashboards meet the specifications in the menu tab specified
 * Used for testing re-ordering tabs and dashboards within an app
 * @param {number} tabPosition
 * @param {number} numberOfDashboards
 */
export function verifyDashboardsNumInTab(tabPosition: number, numberOfDashboards: number) {
  cy.get(apps.locators.menusections + ':eq(' + tabPosition + ')')
    .find(apps.locators.dashboards_in_submenu)
    .should('have.length', numberOfDashboards);
}

/**
 * Verify that a dashboard exists in the menu tab in the specified position
 * Used for testing re-ordering tabs and dashboards within an app
 * @param {number} tabPosition
 * @param {number} dashboardPosition
 * @param {string} dashboardName
 */
export function verifyDashboardsNameInTab(
  tabPosition: number,
  dashboardPosition: number,
  dashboardName: string
) {
  cy.get(apps.locators.menusections + ':eq(' + tabPosition + ')')
    .find(apps.locators.dashboards_in_submenu + ':eq(' + dashboardPosition + ')')
    .should('contain', dashboardName);
}

/**
 * move subgroups around in an app
 * @param {number} sourcePosition Original position based on which tab to move
 * @param {number} targetPosition Tgt position of where to put the tab
 */
export function moveMenuTab(sourcePosition: number, targetPosition: number) {
  let targetMenuOffset: JQueryCoordinates | undefined;
  let sourceMenuOffset: JQueryCoordinates | undefined;
  cy.get(apps.locators.menusections + ':eq(' + targetPosition + ')').then(($targetMenu) => {
    targetMenuOffset = $targetMenu.offset();
    cy.get(apps.locators.menusections + ':eq(' + sourcePosition + ')').then(($sourceMenu) => {
      sourceMenuOffset = $sourceMenu.offset();
      const sourceRect = $sourceMenu[0].getBoundingClientRect();
      const targetRect = $targetMenu[0].getBoundingClientRect();
      console.log('sourceMenuOffset - mousedown  is ', sourceMenuOffset);
      console.log('targetMenuOffset - mousemove to is ', targetMenuOffset);

      if (!(targetMenuOffset && sourceMenuOffset)) {
        return;
      }

      cy.get(apps.locators.menusections + ':eq(' + sourcePosition + ')')
        .trigger('mousedown', {
          which: 1,
          pageX: sourceMenuOffset.left + sourceRect.width / 2,
          pageY: sourceMenuOffset.top + sourceRect.height / 2,
          force: true,
        })
        .trigger('mousemove', {
          which: 1,
          pageX: targetMenuOffset.left + targetRect.width / 2,
          pageY: targetMenuOffset.top + targetRect.height / 2,
          force: true,
        })
        .wait(1000)
        .trigger('mouseup', {
          force: true,
        });
    });
  });
}

/**
 * This will move a dashboard from a menu tab to another menu tab within an app
 * @param {number} sourceMenuTab
 * @param {number} sourceDashboardPosition
 * @param {number} targetMenuTab
 * @param {number} targetMenuPosition
 */
export function moveDashboardWithinApp(
  sourceMenuTab: number,
  sourceDashboardPosition: number,
  targetMenuTab: number,
  targetMenuPosition: number
) {
  let srcOffset: JQuery.Coordinates | undefined;
  let tgtOffset: JQuery.Coordinates | undefined;

  const srcLocator =
    '.app-menu-section:eq(' +
    sourceMenuTab +
    ') .menu-viz-list li:eq(' +
    sourceDashboardPosition +
    ')';
  let tgtLocator =
    '.app-menu-section:eq(' + targetMenuTab + ') .menu-viz-list li:eq(' + targetMenuPosition + ')';
  const tgtListLocator = '.app-menu-section:eq(' + targetMenuTab + ') .menu-viz-list';

  cy.get(tgtListLocator).then(($tgt) => {
    if ($tgt.find('li').length === 0) {
      tgtLocator = tgtListLocator;
      console.log('changing target locator to be just the top list');
    }
  });

  cy.get(srcLocator).then(($src) => {
    srcOffset = $src.offset();
    cy.get(tgtLocator).then(($tgt) => {
      tgtOffset = $tgt.offset();
      if (!(srcOffset && tgtOffset)) {
        return;
      }
      cy.get(srcLocator)
        .trigger('mousedown', {
          which: 1,
          pageX: srcOffset.left + 70,
          pageY: srcOffset.top + 87,
          force: true,
        })
        .trigger('mousemove', {
          which: 1,
          pageX: tgtOffset.left + 100, // add ~ height/2 of element to offset
          pageY: tgtOffset.top + 87 - 30,
          force: true,
        })
        .wait(1000)
        .trigger('mousemove', {
          which: 1,
          pageX: tgtOffset.left + 70, // add ~ height/2 of element to offset
          pageY: tgtOffset.top + 87 - 30,
          force: true,
        })
        .wait(1000)
        .trigger('mouseup', {
          which: 1,
          force: true,
        });
    });
  });
}

/**
 * This will add a subgroup with specified number of dashboards.
 * Assumes that appbuilder page is already open with an existing app.
 * Assumes samples already exist in metadata
 * @param numberOfDashboards : valid values are 0, 1, 2 for now. This
 * may be extended to more if needed in future tests.
 */
export function addSubgroupWithDashboards(numberOfDashboards: number) {
  // Add subgroup
  cy.get(apps.locators.addmenu).click();

  cy.get(apps.locators.menutitle_last)
    .click()
    .type('subgroup at ' + new Date());

  if (numberOfDashboards > 0) {
    // Add dashboard to latest subgroup
    cy.get(apps.locators.dataconnections)
      .click()
      .find('input')
      .type('samples');
    cy.get(apps.locators.dataconnections)
      .find('.chosen-results')
      .contains('samples')
      .click({ force: true });

    cy.get(apps.locators.datasets)
      .click()
      .find('input')
      .type('World Life Expectancy');
    cy.get(apps.locators.datasets)
      .find('.chosen-results')
      .contains('World Life Expectancy')
      .click({ force: true });
    cy.get(apps.locators.dashboard_selection)
      .contains('Life Expectancy Dashboard')
      .click();
  }

  if (numberOfDashboards === 2) {
    cy.get(apps.locators.dashboard_selection)
      .contains('Animated')
      .click();
  }
}

/**
 * add a Menu Tab to an app, and specify whether to save in optional parameter
 * @param {boolean} save -- optional param, specify true if need save after
 * menu tab creation
 */
export function addMenuTab(save?: boolean) {
  cy.get(apps.locators.addmenu).click();

  cy.get(apps.locators.menutitle_last)
    .click()
    .type('Menu tab at ' + new Date() + '{enter}');

  if (save) {
    saveApp();
  }
}

/**
 * save the existing App
 */
export function saveApp() {
  cy.intercept('GET', '/arc/reports/appgroup/*').as('appgroupDone');
  cy.get(apps.locators.appsave).click();
  cy.wait('@appgroupDone');
  cy.wait(4500);
  cy.get(apps.locators.appsave).should('not.have.class', 'app-btn-highlight');
}

/**
 * Adds the specified dashboard from the specified dataconn/dataset to the
 * menu tab specified. If menu tab not specified, adds it to the latest active
 * menu tab
 * @param {string} dataconnection
 * @param {string} dataset
 * @param {string} dashboard
 * @param {number} menuTabPosition - optional parameter
 */
export function addDashboard(
  dataconnection: string,
  dataset: string,
  dashboard: string,
  menuTabPosition?: number
) {
  if (menuTabPosition) {
    // make that menu tab position active so that the dashboard gets added to that
    cy.get(apps.locators.menusections + ':eq(' + menuTabPosition + ')').click();
  }
  // Add dashboard to latest subgroup
  cy.get(apps.locators.dataconnections)
    .click()
    .find('input')
    .type(dataconnection);
  cy.get(apps.locators.dataconnections)
    .find('.chosen-results')
    .contains(dataconnection)
    .click({ force: true });
  cy.get(apps.locators.datasets)
    .click()
    .find('input')
    .type(dataset);
  cy.get(apps.locators.datasets)
    .find('.chosen-results')
    .contains(dataset)
    .click({ force: true });
  cy.get(apps.locators.dashboard_selection)
    .contains(dashboard)
    .click();
}
/**
 * This will delete the latest dashboard that’s been added. Take the last
 * menusection and find the last dashboard in it and delete it
 * @param save
 */
export function deleteLastDashboard(save: boolean = false) {
  // No hover() functionality exists, so use trigger(mouseover) as workaround
  cy.get(apps.locators.dashboard_in_submenu_last)
    .trigger('mouseover')
    .find(apps.locators.dashboard_delete)
    .click();

  if (save) {
    saveApp();
  }
}

/**
 * This will delete the last menusection, not wait for save by default
 * @param save
 */
export function deleteLastSubgroup(save: boolean = false) {
  cy.get(apps.locators.menusections_last)
    .trigger('mouseover')
    .get(apps.locators.menusections_delete_last)
    .click();

  if (save) {
    saveApp();
  }
}

/**
 * delete an app
 * assumes the appedit builder page is loaded and you are trying to delete an
 * app within the page
 */
export function deleteAppFromBuilder() {
  // Reset test, delete app that was created
  cy.get(apps.locators.appmore).click();
  cy.get(apps.locators.appdelete).click();
  cy.get(apps.locators.appdeleteconfirmationtxt).type('DELETE');
  cy.get(apps.locators.appdeleteconfirmationbtn).click();

  cy.url().should('include', '/arc/apps/apps');
}

/**
 * create a new app by visiting the appedit page and giving a name to the app
 * and to the first menutab since it wont allow saving if at least one menu tab
 * doesn’t exist
 */
export function createNewApp(
  appName: string = 'Re-ordering App',
  tabname: string = 'First Tab at ' + new Date()
) {
  cy.visit('/arc/apps/apps');

  cy.intercept('GET', '/arc/reports/customcss').as('AppGetDone');
  cy.intercept('GET', '/arc/reports/report?report_ids=*').as('AppGetDone1');
  cy.intercept('GET', '/arc/datasets/dataset').as('AppGetDone2');

  cy.get(apps.locators.createNewButton).click();
  cy.get(apps.locators.appnew).click();

  cy.wait('@AppGetDone');
  cy.wait('@AppGetDone1');
  cy.wait('@AppGetDone2');

  cy.get('#app-name-input').type(appName);

  // give existing tab a title
  cy.get('.app-menu-section .app-menu-title')
    .click()
    .type(tabname + '{enter}');

  // 6. save
  saveApp();
}

/**
 * This will delete the app from the new app UI.
 * @param appName : name of the app which is to be deleted
 */
export function deleteAppFromNewUI(appName: string) {
  cy.intercept('DELETE', '/arc/reports/appgroup/*').as('reportDeleted');
  cy.intercept('GET', '/arc/reports/appgroup').as('appgroup');
  cy.get(`${CSSselector.Visual_page}`).click();
  cy.get(CSSselector.apps.favourites_tab).click();
  cy.get(`${CSSselector.Visuals_all}`).click();
  cy.get(`${CSSselector.gridview_Visualspage}`).click();
  cy.get(apps.locators.apps_list).click();

  functions.typeInSearchField(`"${appName}"`);

  visualsPageFunctions.selectReportInGrid(appName);

  cy.get(apps.locators.appdelete_new).click();

  cy.get(apps.locators.appdeleteconfirmationtxt).type('DELETE');

  cy.get(apps.locators.appdeleteconfirmationbtn).click();

  cy.wait('@reportDeleted')
    .its('response.statusCode')
    .should('eq', 200);
  cy.wait('@appgroup')
    .its('response.statusCode')
    .should('eq', 200);
}

/**
 * This will select the app from the new app UI.
 * @param appName : name of the app which is to be deleted
 */
export function selectAppFromVisualsPage(appName: string) {
  cy.intercept('DELETE', '/arc/reports/appgroup/*').as('reportDeleted');
  cy.intercept('GET', '/arc/reports/appgroup').as('appgroup');
  functions.navigate_to_visuals();

  cy.get(`${CSSselector.listview_Visualspage}`).click();
  cy.get(apps.locators.apps_list).click();

  functions.typeInSearchField(appName);

  visualsPageFunctions.selectReportInGrid(appName);
}

/** Launch the app
 * @param appName : name of app
 */
export function launchApp(appName: string) {
  functions.navigate_to_visuals();
  cy.get(`${CSSselector.Visuals_all}`).click();
  cy.get(`${CSSselector.listview_Visualspage}`).click();
  cy.get(apps.locators.apps_list).click();

  functions.typeInSearchField('"' + appName + '"');
  cy.wait(2000);
  cy.get(`[data-test-id="${appName}"]`).then(($el) => {
    const id = $el.data('test-report-id');
    cy.visit(`/arc/apps/appgroup/${id}`);
  });
}
