import { uglify, uglifyWithoutRandomizing } from './util/export-transform';

/**
 * Creates an API key that can be used for sending HTTP request to the Admin API
 * @param config.expires - example '2025-05-15T00:00:00+02:00' or 'Invalid date' (in this case it never expires)
 * @returns api key that can be used in request headers like this `Authorization: apikey ${apiKey}`
 */
export const createApiKey = (config?: {
  username?: string;
  password?: string;
  active?: 'true' | 'false';
  expires?: string;
}) => {
  const { username, password, active, expires } = {
    username: 'vizapps_admin',
    password: 'vizapps_admin',
    active: 'true',
    expires: 'Invalid date',
    ...(config ?? {}),
  };

  const formData = new FormData();
  formData.append('username', username);
  formData.append('password', password);
  formData.append('apikey', 'New key will be generated on save');
  formData.append('active', active);
  formData.append('expires', expires);

  return cy
    .request({
      method: 'POST',
      url: '/arc/apps/demoapikeys',
      body: formData,
    })
    .then((resp) => {
      const decoder = new TextDecoder();
      const decodedBody = decoder.decode(resp.body);

      return JSON.parse(decodedBody).secret_apikey;
    });
};

const updateProfile = (
  username: string,
  {
    should_change_password,
    user_data,
  }: {
    should_change_password?: boolean;
    user_data: {
      new_release_modal_seen?: string;
      homeapp?: number;
    };
  }
) => {
  return createApiKey().then((apiKey) => {
    const formData = new FormData();

    formData.append('should_change_password', should_change_password ? '1' : '0');
    formData.append('user_data', JSON.stringify(user_data));

    return cy
      .request({
        method: 'POST',
        url: `/arc/adminapi/edituserprofile/${username}`,
        headers: {
          Authorization: `apikey ${apiKey}`,
        },
        body: formData,
      })
      .then((response) => {
        const decoder = new TextDecoder();
        const decodedBody = decoder.decode(response.body);
        const body = JSON.parse(decodedBody) as {
          should_change_password: boolean;
          user_data: string;
        };

        const userData = JSON.parse(body.user_data);

        return {
          shouldChangePassowrd: body.should_change_password,
          userData,
        };
      });
  });
};

export enum DefaultRoleIds {
  SystemAdmin = 1,
  DatabaseAdmin = 2,
  Analyst = 3,
  VisualConsumer = 4,
}

export const VIZAPPS_ADMIN_USER_ID = 1;

interface User {
  id: number;
  username: string;
  is_superuser: boolean;
  is_active: boolean;
  date_joined: string;
  last_login: string;
  first_name: string;
  last_name: string;
  groups: Array<{ id: number; name: string }>;
  roles: Array<{ id: number; name: string }>;
  // these are the known keys, there might be others
  profile: {
    homeapp?: number;
    new_release_modal_seen?: string;
    last_viewed?: string[];
  };
}

interface Group {
  id: number;
  name: string;
  users: Array<{ id: number; name: string }>;
  roles: Array<{ id: number; name: string }>;
}

export const BackendAPI = {
  roles: {
    create: ({
      userNames: users,
      groupNames: groups,
      ...role
    }: {
      name: string;
      privs: Array<{ ptype: string; perms: string[]; dcid?: string; dslist?: string[] }>;
      desc?: string;
      userNames?: string[];
      groupNames?: string[];
    }) => {
      return createApiKey().then((apiKey) => {
        type ReturnType = [
          {
            id: number;
            name: string;
            desc: string;
            require_all_groups: boolean | null;
            users: string[];
            groups: string[];
            privs: [{ ptype: string; perms: string[] }];
          }
        ];

        return cy
          .request<ReturnType>({
            method: 'POST',
            url: `/arc/adminapi/v1/roles`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            form: true,
            body: { data: JSON.stringify([{ ...role, users, groups }]) },
          })
          .then((response) => response.body[0]);
      });
    },
    getByName: (name: string) =>
      createApiKey().then((apiKey) =>
        cy
          .request<
            Array<{
              id: number;
              name: string;
              desc: string;
              require_all_groups: boolean;
              perms: string[];
              users: string[];
              groups: string[];
            }>
          >({
            method: 'GET',
            url: `/arc/adminapi/v1/roles?name=${name}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
          })
          .then((response) => response.body[0])
      ),
    update: ({
      id,
      name,
      desc,
      groups,
      users,
    }: {
      id: number;
      name?: string;
      desc?: string;
      groups?: string[];
      users?: string[];
    }) =>
      createApiKey().then((apiKey) => {
        const role: {
          name?: string;
          desc?: string;
          groups?: string[];
          users?: string[];
        } = {};

        if (name) {
          role.name = name;
        }

        if (desc) {
          role.desc = desc;
        }

        if (groups) {
          role.groups = groups;
        }

        if (users) {
          role.users = users;
        }

        return cy
          .request({
            method: 'POST',
            url: `/arc/adminapi/v1/roles/${id}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            form: true,
            body: { data: JSON.stringify([role]) },
          })
          .then((response) => response.body[0]);
      }),
    delete: (id: number) =>
      createApiKey().then((apiKey) =>
        cy.request({
          method: 'DELETE',
          url: `/arc/adminapi/v1/roles/${id}`,
          headers: {
            Authorization: `apikey ${apiKey}`,
          },
        })
      ),
  },
  users: {
    getById: (id: number) =>
      createApiKey().then((apiKey) =>
        cy
          .request<[User]>({
            method: 'GET',
            url: `/arc/adminapi/v1/users/${id}?detail=1`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
          })
          .then((response) => response.body[0])
      ),
    getByName: (name: string) =>
      createApiKey().then((apiKey) =>
        cy
          .request<[User]>({
            method: 'GET',
            url: `/arc/adminapi/v1/users?name=${name}&detail=1`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            failOnStatusCode: false,
          })
          .then((response) => {
            if (response.status === 500 && (response.body as any) === 'User not found') {
              return (null as unknown) as User;
            }
            expect(response.status).to.equal(200);
            return response.body[0];
          })
      ),
    createOrRecreate: (user: { username: string; password: string }, roleIds?: number[]) => {
      // if the user exists, we delete it first:
      return BackendAPI.users
        .getByName(user.username)
        .then((foundUser) => {
          if (foundUser) {
            return BackendAPI.users.delete(foundUser.id);
          } else {
            return cy.wrap(null);
          }
        })
        .then(() =>
          BackendAPI.users.create({
            username: user.username,
            password: user.password,
            roleIds,
          })
        );
    },
    create: (
      {
        groupIds,
        roleIds,
        ...user
      }: {
        username: string;
        password: string;
        is_superuser?: boolean;
        is_active?: boolean;
        groupIds?: number[];
        roleIds?: number[];
      },
      config = {
        shouldChangePassword: false,
        shouldSeenWhatSNewModal: true,
      }
    ) => {
      return createApiKey().then((apiKey) => {
        let whatSNewVersion: string | undefined;

        if (config.shouldSeenWhatSNewModal) {
          cy.request({
            method: 'GET',
            url: '/arc/apps/home',
          }).then((response) => {
            const body = response.body as string;
            const whatSNewVersionExpression = new RegExp(/(?<=whatSNewVersion: ")[^"]*(?=")/);
            const matches = whatSNewVersionExpression.exec(body);
            if (matches === null) {
              throw new Error("What's new version was not found.");
            }

            whatSNewVersion = matches[0];
          });
        }

        const userData: {
          username: string;
          password: string;
          is_superuser?: boolean;
          is_active?: boolean;
          groups?: Array<{ id: number }>;
          roles?: Array<{ id: number }>;
        } = user;

        if (groupIds) {
          userData.groups = groupIds?.map((id) => ({ id }));
        }

        if (roleIds) {
          userData.roles = roleIds?.map((id) => ({ id }));
        }

        let createdUser: User;

        return cy
          .request<[User]>({
            method: 'POST',
            url: `/arc/adminapi/v1/users`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            form: true,
            body: {
              data: JSON.stringify([userData]),
            },
          })
          .then((response) => {
            createdUser = response.body[0];

            if (!config.shouldChangePassword || config.shouldSeenWhatSNewModal) {
              const data: {
                should_change_password: boolean;
                user_data: {
                  new_release_modal_seen?: string;
                  homeapp?: number;
                };
              } = {
                should_change_password: config.shouldChangePassword,
                user_data: { ...createdUser.profile },
              };

              if (config.shouldSeenWhatSNewModal) {
                if (whatSNewVersion === undefined) {
                  throw new Error("What's new version is not set");
                }

                data.user_data = { new_release_modal_seen: whatSNewVersion };
              }

              updateProfile(createdUser.username, data);
            }
          })
          .then(() => createdUser);
      });
    },
    update: ({
      id,
      groupIds,
      roleIds,
      username,
    }: {
      id: number;
      username?: string;
      groupIds?: number[];
      roleIds?: number[];
    }) => {
      return createApiKey().then((apiKey) => {
        const user: {
          username?: string;
          groups?: Array<{ id: number }>;
          roles?: Array<{ id: number }>;
        } = {};

        if (username) {
          user.username = username;
        }

        if (groupIds) {
          user.groups = groupIds.map((groupId) => ({ id: groupId }));
        }

        if (roleIds) {
          user.roles = roleIds.map((roleId) => ({ id: roleId }));
        }

        return cy
          .request({
            method: 'POST',
            url: `/arc/adminapi/v1/users/${id}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            form: true,
            body: { data: JSON.stringify([user]) },
          })
          .then((response) => response.body[0]);
      });
    },
    updateProfile,
    delete: (id: number) =>
      createApiKey().then((apiKey) =>
        cy.request({
          method: 'DELETE',
          url: `/arc/adminapi/v1/users/${id}`,
          headers: {
            Authorization: `apikey ${apiKey}`,
          },
        })
      ),
  },
  connections: {
    getByName: (name: string) =>
      createApiKey().then((apiKey) =>
        cy
          .request<[{ id: number; name: string; type: string }]>({
            method: 'GET',
            url: `/arc/adminapi/v1/connections?name=${name}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
          })
          .then((response) => response.body[0])
      ),
  },
  dashboards: {
    import: ({
      fileName,
      dataconnectionName,
      workspaceName,
      randomised = true,
    }: {
      fileName: string;
      dataconnectionName: string;
      workspaceName: string;
      randomised?: boolean;
    }): Cypress.Chainable<{
      dashboard: { id: string; name: string };
      datasets: [{ id: string; name: string }];
    }> =>
      cy.fixture(fileName).then((dashboardJson) => {
        randomised ? uglify(dashboardJson) : uglifyWithoutRandomizing(dashboardJson);

        return createApiKey().then((apiKey) =>
          cy
            .request<{
              dashboards: [{ id: string; name: string }];
              datasets: [{ id: string; name: string }];
            }>({
              method: 'POST',
              url: '/arc/migration/api/import/',
              form: true,
              body: {
                json_data: JSON.stringify(dashboardJson),
                dry_run: 'False',
                dataconnection_name: dataconnectionName,
                workspace: workspaceName,
              },
              headers: {
                Authorization: `apikey ${apiKey}`,
              },
            })
            .then((response) => {
              const dashboard = response.body.dashboards?.[0];
              const datasets = response.body.datasets;

              return { dashboard, datasets };
            })
        );
      }),
    getByName: (name: string) =>
      createApiKey().then((apiKey) =>
        cy
          .request<
            Array<{
              id: number;
              title: string;
              type: string;
              description: string;
            }>
          >({
            method: 'GET',
            url: `/arc/adminapi/v1/visuals?name=${name}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
          })
          .then((response) => response.body[0])
      ),
    delete: (id: number) =>
      createApiKey().then((apiKey) =>
        cy.request({
          method: 'DELETE',
          url: `/arc/adminapi/v1/visuals/${id}`,
          headers: {
            Authorization: `apikey ${apiKey}`,
          },
        })
      ),
  },
  groups: {
    getByName: (name: string) =>
      createApiKey().then((apiKey) =>
        cy
          .request<Group[]>({
            method: 'GET',
            url: `/arc/adminapi/v1/groups?name=${name}&detail=1`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            failOnStatusCode: false,
          })
          .then((response) => {
            if (response.status === 500 && (response.body as any) === 'Group not found') {
              return (null as unknown) as Group;
            }
            expect(response.status).to.equal(200);
            return response.body[0];
          })
      ),
    update: ({
      id,
      name,
      roleIds,
      userIds,
    }: {
      id: number;
      name?: string;
      roleIds?: number[];
      userIds?: number[];
    }) =>
      createApiKey().then((apiKey) => {
        const group: {
          name?: string;
          roles?: Array<{ id: number }>;
          users?: Array<{ id: number }>;
        } = {};

        if (name) {
          group.name = name;
        }

        if (roleIds) {
          group.roles = roleIds.map((roleId) => ({ id: roleId }));
        }

        if (userIds) {
          group.users = userIds.map((groupId) => ({ id: groupId }));
        }

        return cy
          .request({
            method: 'POST',
            url: `/arc/adminapi/v1/groups/${id}`,
            headers: {
              Authorization: `apikey ${apiKey}`,
            },
            form: true,
            body: { data: JSON.stringify([group]) },
          })
          .then((response) => response.body[0]);
      }),
  },
  apps: {
    create: ({ name, workspaceId }: { name: string; workspaceId: number }) => {
      let id: number | null = null;
      let csrfToken: string | undefined;

      const groupOptions =
        '{"user_settings":{"container_css_class":"","css_styles":"[]","inline_css":"","menu_on_left":"false","navigate_to_apps_page":"false","app_menubar_bg_color":"","app_menubar_bottombar_color":"","app_menubar_barcolor_selected":"","app_menutext_font":"default","app_menutext_font_size":"","app_menutext_font_weight":"default","app_menutext_color":"","app_menutext_color_selected":"","app_menutext_uppercase":"true","app_logo_image":"","app_logo_width":"185","app_logo_height":"45","app_logo_margintop":"5","app_logo_marginleft":"15","app_bg_bg_color":"","app_hide_dropshadow":"false","app_show_title":"false"}}';

      return cy
        .getCookie('arccsrftoken')
        .then((cookie) => {
          csrfToken = cookie?.value;

          return cy.request<{ saved_app_id: number }>({
            method: 'POST',
            url: '/arc/reports/appgroup',
            form: true,
            body: {
              csrfmiddlewaretoken: csrfToken,
              data: JSON.stringify([
                {
                  group_options: groupOptions,
                  group_name: name,
                  in_userlist: 'true',
                },
              ]),
              workspace_id: workspaceId,
            },
            headers: {
              Origin: Cypress.config().baseUrl,
            },
          });
        })
        .then((response) => {
          id = response.body.saved_app_id;

          cy.request({
            method: 'POST',
            url: '/arc/reports/move',
            form: true,
            body: {
              csrfmiddlewaretoken: csrfToken,
              dashboards_info: JSON.stringify([
                { report_id: response.body.saved_app_id.toString(), type: 'apps' },
              ]),
              current_workspace_id: workspaceId,
            },
            headers: {
              Origin: Cypress.config().baseUrl,
            },
          });
        })
        .then(() => id as number);
    },
  },
} as const;
