import * as arctest from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { dbClone, saveDashboard } from 'lib/dashboard';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';
import { locators } from './locators';

export function chooseVisualTab(tab: string, expandIt = true, visualIndex = 1) {
  console.log('Choosing data tab');
  editVisual(visualIndex);
  let tabSelector = '';
  let tabPanelSelector = '';
  let sectionShouldBeOpen = '';
  switch (tab) {
    case 'data':
      tabSelector = '[data-test-id="visual-data-tab-a"]';
      sectionShouldBeOpen = '#dash-visual-data-tab';
      break;
    case 'settings':
      tabSelector = '[data-test-id="visual-settings-tab-a"]';
      tabPanelSelector = '[data-test-id="dash-visual-settings-tab"] .header-expand';
      sectionShouldBeOpen = '[data-test-id="dash-visual-settings-tab"]';
      break;
    case 'colors':
    case 'style':
    case 'customStyle':
      tabSelector = '[data-test-id="visual-style-tab-a"]';
      tabPanelSelector = '[data-test-id="visual-style-tab-header-expand"]';
      sectionShouldBeOpen = '[data-test-id="dash-visual-style-tab"]';
      break;
    case 'segments':
      tabSelector = '#segments a';
      break;
  }

  cy.get(sectionShouldBeOpen).then(($elem) => {
    // avoid uninended clicking on buttons, which would hide the panel instead show it!
    if ($elem.hasClass('active')) {
      return;
    }

    cy.get(tabSelector).click();
    if (tabPanelSelector !== '') {
      cy.get(tabPanelSelector)
        .parent()
        .then(($el) => {
          const cls = $el.prop('class');
          cy.log('class:' + cls);
          if (cls.indexOf('closed') >= 0) {
            if (expandIt) {
              cy.log('Expanding the menu');
              cy.get(tabPanelSelector)
                .parent()
                .click();
            }
          } else if (expandIt === false) {
            cy.log('Closing the menu');
            cy.get(tabPanelSelector)
              .parent()
              .click();
          }
        });
    }
  });
}

export function openSettingsMarks() {
  cy.get("div[class='settings-tab-settings-holder'][data-tabname='Marks']")
    .parent()
    .then(($el) => {
      const cls = $el.prop('class');
      console.log('class:' + cls);
      if (cls.indexOf('closed') >= 0) {
        console.log('Expanding the menu');
        cy.get("div[class='settings-tab-settings-holder'][data-tabname='Marks']")
          .parent()
          .click();
      }
    });
}

export function closeVisualModal() {
  cy.get('.super-modal .report-menu-list li:first .modal-close-btn').click();
}

export function editVisual(index = 1) {
  const functionName = 'editVisual';
  cy.log(`${functionName} - start`);
  cy.get(CSSselector.visual_edit_specific(index)).click({
    force: true,
  });
  cy.log(`${functionName} - end`);
}

export function closeVisual(discard: boolean) {
  const functionName = 'closeVisual';
  cy.log(`${functionName} - start`);
  if (!discard) {
    cy.log(`${functionName} - saving changes`);
    saveVisual();
  }
  cy.log(`${functionName} - end`);
}

export function selectSpecificVisual(index = 1) {
  cy.get(locators.visual_select_specific.replace('index', index.toString())).click({
    force: true,
  });

  // wait for the "VISUAL" section to appear on the toolbar
  cy.get('.tabs-section-label-viz').should('be.visible');
}

export function activateBuildTab() {
  const functionName = 'activateBuildTab';
  cy.log(`${functionName} - start`);
  selectSpecificVisual();
  cy.get('[data-test-id="visual-data-tab-a"]').then(($a) => {
    const isAlreadyVisible = $a.hasClass('active');
    if (isAlreadyVisible) {
      cy.log(`buildTab already visible - ${functionName} - end`);
      return;
    }
    $a.trigger('click');
  });
  cy.log(`${functionName} - end`);
}

export function refreshVisual() {
  const functionName = 'refreshVisual';
  cy.log(`${functionName} - start`);
  activateBuildTab();
  cy.get('[data-test-id="viz-refresh"]').click();
  cy.log(`${functionName} - end`);
}

export function saveVisual() {
  const functionName = 'saveVisual';
  cy.log(`${functionName} -  started`);
  refreshVisual();

  dashboards.saveDashboard();

  cy.log(`${functionName} -  finished`);
}

export function openMoreButtonInVisualEdit() {
  cy.get('#report-menu-list .dropdown').within(() => {
    cy.get('.fa.fa-ellipsis-h').click();
  });
}

/**
 * Deletes the dashboard with  'cloneDashboardName' and then clones the dashboard with 'orgDashboardName'
 * and saves it with the 'cloneDashboardName'
 * @export
 * @param {string} orgDashboardName
 * @param {string} cloneDashboardName
 */
export function cloneDashboard(orgDashboardName: string, cloneDashboardName: string) {
  dashboards.deleteDashboard(cloneDashboardName);
  dashboards.openDashboardforEdit(orgDashboardName);

  dbClone();

  cy.get('[data-test-id="dashboard-title"]')
    .click({ force: true })
    .clear()
    .type(cloneDashboardName);

  saveDashboard();
}

export function createBarVisual(
  testDashbboardName: string,
  dataconnectionName = 'samples',
  dataSetName = 'Cereals',
  shelfs = {
    x_shelfs: ['manufacturer'],
    y_shelfs: ['protein_grams'],
    color_shelfs: ['fat_grams', 'sodium_mg', 'sugars_grams'],
  },
  barType: '#chart-trellis-bars' | '#chart-trellis-groupedbars' = '#chart-trellis-bars'
) {
  arctest.misc.newDashboardEnterModal(dataconnectionName, dataSetName, testDashbboardName);
  dashboards.editVisual();
  // Change type to Bar visual
  cy.get(`${visual.locators.visual_icons}${barType}`).click();
  shelfs?.x_shelfs.forEach((dimension) =>
    arctest.screen.builder.modal.addShelfColumn('x_shelf', dimension)
  );
  shelfs?.y_shelfs.forEach((measures) =>
    arctest.screen.builder.modal.addShelfColumn('y_shelf', measures)
  );
  shelfs?.color_shelfs.forEach((color) =>
    arctest.screen.builder.modal.addShelfColumn('color_shelf', color)
  );
  // fill with some value X and Y and color shelfs
  cy.get('[data-test-id="viz-refresh"]').click({ force: true });
}

export function createTableVisual(
  testDashbboardName: string,
  dataconnectionName = 'samples',
  dataSetName = 'Cereals',
  shelfs = {
    x_shelfs: ['manufacturer'],
    y_shelfs: ['protein_grams'],
  }
) {
  const type = 'chart-icon-table';
  arctest.misc.newDashboardEnterModal(dataconnectionName, dataSetName, testDashbboardName);
  dashboards.editVisual();
  shelfs?.x_shelfs.forEach((dimension) =>
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', dimension)
  );
  shelfs?.y_shelfs.forEach((measures) =>
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', measures)
  );

  // fill with some value X and Y and color shelfs
  cy.get('[data-test-id="viz-refresh"]').click({ force: true });
}
