import { notifications } from 'fixtures/locators/notifications';
import * as apps from 'lib/apps';
import * as dashboards from 'lib/dashboard';
import {
  Dashboard,
  DropdownFilter,
  FilterTypes,
  Visual,
} from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import * as functions from 'lib/functions';
import { createNewVisual, modalClick, waitForVisualsLoaded } from 'lib/functions';
import * as visuals from 'lib/visual';
import { visualsPageFunctions } from 'lib/visuals-page/visuals-page-functions';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

export function newDashboard(dashboardDetail: Dashboard): void {
  const functionName = 'newDashboard';
  cy.log(`${functionName} - started`);

  cy.get(CSSselector.homepage).click();
  cy.get(CSSselector.home_new_dashboard).click();

  cy.get(CSSselector.Dashboard_title).should('be.visible');
  cy.get(CSSselector.Dashboard_title)
    .click()
    .type(dashboardDetail.title);
  cy.get(CSSselector.Dashboard_subtitle)
    .click()
    .type(dashboardDetail.subTitle);
  cy.log(`${functionName} - finished`);
}

export function newVisual(visualDetail: Visual): void {
  return functions.createNewVisual(visualDetail.connection_name, visualDetail.dataset_name);
}

export function deleteDashboard(dashboardTitle: string): void {
  functions.navigate_to_visuals();
  cy.get(CSSselector.Visuals_all).click();

  cy.get(workspace.locators.dashboard_radio_btn)
    .scrollIntoView()
    .click();

  functions.typeInSearchField(dashboardTitle);

  const dashboardTileSelector = `[data-test-id="${dashboardTitle}"]`;
  cy.document().then((doc) => {
    if (doc.querySelectorAll(dashboardTileSelector).length) {
      visualsPageFunctions.selectReportInGrid(dashboardTitle);

      cy.get(apps.locators.appdelete_new).click();

      cy.get(`${apps.locators.appdeleteconfirmationtxt}:visible`).type('DELETE');

      cy.intercept('DELETE', '/arc/reports/report/*').as('reportDeleted');

      cy.get(apps.locators.appdeleteconfirmationbtn).click();

      cy.wait('@reportDeleted');

      cy.get(`[data-test-id="${dashboardTitle}"]`).should('not.exist');
    }
  });
}

export function openDashboardforEdit(dashboardTitle: string) {
  openDashboard(dashboardTitle);

  cy.get('[data-test-id="report-menu-list-editapp-link"]').click();
  cy.url().should('include', '/arc/apps/builder/');

  functions.waitForVisualsLoaded();
}

export function openDashboard(dashboardTitle: string) {
  functions.navigate_to_visuals();

  cy.get(workspace.locators.dashboard_radio_btn).click();

  functions.typeInSearchField(dashboardTitle);

  cy.get(`[data-test-id="${dashboardTitle}"]`).click();

  cy.url().should('include', '/arc/apps/app/');
  functions.waitForVisualsLoaded();
}

export function createDashboard(dashboardDetail: Dashboard) {
  const functionName = 'createDashboard';
  cy.log(`${functionName} - started`);
  deleteDashboard(dashboardDetail.title);
  functions.navigate_to_home();
  newDashboard(dashboardDetail);

  cy.log(`${functionName} - start adding sheets and visuals`);
  for (const sheet of dashboardDetail.sheets) {
    visuals.createNewSheet(sheet);
    sheet.visuals.forEach((visual, visualNumber) => {
      newVisual(visual);
      visuals.editVisual(visualNumber + 1, visual); // +1 as visual numbers start from 1 not 0
    });

    if (sheet?.dropdownFilters && sheet?.dropdownFilters.length) {
      sheet.dropdownFilters.forEach((dropdownFilter) => {
        createDashboardFilter(dropdownFilter);
      });
    }
    // click on save dashboard button
    saveDashboard();
  }

  if (dashboardDetail.sheets.length > 1) {
    cy.get("[data-test-id='sheet-number-1']").trigger('click');
  }

  cy.log(`${functionName} - sucessfully added sheets and visuals`);
  cy.log(`${functionName} - fininshed`);

  return cy.url().then((url) => {
    const dashboadId = url.split('/apps/builder/')[1].split('?')[0];
    return cy.wrap(dashboadId);
  });
}

export function createDashboardFilter(dropdownFilter: DropdownFilter) {
  const filter_setting_button = dashboards.locators.filter_setting_button;
  cy.get(filter_setting_button).trigger('click');

  const filterItem = dashboards.locators.filter_element(
    dropdownFilter.shelfType,
    dropdownFilter.fieldName
  );
  cy.get(filterItem).trigger('click');
  cy.wait(2000);

  if (dropdownFilter.filterOptions) {
    if (dropdownFilter.filterType === FilterTypes.DATE) {
      configureDateFilter(dropdownFilter);
    } else if (dropdownFilter.filterType === FilterTypes.RANGE) {
      configureRangeFilter(dropdownFilter);
    } else {
      configureFilter(dropdownFilter);
    }

    if (
      dropdownFilter.filterOptions.values.length &&
      dropdownFilter.filterType !== FilterTypes.RANGE
    ) {
      filter.applyDropdownFilterToVisual(dropdownFilter.filterOptions.values);
    }
  }
  saveDashboard();
}

function configureDateFilter(dropdownFilter: DropdownFilter) {
  if (dropdownFilter.filterOptions) {
    filter.configureDateFilterBasic(
      dropdownFilter.fieldName,
      dropdownFilter.filterOptions.title,
      dropdownFilter.fieldName,
      dropdownFilter.filterOptions.allowTimeRange
    );
    filter.configureDateFilterScope(
      dropdownFilter.filterOptions.visibility,
      dropdownFilter.filterOptions.scope
    );
  }
}

function configureFilter(dropdownFilter: DropdownFilter) {
  if (dropdownFilter.filterOptions) {
    filter.configureFilterValues(
      dropdownFilter.fieldName,
      dropdownFilter.filterOptions.title,
      dropdownFilter.fieldName,
      ''
    );
    filter.configureFilterScope(
      dropdownFilter.filterOptions.visibility,
      dropdownFilter.filterOptions.scope
    );
  }
}

function configureRangeFilter(dropdownFilter: DropdownFilter) {
  // TODO: Configure this range filter if needed
}

export function saveDashboard() {
  waitForVisualsLoaded();

  cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
  cy.get(CSSselector.Save).click();
  cy.wait('@savedashboard')
    .its('response.statusCode')
    .should('eq', 200);

  waitForVisualsLoaded();
  cy.get('[data-test-id="app-save-link"]:not(.app-btn-highlight):visible').should('exist');
}

export function createLinkedVisual(
  vIndex: number,
  vName: string,
  dataconnection: string,
  dataset: string
) {
  createNewVisual(dataconnection, dataset);

  cy.get(`${CSSselector.visualEdit.visual}:nth-child(${vIndex})`).dblclick();

  if (vName !== '') {
    cy.get(CSSselector.visual_title_specific(vIndex))
      .click()
      .clear()
      .type(vName);

    // we must click outside of the input field as we do not have a {tab} key
    cy.get(CSSselector.Dashboard_title).click();
  }

  cy.get(CSSselector.visualEdit.ellipsis_index.replace('index', String(vIndex))).click();
  cy.get(CSSselector.visualEdit.linkUnlink).click();

  cy.intercept('POST', 'arc/reports/report/*').as('visualSaved');
  modalClick(CSSselector.visualEdit.linkUnlinkConfirm);
  cy.wait('@visualSaved');

  saveDashboard();
}

export function addVisual(vIndex: number, vName: string) {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  cy.intercept('GET', '/arc/reports/report/*').as('visualGet');
  cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');

  cy.get("[data-test-id='#dash-viz-tab']").then(($a) => {
    if ($a.closest('li').hasClass('active')) {
      return;
    }

    $a.trigger('click');
  });

  cy.get(CSSselector.selecting_visuals).click();
  cy.wait('@visualCreated');
  cy.wait('@visualGet');
  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  functions.waitForVisualsLoaded();

  cy.get(`${CSSselector.visualEdit.visual}:nth-child(${vIndex})`).dblclick();
  if (vName !== '') {
    cy.get(CSSselector.visual_title_specific(vIndex))
      .click()
      .clear()
      .type(vName);
  }
  cy.intercept('POST', 'arc/reports/report/*').as('visualSaved');

  cy.get(CSSselector.Save).click();
  cy.wait('@visualSaved')
    .its('response.statusCode')
    .should('eq', 200);
  cy.get(CSSselector.Save).should('not.have.class', 'app-btn-highlight');
}

export function dbMoveToWS(workspaceName: string) {
  cy.intercept('GET', '/arc/workspace/workspace').as('workspaceLoaded');
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
  ).click();
  cy.wait('@workspaceLoaded');
  cy.get(`[data-workspace-name="${workspaceName}"]:visible`).click();
  cy.get(CSSselector.dashboardDetail.workspaceMove).click();
  cy.get(
    `${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${CSSselector.dashboardDetail.workspaceMoveMoveVisualRadio}`
  ).each(($el) => {
    cy.wrap($el).click();
  });
  cy.intercept('POST', '/arc/reports/move').as('moveDashboard');
  modalClick(
    `${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${CSSselector.dashboardDetail.workspaceMoveMoveNow}`
  );
  cy.wait('@moveDashboard')
    .its('response.statusCode')
    .should('eq', 200);
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_close).click();
}

/** This will simply clone the dashboard which will be placed in user's private workspace by default.
 * @export
 */
export function dbClone() {
  cy.intercept('POST', '/arc/reports/savedashboard').as('cloneDashboard');

  return interceptTabOpening(() => {
    cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
    cy.get(CSSselector.workspace_regression.Dashboard_clone).click();

    cy.wait('@cloneDashboard')
      .its('response.statusCode')
      .should('eq', 200);

    // TODO figure out why we need to wait here
    return cy.wait(100);
  }).then((url) => {
    const clonedDashboardId = url.split('/apps/builder/')[1];
    cy.log(`about to visit dashboard ${clonedDashboardId}`);
    cy.visit(`/arc/apps/builder/${clonedDashboardId}`);
    waitForVisualsLoaded();
    return cy.wrap(clonedDashboardId);
  });
}

export function dbCloneToWS(workspaceName: string) {
  cy.intercept('GET', '/arc/workspace/workspace').as('workspaceLoaded');
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
  ).click();
  cy.wait('@workspaceLoaded');
  cy.get(`[data-workspace-name="${workspaceName}"]:visible`).click();
  cy.get(CSSselector.dashboardDetail.workspaceMove).click();
  cy.get(
    `${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${CSSselector.dashboardDetail.workspaceMoveCloneVisualRadio}`
  ).each(($el) => {
    cy.wrap($el).click();
  });
  cy.intercept('POST', '/arc/reports/move').as('moveDashboard');
  modalClick(
    `${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${CSSselector.dashboardDetail.workspaceMoveMoveNow}`
  );
  cy.wait('@moveDashboard')
    .its('response.statusCode')
    .should('eq', 200);
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_close).click();
}

function interceptTabOpening(action: () => Cypress.Chainable<any>) {
  cy.window().then((win) => {
    cy.stub(win, 'open').as('open');
  });

  return action().then(() => {
    return cy.get<{ getCall: (index: number) => { args: string[] } }>('@open').then((subject) => {
      const [url, arg] = subject.getCall(0).args;

      expect(arg).to.equal('_blank');
      return url;
    });
  });
}

export function dbCloneMoveoptionSelection(selector?: string) {
  cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
  cy.get(CSSselector.workspace_regression.Dashboard_clone).click();

  if (selector) {
    cy.get(`${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${selector}`).each(($el) => {
      cy.wrap($el).click();
    });
  }

  cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');

  interceptTabOpening(() => {
    if (selector) {
      modalClick(
        `${CSSselector.dashboardDetail.workspaceMoveCloneModel} ${CSSselector.dashboardDetail.workspaceMoveMoveNow}`
      );
    }

    cy.wait('@savedashboard')
      .its('response.statusCode')
      .should('eq', 200);

    // TODO figure out why we need to wait here
    return cy.wait(5_000);
  }).then((url: any) => {
    const clonedDashboardId = url.split('/apps/builder/')[1];
    cy.log(`about to visit dashboard ${clonedDashboardId}`);
    cy.visit(`/arc/apps/builder/${clonedDashboardId}`);
    waitForVisualsLoaded();
  });
}

/**
 * this will verify name of a sheet in the dashboard.
 * @param index : sheet index in the dashboard
 * @param sheetName: sheet name to be verified
 */
export function verifyDashboardSheetName(index: number, sheetName: string) {
  cy.get(`${CSSselector.renameSheet.sheet}:nth-child(${index})`)
    .find(CSSselector.Sheet_title)
    .should('have.text', sheetName);
}

/**
 * This function is used when user is on dashboard edit and want to move to dashboad view page
 */
export function viewDashboard() {
  cy.intercept('GET', '/arc/workspace/workspace').as('viewDashboard');
  cy.get(CSSselector.view_mode).click({ force: true });
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('dashboardviewed');
  cy.wait('@dashboardviewed')
    .its('response.statusCode')
    .should('eq', 200);
}

/**
 * This function is used when user is on dashboard view page and want to move to dashboard edit page
 */
export function editDashboard() {
  cy.intercept('GET', '/arc/datasets/dataconnection').as('editDashboard_dataconnection');
  cy.get(CSSselector.edit_dashboard).click({ force: true });
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('dashboardedited');
  cy.wait('@editDashboard_dataconnection')
    .its('response.statusCode')
    .should('eq', 200);
  cy.wait('@dashboardedited');
}

/**
 * Opens the given dashboard in view mode
 * @param dashboardId
 */
export function viewDashboardById(dashboardId: string | number) {
  const url = `/arc/apps/app/${dashboardId}`;
  cy.visit(url);
  functions.waitForVisualsLoaded();
}

/**
 * Opens the given dashboard in edit mode
 * @param dashboardId
 */
export function editDashboardById(dashboardId: string | number) {
  const url = `/arc/apps/builder/${dashboardId}`;
  cy.visit(url);
  functions.waitForVisualsLoaded();
}

export function checkErrorMessage(mode: string) {
  cy.get('[data-test-id="main-app-arc-app-container"].' + mode + '-mode').then(($con) => {
    if ($con && $con.attr('class')?.includes('error')) {
      cy.log('Error thrown...');
      cy.get('#main>span .fa.fa-exclamation-circle').click({ force: true });
      cy.get('.popover.fade.bottom.in .error-list-detail').then(($message) => {
        cy.log($message.text());
      });
      cy.get('.popover.fade.bottom.in .btn.btn-default.btn-sm.popover-close').click();
      // cy.get('#app-selected-list').click({ force: true });
    } else {
      cy.log('No Error thrown...');
    }
  });
}

/**
 * Locates the orange error indicators in a dashboard
 */
export function getErrorIndicators() {
  return cy.get('body').find(dashboards.locators.error_indicator_visible);
}
