// ***********************************************
// This example commands.js shows you how to
// create various custom commands and overwrite
// existing commands.
//
// For more comprehensive examples of custom
// commands please read more here:
// https://on.cypress.io/custom-commands
// ***********************************************
//
//
// -- This is a parent command --
// Cypress.Commands.add("login", (email, password) => { ... })
//
//
// -- This is a child command --
// Cypress.Commands.add("drag", { prevSubject: 'element'}, (subject, options) => { ... })
//
//
// -- This is a dual command --
// Cypress.Commands.add("dismiss", { prevSubject: 'optional'}, (subject, options) => { ... })
//
//
// -- This is will overwrite an existing command --
// Cypress.Commands.overwrite("visit", (originalFn, url, options) => { ... })
import { jobs } from 'fixtures/locators/jobs';
import { notifications } from 'fixtures/locators/notifications';
import { navigateToConnection } from 'lib/analyticalview_functions';
import { createApiKey } from 'lib/backend-api';
import { locators as dashoardBuilderPage, openDashboard } from 'lib/dashboard';
import { locators } from 'lib/users_groups/locators';
import { CSSselector, CSSselectorType } from 'locators/common/CSSselector';

interface LookupObject {
  [key: string]: any;
}

/**
 * method assumes you have navigated to data tab and have the correct connection name selected
 * called in the createDatasetFromConnection
 * @export
 * @param {string} databaseName
 * @param {string} tableName
 * @param {string} datasetName
 */
export function createNewDataset(databaseName: string, tableName: string, datasetName: string) {
  let createDatasetFlag = true;
  cy.get(CSSselector.data.dataset_table_row).each(($el, index, $list) => {
    // index starts from 1 so we account for it while clicking
    if ($el.text().includes(datasetName)) {
      // yup found it
      console.log(`matched for the dataset name ${$el.text()}`);
      createDatasetFlag = false;
      return false;
    } else {
      if ($list.length === index + 1 && createDatasetFlag) {
        console.log(`not matched to ${datasetName}`);
        cy.get(CSSselector.data.new_dataset_button).click();
        cy.intercept('GET', '/arc/reports/columns/1/*').as('get_report');
        cy.intercept('POST', 'arc/datasets/dataset').as('dataset_created');

        cy.get('[data-test-id="dataset-create-modal"] [data-test-id="loading-spinner"]').should(
          'not.exist'
        );

        cy.get(CSSselector.data.new_dataset_modal.dataset_title).type(datasetName);
        cy.get(CSSselector.data.new_dataset_modal.database_select)
          .click()
          .find('input')
          .type(databaseName)
          .type('{enter}');

        cy.get('[data-test-id="dataset-create-modal"] [data-test-id="loading-spinner"]').should(
          'not.exist'
        );

        cy.get(CSSselector.data.new_dataset_modal.table_select)
          .click()
          .find('input')
          .type(tableName)
          .type('{enter}');
        cy.get(CSSselector.data.new_dataset_modal.create_button).click();
        cy.wait('@get_report');
        cy.wait('@dataset_created')
          .its('response.statusCode')
          .should('eq', 200);
      }
    }
  });
}

/**
 * creates a connection of type arcadia with deafault hostname if it doesnt exist else use the existing one if it does
 * creates a dataset if it doesnt exist else it doesnt create
 * @export
 * @param {string} connectionName
 * @param {string} databaseName
 * @param {string} tableName
 * @param {string} datasetName
 */
export function createDatasetFromConnection(
  connectionName: string,
  databaseName: string,
  tableName: string,
  datasetName: string
) {
  // Set aliases to validate or wait on
  let createConnectionFlag = true;
  navigate_to_data();

  cy.get(CSSselector.data.connection_list).each(($el, index, $list) => {
    // index starts from 1 so we account for it while clicking
    console.log(`The index is ${index} and the length is ${$list.length}`);
    // if ($el.filter((element) => { return element.text() === connection_name})) {
    console.log(`for the name ${$el.text()}`);

    if ($el.text() === connectionName) {
      // yup found it
      console.log(`found a connection name ${$el.text()}`);
      createConnectionFlag = false;
      return false;
    } else {
      // check if we reached the end of connections list while searching for the connection name or connection name is found
      if ($list.length === index + 1 && createConnectionFlag) {
        console.log('not found a connection name');
        // not found the connection name - create one of type SQLite
        cy.get(CSSselector.data.newconnection_button).click();
        cy.intercept('POST', '/arc/datasets/dataconnection').as('connection_created');
        // create default arcadia connection
        cy.fixture('connections.json').then((connection) => {
          cy.get(CSSselector.data.new_data_connection.connection_type_dropdown).select(
            connection.default.connection_type,
            { force: true }
          );
          cy.get(CSSselector.data.new_data_connection.connection_name_textbox).type(connectionName);
          cy.get(CSSselector.data.new_data_connection.hostname_textbox).type(
            connection.default.hostname
          );
          cy.get(CSSselector.data.new_data_connection.port_textbox)
            .clear()
            .type(connection.default.port);
          cy.get(CSSselector.data.new_data_connection.save_button).click();
          cy.wait('@connection_created')
            .its('response.statusCode')
            .should('eq', 200);
        });
      }
    }
  });

  cy.get(CSSselector.data.connection_list).should('contain', connectionName);
  cy.get(CSSselector.data.connection_list + ' a[title=' + connectionName + ']').click();
  // cy.get(CSSselector.data.connection_list).contains(connection_name).click()

  cy.get(CSSselector.data.datasets_tab).click();

  createNewDataset(databaseName, tableName, datasetName);
}

export function deleteDatasetFromConnection(connectionName: string, datasetName: string) {
  const functionName = 'deleteDatasetFromConnection';
  cy.log(`${functionName} - started`);
  navigateToConnection(connectionName);
  cy.get('#datasets-table').then(($elem) => {
    const datasetTitles = $elem.find('.dataset-title');
    let i;
    for (i = 0; i < datasetTitles.length; i++) {
      if (datasetTitles[i].textContent === datasetName) {
        cy.log(`${functionName} - ${datasetName} found and will be deleted!`);
        $elem.find(CSSselector.data.trash_icon)[i].click();
        cy.get('input[data-test-id=viz-delete-dataset-confirm]').type('DELETE');
        cy.get('button[data-test-id=delete-confirmation-yes]')
          .eq(1)
          .should('be.not.disabled')
          .click();
        break;
      }
    }
  });
  cy.log(`${functionName} - finished`);
}

/**
 * navigates to homepage and clicks on new dashboard
 * and creates a new dashboard with the title @param dashname
 * and subtitle @param dashname+'subtitle'
 * @export
 * @param {string} dashname
 */
export function addNewDB(dashname: string) {
  // Set aliases to validate or wait on

  cy.intercept('GET', '/arc/datasets/dataset').as('createDashboardNav');

  // cy.get(CSSselectorects.homepage).click();
  navigate_to_home();
  cy.get(CSSselector.home_new_dashboard).click();

  cy.wait('@createDashboardNav');
  cy.get(CSSselector.Dashboard_title)
    .click()
    .type(dashname);
  cy.get(CSSselector.Dashboard_subtitle)
    .click()
    .type(`${dashname} subtitle`);

  // cy.createNewVisual(dataconnection, dataset)
  // cy.get(CSSselectorects.Save).click()
}

/**
 * Creates a default table visual from the dataconnection and dataset specified
 *
 * @export
 * @param {string} dataconnection
 * @param {string} dataset
 */
export function createNewVisual(dataconnection: string, dataset: string) {
  cy.intercept('GET', '/arc/reports/report/*').as('visualGet');
  cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');

  cy.get("[data-test-id='#dash-viz-tab']:visible", { timeout: 30000 }).should('exist');

  cy.get("[data-test-id='#dash-viz-tab']").then(($a) => {
    const isAlreadyVisible = $a.hasClass('active');
    if (isAlreadyVisible) {
      return;
    }
    $a.trigger('click');
  });

  cy.get(CSSselector.selecting_dataconnection)
    .click()
    .find('input')
    .type(dataconnection);
  cy.get(CSSselector.selecting_dataconnection)
    .find('.chosen-results')
    .contains(new RegExp(`^${dataconnection}$`))
    .click();

  cy.get(CSSselector.selecting_datasets)
    .click()
    .find('input')
    .type(dataset);
  cy.get(CSSselector.selecting_datasets)
    .find('.chosen-results')
    .contains(new RegExp(`^${dataset}$`))
    .click();

  cy.get(CSSselector.selecting_visuals).click();

  cy.wait('@visualCreated')
    .its('response.statusCode')
    .should('eq', 200);
  cy.wait('@visualGet');

  waitForVisualsLoaded();
}

/**
 * Adds a new filter field from filter sidemenu
 * on the edit dashboard screen
 *
 * @export
 * @param {string} dataconnection
 * @param {string} dataset
 * @param {string} field
 */
export function addNewFilterFieldToVisual(dataconnection: string, dataset: string, field: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('filterAdded');
  cy.intercept('GET', '/arc/reports/report/*').as('filterCreated');

  // Conditional wait - an example : just ensure the first element you get on is always completely renedered by this time
  cy.get(dashoardBuilderPage.dashboardSettingPanel.filterSection).then(($sidePanel) => {
    // dash-settings-panel
    if ($sidePanel.find('.active').length) {
      // found - do nothing
      console.log(`Found side panel to be active - ${$sidePanel.find('active').length}`);
    } else {
      cy.get(CSSselector.filters_tab).click();
    }
  }); // get the sidepannel to a state where side panel is active

  cy.get(CSSselector.selecting_dataconnection_filter).select(dataconnection);

  // setting force: true because this happens in between the selection of dataconnection and data set - > This element '<span.selection_field>' is not visible because its parent '<div.dash-settings-panel.is-open>' has CSS property: 'display: none'
  cy.get(CSSselector.selecting_datasets_filter).select(dataset, {
    force: true,
  });
  cy.get(CSSselector.selecting_filter_field.replace('field_name', field)).click({
    force: true,
  });

  cy.wait('@filterAdded')
    .its('xhr.responseText' as any)
    .should('not.be', null);

  cy.wait('@filterCreated')
    .its('response.body.0.dataconn_name' as any)
    .should('eq', dataconnection);
  // validate that the ui component showed up
  // cy.get("ul[style='display: block;'] div[class='selected-control-wrapper']").should('exist');

  //     .should('contains.oneOf',  get_current_timestamps());
}

export function dismissWhatNewModal() {
  const gotitButton = CSSselector.welcome_screen.gotit_button;
  cy.get('body').then(($body) => {
    const releaseModalDisplayed = $body.find(gotitButton).length > 0;
    if (releaseModalDisplayed) {
      cy.get(gotitButton).click();
      cy.log('Release modal dismissed via GotIt clicked');
      cy.get(gotitButton).should('not.exist');
    } else {
      cy.log('What new in release modal is hidden');
    }
  });
}

export function getVersion() {
  return cy
    .get("[data-test-id='whats-new-in-version']")
    .invoke('text')
    .then((text) => {
      return text.match(new RegExp(/\d.\d.\d/))?.[0][0];
    });
}

export const homepageRequests = [
  // { method: 'GET', url: '/arc/reports/appgroup', name: 'isLoginSuccess' },
  { method: 'GET', url: '/arc/apps/homepage_statistics', name: 'homepageStatistics' },
  {
    method: 'GET',
    url: '/arc/apps/homepage_number_of_recently_created_resources',
    name: 'homepageNumberOfRecentlyCreatedResources',
  },
  {
    method: 'GET',
    url: '/arc/apps/homepage_recently_created_dashboards_by_user',
    name: 'homepageRecentlyCreatedDashboardsByUser',
  },
  {
    method: 'GET',
    url: '/arc/apps/homepage_last_viewed_reports_by_user',
    name: 'homepageLastViewedReportsByUser',
  },
  {
    method: 'GET',
    url: '/arc/apps/homepage_most_popular_reports',
    name: 'homepageMostPopularReports',
  },
  {
    method: 'GET',
    url: '/arc/apps/homepage_samples_dashboards',
    name: 'homepageSamplesDashboards',
  },
  {
    method: 'GET',
    url: '/arc/apps/homepage_favorite_dashboards',
    name: 'homepageFavoriteDashboards',
  },
  { method: 'GET', url: '/arc/apps/homepage_datasets?limit=30', name: 'homepageDatasets' },
];

/**
 * equvivalent of clicking on the home tab from any screen
 * accounts for the loading delays
 * @export
 */
export function navigate_to_home() {
  cy.url().then((url) => {
    const homeUrl = /\/arc\/apps\/home$/;
    if (homeUrl.test(url)) {
      // we are already on the home page
      cy.log('already on home page');
      dismissWhatNewModal();
    } else {
      cy.log('navigating to home page');
      do_navigate_to_home(true);
    }
  });
}

export function do_navigate_to_home(useDirectUrl = false) {
  if (useDirectUrl) {
    cy.visit('/arc/apps/home');
  } else {
    // click the navbar:
    cy.get(CSSselector.homepage).click();
  }

  // wait for the new dashboard icon to appear:
  cy.get(CSSselector.home_new_dashboard, { timeout: 30000 });

  dismissWhatNewModal();
}

/**
 * equvivalent of clicking on the data tab from any screen
 * accounts for the loading delays
 * @export
 */
export function navigate_to_data() {
  cy.visit('/arc/apps/data');

  cy.get('a[title="All Connections"]', { timeout: 30000 }).should('be.visible');
  cy.get('[data-test-id="loading-spinner"]').should('not.be.visible');
}

/**
 *
 * navigates to /arc/apps/apps
 * @export
 */
export function navigate_to_visuals() {
  cy.visit('/arc/apps/apps');

  // click all
  cy.get(CSSselector.Visuals_all)
    .should('be.visible')
    .contains('ALL')
    .click();

  cy.get(CSSselector.Visuals_all).should('have.attr', 'data-test-active', 'true');
}
/**
 * click email visual from dashboard edit page
 *
 * @export
 */
export function click_email() {
  cy.intercept('POST', '/arc/apps/apps/*');
  cy.get('.subnav-report')
    .find('#report-menu-list')
    .find('.dropdown')
    .find('.fa.fa-ellipsis-h')
    .click();
  cy.get('.subnav-report')
    .find('#report-menu-list')
    .find('.dropdown-menu')
    .find('.email-visual')
    .click();
}

export function searchForUser(user: string) {
  cy.get('#users-table_filter input')
    .clear()
    .type(user);
  cy.wait(1000);
  cy.get('.dataTables_processing').should('not.be.visible');
}

/**
 * delete an existing user to arcviz by clicking the top nav menu dropdow (settings icon)
 * @export
 * @param {string} username
 */
export function deleteUser(username: string) {
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
  searchForUser(username);
  // check for existing user DELETE any user with similar name
  cy.get(`${CSSselector.users.users_table_row}`).each(($row) => {
    // synchronously ask for the body's text
    // and do something based on whether it includes
    // another string
    console.log(`row data is ${$row.text()}expected is ${username}`);
    if ($row.text().includes(username)) {
      cy.wrap($row)
        .find('span')
        .filter(CSSselector.users.delete_bin)
        .as('binIcon');

      cy.get('@binIcon').click();

      cy.get(locators.users.delete).click();
      cy.wait(1000);
      cy.get(CSSselector.users.confirm_delete_button).click();
    }
  });
}

export function deleteGroup(groupName: string) {
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
  cy.get(`${CSSselector.groups.groups_table_row}`).each(($row) => {
    console.log(`row data is ${$row.text()}expected is ${groupName}`);
    if ($row.text().includes(groupName)) {
      cy.wrap($row)
        .find('span')
        .filter(CSSselector.groups.delete_bin)
        .click({ force: true });
      cy.wait(1000);
      cy.get(CSSselector.groups.confirm_delete_button).click({ force: true });
    }
  });
}

/**
 * adds a new user to arcviz by clicking the top nav menu dropdow (settings icon)
 * if a user with the same name exists it first deletes the older user and creates a new one
 * @export
 * @param {string} username
 */
export function addUser(username: string) {
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
  // before you create a user check for existing user DELETE any user with similar name
  cy.get(`${CSSselector.users.users_table_row}`).each(($row) => {
    // synchronously ask for the body's text
    // and do something based on whether it includes
    // another string
    console.log(`row data is ${$row.text()}expected is ${username}`);
    if ($row.text().includes(username)) {
      cy.wrap($row)
        .find('span')
        .filter(CSSselector.users.delete_bin)
        .click();
      cy.get(CSSselector.users.confirm_delete_button).click();
    }
  });
  // OK now continue creating user
  cy.get(CSSselector.users.new_user_button).click();
  cy.get(CSSselector.users.username_textbox).click();
  cy.get(CSSselector.users.username_textbox).type(username);
  cy.get(CSSselector.users.password_textbox).type(username);
  cy.get(CSSselector.users.save_user_button).click();
}

/**
 * open site settings from the top nav menu dropdown (settings icon)
 *
 * @export
 */
export function goto_arcviz_settings() {
  cy.visit('/arc/apps/settings');
}

/**
 * open job status(admin logs) from the top nav menu dropdown (settings icon)
 *
 * @export
 */
export function goto_job_status() {
  cy.visit('/arc/jobs/adminlogs');
}

export function goToJobSchedulePage() {
  goto_job_status();
  cy.get(jobs.job_overview.jobs_tab).click();
}

export function goToDataPage() {
  cy.visit('/arc/apps/data');
}

export function checkSuccessfulEmailCreationNotification() {
  const message = 'Schedule successfully created';
  cy.contains(message, { timeout: 30000 } as any);
  cy.get(notifications.notification_message, { timeout: 30000 }).should(($message) => {
    expect($message).to.contain(message);
    expect($message).to.have.text(message);
  });
}

export function typeInSearchField(text: string) {
  /**
   * Something is wrong with typing into the visuals search field.
   * Until we find out what, try delaying the typing action, maybe it
   * is a delayed action that is the cause.
   */
  cy.wait(1234); // this will show up in the logs

  // try a longer typing delay (default is 10ms)
  cy.get(CSSselector.Search_Dashboard_Visuals_Page)
    .clear()
    .type(text);

  cy.wait(123);

  cy.get(CSSselector.Search_Dashboard_Visuals_Page).should('have.value', text);
}
/**
 * finds the ID of a specific app from the visuals page left menu
 *
 * @export
 * @param {string} appName
 * @returns {string} the promise which n resolving will give id of the app name
 */
export function getBuilderUrlFromAppName(appName: string) {
  navigate_to_visuals();

  cy.get(CSSselector.allTypeButton).click();

  typeInSearchField(`"${appName}"`);

  return cy
    .get(`[data-test-id="${appName}"] [data-test-id="edit-report-button"]`)
    .invoke('attr', 'href');
}

export function getIDNumberFromAppName(appName: string) {
  return getBuilderUrlFromAppName(appName).then((appUrl) => {
    return parseInt(appUrl!.split('/arc/apps/builder/')[1], 10);
  });
}

/**
 * Search for a unique dashboardName in the visuals tab
 * assert only one result was returned
 * @export
 * @param {string} dashboardname
 */
export function searchDashboard(dashboardname: string) {
  navigate_to_visuals();
  cy.get(CSSselector.listview_Visualspage).click();

  typeInSearchField(dashboardname);
  cy.get(`[data-test-id="${dashboardname}"]`).should('have.length', 1);
  // add assertion that name searched is the name in all displayed results - above one is limiting to one search result
}

/**
 * Add a single filter to a dashboard that is in edit dashboard screen
 *
 * @export
 * @param {string} filterName
 */
export function addFilter(filterName: string, dataconnection: string, dataset: string) {
  cy.get(`${CSSselector.filters_tab}`).click();

  cy.get('#dash-filters-tab > form > div:nth-child(1) > div > div')
    .scrollIntoView()
    .click()
    .find('input')
    .type(dataconnection);
  cy.get('#dash-filters-tab > form > div:nth-child(1) > div > div')
    .find('.chosen-results')
    .contains(dataconnection)
    .click({ force: true });
  cy.get('#dash-filters-tab > form > div:nth-child(2) > div > div')
    .scrollIntoView()
    .click()
    .find('input')
    .type(dataset);
  cy.get('#dash-filters-tab > form > div:nth-child(2) > div > div')
    .find('.chosen-results')
    .contains(dataset)
    .click({ force: true });

  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  cy.intercept('GET', '/arc/reports/report/*').as('visualGet');
  cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
  // cy.get(`${arcvizSelectors.filterTab.filter}`).click();
  cy.get('#dash-filters-tab > ul.app-params.dim-app-params > div > li .selection_field')
    .contains('cereal_name')
    .click();
  cy.wait('@visualCreated');
  cy.wait('@visualGet');
  cy.wait('@visualUpdated');

  // cy.get('.control-list.ui-sortable div.selected-control-title > span').trigger('mouseover');
  // cy.get(
  //   `${arcvizSelectors.selectedControl.icon} ${arcvizSelectors.selectedControl.settingIcon}`
  // ).click({force: true});
  cy.get('li a[data-tooltip-contents="Configure filter"]').trigger('focusin', { force: true });

  cy.get('li a[data-tooltip-contents="Configure filter"]').click({ force: true });

  waitForVisualsLoaded();

  cy.get(`${CSSselector.selectedControl.filterName}`)
    .clear()
    .type(filterName);
  cy.get(`${CSSselector.selectedControl.save}`).click();
  cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
  cy.get(`${CSSselector.Save}`).click();
  cy.wait('@savedashboard')
    .its('response.statusCode')
    .should('eq', 200);
  cy.get(`${CSSselector.Save}`).should('not.have.class', 'app-btn-highlight');
}

export function getSelectedUrl(): any {
  return new Promise((resolve) => {
    cy.get('#link-modal-link')
      .invoke('val', '')
      .then(($url) => {
        console.log('test selected AppId : ' + $url);
        resolve($url);
      });
  });
}

export function checkArcenginePresence(testContext: Mocha.Context) {
  checkFeatureAvailable(testContext, 'ARCENGINE_COLO');
}

export function checkFeatureAvailable(testContext: Mocha.Context, featureName: string) {
  if (!Cypress.env(featureName)) {
    cy.log(`feature unavailable: "${featureName}"`);
    testContext.skip();
  }
}

export function skipIfCDW(testContext: Mocha.Context) {
  isCDW().then((value) => {
    if (value) {
      cy.log('Skipping in CDW mode');
      testContext.skip();
    }
  });
}

export function isCDW(): Cypress.Chainable {
  return cy
    .window()
    .its('arc.frontend.globals.settings.cdvMode')
    .then((cdvMode) => cdvMode === 'cdw');
}

export function waitForVisualsLoaded(timeInMilliseconds: number = 30_000) {
  cy.wait(600);
  cy.window()
    .its('arc.misc')
    .invoke({ timeout: timeInMilliseconds }, 'allVisualsLoaded')
    .should('eq', true);
  cy.wait(600);
}

export function waitForAceEditorReady(aceEditor: JQuery<HTMLElement>) {
  cy.window().then((win: any) => {
    console.log('cyhelper check');
    if (!win.cyhelper) {
      const aceEditorReady = (editor: HTMLElement) => {
        console.log('aceEditorCheck: ', win.ace.edit(editor).session.getMode());
        return typeof win.ace.edit(editor).session.getMode() !== 'string';
      };

      win.cyhelper = {
        aceEditorReady,
      };
      console.log('cyhelper created');
    }
  });

  cy.window()
    .its('cyhelper')
    .invoke({ timeout: 30000 }, 'aceEditorReady', aceEditor[0])
    .should('eq', true);

  return cy.wrap(aceEditor);
}

export function modalClick(selector: string) {
  // in CDV 7 the bootstrap modals register the click handlers later than in version 6
  // we must wait a litle otherwise the modal will perform its task but will stay open
  cy.wait(1000);
  cy.get(selector).click();
}

export function loadingSpinnerCheck(selector: string) {
  cy.get(selector).should('be.visible');
  cy.get(selector).should('not.be.visible');
}

export function goToSiteAdminSettingsMenu(
  menuName: keyof CSSselectorType['site_admin_menus'],
  isForced = false
) {
  cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle)
    .click({ force: isForced })
    .next()
    .find(CSSselector.site_admin_menus[menuName])
    .click({ force: isForced });
}

export function deleteObjectApi(
  objectType: string,
  id: number,
  apiKey: string
): Cypress.Chainable<number> {
  return cy
    .request({
      method: 'DELETE',
      url: `/arc/adminapi/v1/${objectType}/${id}`,
      headers: {
        Authorization: 'apikey ' + apiKey,
      },
    })
    .then((resp) => {
      return resp.status;
    });
}

export function createorUpdateObjectApi(
  objectType: string,
  apiKey: string,
  objectData: string,
  updateIdParam: string = ''
): Cypress.Chainable<any> {
  return cy.request({
    method: 'POST',
    url: `/arc/adminapi/v1/${objectType}${updateIdParam}`,
    headers: {
      Authorization: 'apikey ' + apiKey,
      'Content-Type': 'application/x-www-form-urlencoded',
    },
    body: objectData,
  });
}

export function getObjectDetailsApi(
  objectType: string,
  apiKey: string = ''
): Cypress.Chainable<LookupObject[]> {
  return cy
    .request({
      method: 'GET',
      url: `/arc/adminapi/v1/${objectType}` + '?detail=1',
      headers: {
        Authorization: 'apikey ' + apiKey,
      },
    })
    .then((resp) => {
      return resp.body;
    });
}

export function bulkObjectDeleteApi(
  objectType: string,
  objectIds: number[],
  objectNames?: string[],
  nameProp?: string
): void {
  createApiKey().then((apiKey) => {
    if (objectNames && nameProp) {
      getObjectIdsApi(objectType, nameProp, objectNames, apiKey).then((objectIdsToDelete) => {
        _multipleObjectDeleteApi(objectType, apiKey, objectIdsToDelete);
      });
    }
    _multipleObjectDeleteApi(objectType, apiKey, objectIds);
  });
}

function _multipleObjectDeleteApi(
  objectType: string,
  apiKey: string,
  objectIdsToDelete: number[]
): void {
  for (const objectId of objectIdsToDelete) {
    deleteObjectApi(objectType, objectId, apiKey).then((status) => {
      expect(status).to.eq(200);
    });
  }
}

export function getObjectIdsApi(
  objectType: string,
  nameProp: string,
  objectNames: string[],
  apiKey: string
): Cypress.Chainable<number[]> {
  return getObjectDetailsApi(objectType, apiKey).then((objects) => {
    return objects
      .map((object: LookupObject) => {
        if (objectNames.includes(object[nameProp])) {
          return object.id as number;
        }
      })
      .filter((id): id is number => Boolean(id))
      .flat();
  });
}
