import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as nl from 'lib/search';
import { locators as l } from './locators';
import { datasetName } from 'lib/dashboard/default_dashboard';

interface QuestionBreakdown {
  dataset: string;
  dimensions_shelf: string;
  aggregate_shelf: string;
  filter_shelf: string;
}

export enum SearchMenuTabs {
  RESULTS = 'Results',
  BOOKMARKS = 'Bookmarks',
  HISTORY = 'History',
}

export interface WordExpansion {
  inputWord: string;
  outputWord: string;
}

export function openSearchModal() {
  cy.get(l.searchButton).click();
  cy.get(l.searchModal).should('be.visible');
  cy.get(l.searchModal)
    .find(l.searchTextbox)
    .should('be.visible');
}

export function openGlobalSearchModal() {
  cy.log('opening search modal from global scope');
  functions.navigate_to_home();
  cy.intercept('GET', '/arc/searches/search').as('getSearch');
  cy.get('span.topmenu-search-link:not(.natural-language-disabled)').click();
  cy.wait('@getSearch');
}

/**
 * @param {string} searchPhrase
 *
 */
export function searchGlobally(searchPhrase: string) {
  cy.intercept('POST', '/arc/searches/search').as('searchSent');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('sqlrun');
  cy.get('#test-search-modal .pop-search-box').type(searchPhrase + '{enter}');
  cy.wait('@searchSent');
  cy.wait('@sqlrun');

  //  assert for result
  cy.get('.arc-viz-container.open');
}

export function createDashboardFromSearch() {
  cy.intercept('POST', '/arc/reports/add_search_report').as('addDashboard');
  cy.log('Creating dashboard from first search result');
  cy.get('#test-search-modal .search-results .search-result').then(($searchresult) => {
    $searchresult.find('.fa.fa-plus-square.add-dashboard-button.hide-if-no-permissions').click();
    $searchresult.find('.action-icon.add-to-new-dashboard').click();
    cy.wait('@addDashboard');
    cy.get('[data-test-id="notification-message"]>a').then(($dashboardid) => {
      const dblink = String($dashboardid.attr('href'));
      cy.log('dblink ' + dblink);
      // edit the visual from the dashboard and validate the visual
      cy.log('Editing newly created dashboard ' + dblink);
      cy.visit(dblink);
    });
  });
  dashboards.editVisual();
}

export function assertFieldMultiple(fieldName: string, assertText: string) {
  const filterField = '#filters_shelf';
  const cssQuery = '#app-shelves ' + fieldName + ' .selection_field';
  let continueSearch = true;
  cy.get(cssQuery).each(($el, index, $list) => {
    let txt = '';
    if (fieldName.includes(filterField)) {
      txt = String($el.attr('data-original-title'));
      cy.log('field content:' + txt);
    } else {
      txt = String($el.attr('title'));
      cy.log('field content:' + txt);
    }
    if (txt === assertText) {
      cy.log(`got the expected field text ${txt}`);
      continueSearch = false;
    } else {
      if ($list.length === index + 1 && continueSearch) {
        cy.log(`matcing field text ${assertText} not found`);
        assert.fail(txt, assertText, 'text not matching');
      }
    }
  });
}

export function assertFieldSorting(fieldName: string, sorting: string) {
  const cssQuery = '#app-shelves ' + fieldName + ' i.fa.fa-sort-amount-' + sorting;
  cy.get(cssQuery).should('be.visible');
}

export function closeSearchModal() {
  cy.get(l.searchModal)
    .find(l.closeSearchModalButton)
    .click();
  cy.get(l.searchModal).should('not.be.visible');
}

export function verifySearchEnabledOnDataset(
  connectionName: string,
  dataset: string,
  searchQuery?: string
) {
  functions.navigate_to_data();
  cy.intercept('POST', '/arc/reports/create/*').as('reportCreated');
  cy.intercept('GET', '/arc/searches/search').as('searchLoaded');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect_parallel');

  cy.get(l.connectionList).each(($el, index, $list) => {
    if ($el.text() === connectionName) {
      cy.get(l.connectionList + ' a[title=' + connectionName + ']').click();
      cy.get(l.datasetTableRow)
        .contains(dataset)
        .parent()
        .parent()
        .next()
        .children()
        .should('have.class', l.searchLinkClass);
      cy.get(l.datasetTableRow)
        .contains(dataset)
        .parent()
        .parent()
        .next()
        .find('[data-title="New Dashboard"]')
        .click();
    }
  });
  cy.wait('@reportCreated');
  cy.wait('@searchLoaded');
  cy.wait('@jsonselect_parallel');

  cy.get(l.newVisualContainer)
    .prev()
    .should('have.class', l.searchPopoverClass);
  cy.get(l.newVisualContainer)
    .prev()
    .find(l.searchTextbox)
    .type(searchQuery + '{enter}');

  cy.wait('@jsonselect_parallel');

  // validate visual rendered with the search query(if needed)
}

export function verifySerchDisabledOnDataset(connectionName: string, dataset: string) {
  functions.navigate_to_data();
  cy.get(l.connectionList).each(($el, index, $list) => {
    if ($el.text() === connectionName) {
      cy.get(l.connectionList + ' a[title=' + connectionName + ']').click();
      {
        cy.get(l.datasetTableRow)
          .contains(dataset)
          .parent()
          .parent()
          .next()
          .children()
          .should('not.have.class', l.searchLinkClass);
      }
    }
  });
}

export function verifySearchGroup(searchGroup: string, dataset: string) {
  nl.openGlobalSearchModal();
  cy.get(l.searchModal)
    .find(l.searchDataSourceSelector)
    .click();
  cy.get(l.searchModal)
    .find(l.searchDataSourceDropdown)
    .should('contain', searchGroup);
  cy.get(l.searchModal)
    .find(l.searchDataSourceSelector)
    .click();
  closeSearchModal();
}

export function verifySuggestedQuestions(suggestedQuestions: string[]) {
  functions.navigate_to_home();
  openSearchModal();

  // click dropdown and verify suggested qes
  // should I verify type ahead or autocomplete things

  closeSearchModal();
}

export function verifyWordExpansions(expansions: any[]) {
  functions.navigate_to_home();
  openSearchModal();

  // click dropdown and verify suggested qes
  // should I verify type ahead or autocomplete things

  closeSearchModal();
}
