import { notifications } from 'fixtures/locators/notifications';
import * as functions from 'lib/functions';
import { siteSettingLocators } from './siteSettingLocators';

export type SiteSetting =
  | 'Enable Data Extracts'
  | 'Permit emails'
  | 'Enable "Download as Image/PDF" by default'; // and more...
/**
 * set the email configuration from the email settings and save it
 * by passing in an emailSettings json like below
 * "template": {
 *   "name": "Sample template made from automation",
 *   "description": "Email template description",
 *   "email_body_heading": "<u>Heading</u> TEXT",
 *   "email_body_message": "some body",
 *   "email_footer_text": "some neat footer text here",
 *   "email_wrapper_footer": "some footer"
 * }
 * @export
 * @param {*} emailSettings
 */
export function set_email_settings(emailSettings: any) {
  functions.goto_arcviz_settings();
  cy.intercept('POST', '/arc/jobs/testemail').as('emailSent');
  cy.intercept('POST', '/arc/reports/setting').as('settingsSaved');

  cy.fixture('locators/settings/site_settings.json').then((siteSettingsSelectors) => {
    cy.get(siteSettingsSelectors.enable_snapshot)
      .scrollIntoView()
      .check();
    cy.get(`[data-test-id="test-email_enabled"]`).check();
    const tlsCheckbox = cy.get(siteSettingsSelectors.use_tls).scrollIntoView();
    emailSettings.use_tls === 'true' ? tlsCheckbox.check() : tlsCheckbox.uncheck();
    const sslCheckbox = cy.get(siteSettingsSelectors.use_tls).scrollIntoView();
    emailSettings.use_ssl === 'true' ? sslCheckbox.check() : sslCheckbox.uncheck();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_address)
      .clear();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_address)
      .type(emailSettings.address);
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_username)
      .clear();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_user_password)
      .clear();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_user_password)
      .type(emailSettings.password);
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_host)
      .clear();

    let email_host = emailSettings.email_host;
    if (email_host === 'testEmailHost') {
      email_host = Cypress.env('testEmailHost');
    }
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_host)
      .type(email_host);
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_port)
      .clear();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.email_port)
      .type(emailSettings.email_port);
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.default_email_subject)
      .clear();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.default_email_subject)
      .type(emailSettings.default_subject);
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.enable_email)
      .scrollIntoView()
      .check();
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(siteSettingsSelectors.send_test_mail)
      .click();
    cy.wait('@emailSent');
    cy.get(siteSettingsSelectors.email_settings_tab)
      .find(notifications.alert_success)
      .should('be.visible');
    cy.get(siteSettingsSelectors.save).click();
    cy.get(siteSettingsSelectors.confirm_save).click();
    cy.wait('@settingsSaved');
  });
}

export function setSearchSetting(enable: boolean) {
  functions.goto_arcviz_settings();

  siteSettingsSearch('Enable Search');

  cy.get('[data-test-id="test-site_enable_search"]').then(($input) => {
    if (enable && !$input.is(':checked')) {
      cy.wrap($input).check();
      saveSiteSettings();
      return;
    }

    if (!enable && $input.is(':checked')) {
      cy.wrap($input).uncheck();
      saveSiteSettings();
      return;
    }
  });
}

export function siteSettingsSearch(searchTerm: string) {
  cy.get('[data-test-id="settingsSearch"]')
    .click()
    .clear()
    .type(searchTerm);
}

export function ensureCheckboxState(siteSettingName: SiteSetting, enable: boolean) {
  siteSettingsSearch(siteSettingName);

  const checkbox = cy.get(`label:contains(${siteSettingName})`).siblings('input');
  checkbox.should('have.length', 1);

  return checkbox.then(($checkbox) => {
    let needsSave = false;
    const isChecked = $checkbox.is(':checked');
    if (enable && !isChecked) {
      checkbox.check();
      needsSave = true;
    } else if (!enable && isChecked) {
      checkbox.uncheck();
      needsSave = true;
    }
    return cy.wrap(needsSave);
  });
}

/**
 * One-off utility to ensure that a certain site settings checkbox has the desired value.
 * Settings are automatically saved if modified.
 * @param siteSettingName
 * @param enable
 */
export function setSiteSettingCheckbox(siteSettingName: SiteSetting, enable: boolean) {
  functions.goto_arcviz_settings();

  ensureCheckboxState(siteSettingName, enable).then((needsSave) => {
    if (needsSave) {
      saveSiteSettings();
    }
  });
}

export function saveSiteSettings() {
  // the undo button shows that there is a a real change:
  cy.get('[data-test-id="site_settings_undo_btn"]');

  cy.get(siteSettingLocators.saveButtonOfSiteSettingPage).click();
  cy.get(siteSettingLocators.saveButtonfOfSiteSettingChangeModal).click();

  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.alert_success).should('not.exist');
}
