import * as arctest from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as connections from 'lib/connections';
import { TEST_CONNECTIONS } from 'lib/connections/connections';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import { silent } from 'lib/logins';
import * as dsPermissions from 'lib/permissions';
import * as roles_perms from 'lib/roles&permissions';
import { locators } from 'lib/roles&permissions/locators';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * Open the specified role from 'Manage Role' setting
 * @param roleName: Name of a role to be opened
 */
function openRole(roleName: string) {
  cy.visit('/arc/apps/roles');
  cy.intercept('GET', '/arc/roles/roles?page=1&page_size=10&sort=name&order=asc&search=').as(
    'roleLoaded'
  );
  cy.wait(2000);
  cy.get(`${CSSselector.roles.role_row}`).each(($row) => {
    if ($row.text().includes(roleName)) {
      cy.wrap($row)
        .find('td:last-child')
        .find('button[data-test-id="editRole"]')
        .click();
    }
  });
  cy.wait('@roleLoaded')
    .its('response.statusCode')
    .should('eq', 200);
}

function removeDefaultGroupsAndRole(username: string, password: string) {
  // first log in with the new user
  // which will get the viz_quest_groups
  // and Database_admin role
  silent(username, password);
  BackendAPI.users.getByName(username).then((user) => {
    BackendAPI.groups.getByName('viz_guest_group').then((vizGuestGroup) => {
      if (vizGuestGroup) {
        BackendAPI.groups.update({
          id: vizGuestGroup.id,
          userIds: vizGuestGroup.users.map(({ id }) => id).filter((id) => id !== user.id),
        });
      }
    });

    BackendAPI.roles.getByName('Database admin').then((databaseAdminRole) => {
      BackendAPI.roles.update({
        id: databaseAdminRole.id,
        users: databaseAdminRole.users.filter((name) => name !== user.username),
      });
    });
  });
}

function selectDataset(connectionName: string, datasetName: string) {
  dsPermissions.selectDataset(connectionName, datasetName);
  cy.get('#app-dataset-name', { timeout: 30000 }).should('have.text', datasetName);
}

function saveRole() {
  cy.intercept('POST', '/arc/roles/roles').as('roleSaved');
  cy.get(CSSselector.roles.role_details_page.save_role_button).click();
  cy.wait('@roleSaved')
    .its('response.statusCode')
    .should('eq', 200);
}

const creator = {
  username: 'rbac_creator',
  password: 'rbac_creator',
};

const tester = {
  username: 'rbac_tester',
  password: 'rbac_tester',
};

const analyst = {
  username: 'rbac_analyst',
  password: 'rbac_analyst',
};

const visualconsumer = {
  username: 'rbac_visualconsumer',
  password: 'rbac_visualconsumer',
};

describe('RBAC: Role Based Access Control.', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const connectionList = ['All connections'];
  const datasetsPerConnection: any = {};
  const userIds: number[] = [];

  function createUser(user: { username: string; password: string }, roleIds?: number[]) {
    // if the user exists, we delete it first:
    return BackendAPI.users
      .createOrRecreate(user, roleIds)
      .then((createdUser) => userIds.push(createdUser.id));
  }

  /**
   * Must clean-up because tests like users_groups_managamenet fail if there are more than 10 users.
   */
  after(() => {
    userIds.forEach((id) => {
      BackendAPI.users.delete(id);
    });
  });

  context('ARC-11145: Automate RBAC - Smoke', () => {
    before(() => {
      arctest.logins.silent();
      functions.navigate_to_data();
      // Prepare a list of available connections
      cy.get(CSSselector.data.connection_list).each(($el, i) => {
        connectionList.push($el.text());
        cy.wrap($el)
          .click()
          .then(() => {
            // Prepare an object which contains total number of datasets created in all the available connections
            cy.get(CSSselector.data.dataset_table_row).then(($dataset) => {
              datasetsPerConnection[connectionList[i + 1]] = $dataset.length;
            });
          });
      });

      connections.createConnection(TEST_CONNECTIONS.test_maria_10);
      connections.createConnection(TEST_CONNECTIONS.test_postgres);
    });

    it('1 - Create user and role with description', () => {
      cy.fixture('dataset_publishing.json').then((data) => {
        createUser(creator);
        createUser(tester);

        // Add the 'creator-role' role to the user
        roles_perms.addRole(data.roleData[1]);
        // Add the samples - World Life Expectancy - ds_manage privilege to the user
        roles_perms.addPrivilege(data.privilege[0]);

        // Verify the created role with it's description and components
        cy.get(CSSselector.roles.role_details_page.name_field)
          .invoke('val')
          .should('eq', data.roleData[1].roleName);
        cy.get(CSSselector.roles.role_details_page.description)
          .invoke('val')
          .should('eq', data.roleData[1].roleDescription);
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .datasets_permissions.header
        )
          .eq(0)
          .invoke('text')
          .then((text) => {
            const normalizedText = text.replace(/\s+/g, ' ').trim();
            expect(normalizedText).to.match(/samples\s*\/\s*World Life Expectancy/);
          });

        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .datasets_permissions.selectAll
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .datasets_permissions.ds_manage
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .datasets_permissions.ds_appedit
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .datasets_permissions.ds_appview
        ).should('be.checked');
      });

      saveRole();
    });

    it('2 - Verify the given permission of newly created user', () => {
      cy.fixture('dataset_publishing.json').then((data) => {
        removeDefaultGroupsAndRole(creator.username, creator.password);

        functions.navigate_to_data();

        cy.get(CSSselector.data.connection_list).each(($el) => {
          if ($el.text() === connectionName) {
            $el.click();
          }
        });
        cy.get(CSSselector.data.dataset_table_row).as('dataset');
        // rbac_creator has permission to the samples - World Life Expectancy
        // so should be only 1 dataset in Data page
        cy.get('@dataset').should('have.length', 1);
        cy.get('@dataset')
          .find('span.dataset-title')
          .should('contain', data.privilege[0].datasets[0].trim());
        // count how many visuals the user able to see
        // in that case 3 visuals + 1 app
        cy.get('@dataset')
          .find('> td:nth-child(7) > span')
          .then(($dash) => {
            functions.navigate_to_visuals();
            cy.get(CSSselector.allTypeButton).click();
            cy.get(imp_exp.locators.grid_item).should('have.length', 4);
          });

        functions.navigate_to_data();

        const datasetName = data.privilege[0].datasets[0];
        selectDataset(connectionName, datasetName);

        // creator user can edit:
        cy.get('.tab-content i.fa.fa-pencil').should('be.visible');
      });
    });

    it('3 - Verify connections in Connections(s) list for component type Connection and Dataset', () => {
      silent();
      cy.visit('/arc/apps/users');
      cy.fixture('dataset_publishing.json').then((data) => {
        // open the creator-role
        openRole(data.roleData[1].roleName);
        cy.wait(2000);
        // add "test_postgresql", "test_maria_10" full access to the role
        roles_perms.addPrivilege(data.privilege[1]);

        // Verify the newly added connections;
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .connection_permissions.header
        )
          .eq(0)
          .invoke('text')
          .then((text) => {
            const normalizedText = text.replace(/\s+/g, ' ').trim();
            expect(normalizedText).to.match(/test_postgresql\s*\/\s*test_maria_10/);
          });
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .connection_permissions.selectAll
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .connection_permissions.dc_aviews
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .connection_permissions.dc_upload
        ).should('be.checked');
        cy.get(
          CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
            .connection_permissions.dc_explore
        ).should('be.checked');

        saveRole();
      });
    });

    it('4 - Add user to members tab and verify the given permission', () => {
      // Add User
      roles_perms.addUser([tester.username]);
      cy.get(locators.members.users_list).should('contain', tester.username);

      removeDefaultGroupsAndRole(tester.username, tester.password);

      cy.fixture('dataset_publishing.json').then((data) => {
        functions.navigate_to_data();
        cy.get(CSSselector.data.connection_list).each(($el) => {
          if ($el.text() === connectionName) {
            // yup found it
            $el.click();
          }
        });
        cy.get(CSSselector.data.dataset_table_row).as('dataset');
        cy.get('@dataset').should('have.length', 1);
        cy.get('@dataset')
          .find('span.dataset-title')
          .should('have.text', data.privilege[0].datasets[0]);
        cy.get('@dataset')
          .find('> td:nth-child(7) > span')
          .then(($dash) => {
            functions.navigate_to_visuals();
            cy.get(CSSselector.allTypeButton).click();
            cy.get(imp_exp.locators.grid_item).should('have.length', 4);
          });
        // Verify that dataset details should be editable
        functions.navigate_to_data();
        const datasetName = data.privilege[0].datasets[0];
        selectDataset(connectionName, datasetName);
        cy.get('.tab-content i.fa.fa-pencil').should('be.visible');
      });
    });

    it('5 - Remove user from members tab and verify the revocked permission', () => {
      cy.fixture('dataset_publishing.json').then((data) => {
        silent();
        cy.visit('/arc/apps/users');
        // open 'creator-role'
        openRole(data.roleData[1].roleName);
        // Remove 'rbac.tester' user from the role
        roles_perms.removeUser([tester.username]);
        cy.get(CSSselector.roles.role_details_page.members.users_table).should(
          'not.contain',
          tester.username
        );

        silent(tester.username, tester.password);
        functions.navigate_to_visuals();
        cy.get('button[data-test-id="create-new-button"]').should('be.disabled');

        functions.navigate_to_data();

        cy.get(CSSselector.data.newconnection_button).should('not.be.visible');
        cy.get(CSSselector.data.connection_list).should('have.length', 1);

        // tester user now has no access to any dataset:
        cy.get('#datasets .dataTables_empty').should('have.text', 'No data');
      });
    });

    it('6 - Verify the permissions of analyst and visual-consumer users', () => {
      cy.fixture('dataset_publishing.json').then((data) => {
        createUser(analyst, [DefaultRoleIds.Analyst]);

        removeDefaultGroupsAndRole(analyst.username, analyst.password);

        cy.visit('/arc/apps/apps');
        functions.navigate_to_data();

        cy.get(CSSselector.data.new_dataset_button).should('not.be.visible');
        cy.get(CSSselector.data.add_data_button).should('not.be.visible');
        cy.get(CSSselector.data.newconnection_button).should('not.be.visible');
        cy.get(`${CSSselector.data.connection_list} a[title=samples]`).click();
        cy.get('[data-test-id="import-artifacts"]').should('be.visible');

        const datasetName = data.privilege[0].datasets[0];
        selectDataset(connectionName, datasetName);

        cy.get('.tab-content i.fa.fa-pencil').should('not.be.visible');

        createUser(visualconsumer, [DefaultRoleIds.VisualConsumer]);
        removeDefaultGroupsAndRole(visualconsumer.username, visualconsumer.password);

        cy.visit('/arc/apps/apps');
        functions.navigate_to_data();

        cy.get(CSSselector.data.new_dataset_button).should('not.be.visible');
        cy.get(CSSselector.data.add_data_button).should('not.be.visible');
        cy.get(CSSselector.data.newconnection_button).should('not.be.visible');
        cy.get(`${CSSselector.data.connection_list} a[title=samples]`).click();
        cy.get('[data-test-id="import-artifacts"]').should('be.visible');

        selectDataset(connectionName, datasetName);

        cy.get('.tab-content i.fa.fa-pencil').should('not.be.visible');
      });
    });

    it('7 - Delete the role and verify it', () => {
      silent();
      cy.visit('/arc/apps/users');
      functions.goToSiteAdminSettingsMenu('manage_roles_site_admin');
      cy.fixture('dataset_publishing.json').then((data) => {
        cy.get(CSSselector.roles.role_row).should('contain', data.roleData[1].roleName);
        roles_perms.deleteRole(data.roleData[1].roleName);
        cy.get(CSSselector.roles.role_row).should('not.contain', data.roleData[1].roleName);
      });
    });
  });
});
