import { logins } from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import { locators as dashboardLocators } from 'lib/dashboard/locators';
import { getIDNumberFromAppName } from 'lib/functions';
import * as hp from 'lib/homepage';
import { locators as l } from 'lib/homepage/locators';
import { silent } from 'lib/logins';
import { CSSselector } from 'locators/common/CSSselector';

// https://www.cypress.io/blog/2020/02/12/working-with-iframes-in-cypress/
const getIframeBody = () => {
  return (
    cy
      .get('iframe.home-iframe')
      // Cypress yields jQuery element, which has the real
      // DOM element under property "0".
      // From the real DOM iframe element we can get
      // the "document" element, it is stored in "contentDocument" property
      // Cypress "its" command can access deep properties using dot notation
      // https://on.cypress.io/its
      .its('0.contentDocument.body')
      .then(cy.wrap)
  );
};

function verifyHomeDashboard(title: string) {
  cy.visit('/arc/apps');

  getIframeBody()
    .find(dashboardLocators.dashboard_title)
    .contains(title);
}

const visitSetHomePage = () => {
  cy.visit('/arc/apps/homepage');
  cy.get('.form-select__loading-indicator').should('not.exist');
};

describe('Custom homepage.', () => {
  context('set/reset homepage test', () => {
    let dashboardID1: number;
    let dashboardID2: number;
    let adminDefinedDashboardID: number;

    // the dashboards below are present in the Samples:
    const dashboardName1 = 'Inspector View';
    const dashboardName2 = 'Cereal Comparisons';
    const adminDefinedDashboardName = 'Global Threats';

    const nonadminUser = 'nonadmin';
    const nonadminPass = 'nonadmin';

    before(() => {
      BackendAPI.users.create({
        username: nonadminUser,
        password: nonadminPass,
        roleIds: [DefaultRoleIds.Analyst],
      });

      silent();

      getIDNumberFromAppName(dashboardName1).then((dbId) => {
        dashboardID1 = dbId;
      });

      getIDNumberFromAppName(dashboardName2).then((dbId) => {
        dashboardID2 = dbId;
      });

      getIDNumberFromAppName(adminDefinedDashboardName).then((dbId) => {
        adminDefinedDashboardID = dbId;
      });
    });

    beforeEach(() => {
      visitSetHomePage();
    });

    after(() => {
      visitSetHomePage();
      hp.resetHomepage();
    });

    it('let an admin user set homepage', () => {
      hp.verifySetHomepageDefaultScreen();

      hp.setHomepage(dashboardID1, dashboardName1);

      verifyHomeDashboard(dashboardName1);
    });

    it('let an admin user set default homepage for everyone', () => {
      cy.intercept('POST', '/arc/reports/homepage/defaultappid').as('setDefaultHomepage');
      cy.reactSelect('admin', `${adminDefinedDashboardName} (id: ${adminDefinedDashboardID})`);
      cy.get(l.adminOnlyDashboardSave).click();
      cy.wait('@setDefaultHomepage');

      // verify
      cy.get(l.adminOnlyDashboardName).contains(adminDefinedDashboardName);
      cy.get(l.adminOnlyDashboardName).contains(`(id: ${adminDefinedDashboardID})`);

      // verify it by resetting the admin's own default homepage
      hp.resetHomepage(adminDefinedDashboardID, adminDefinedDashboardName);

      verifyHomeDashboard(adminDefinedDashboardName);
    });

    it('let a non admin user override default homepage', () => {
      logins.silent(nonadminUser, nonadminPass);
      visitSetHomePage();

      hp.setHomepage(dashboardID2, dashboardName2);

      verifyHomeDashboard(dashboardName2);
    });

    it('let a non admin user reset to admin-defined homepage', () => {
      hp.resetHomepage(adminDefinedDashboardID, adminDefinedDashboardName);

      verifyHomeDashboard(adminDefinedDashboardName);
    });

    it('let an admin user reset default homepage', () => {
      logins.silent();
      visitSetHomePage();

      cy.intercept('DELETE', '/arc/reports/homepage/defaultappid').as('resetHomepage');
      cy.get(l.adminOnlyResetHomepage).click();
      cy.wait('@resetHomepage');
      cy.get(l.adminOnlyDashboardName).contains(hp.defaultHomePageName);

      hp.resetHomepage();
      hp.verifySetHomepageDefaultScreen();
    });

    it('let an admin user disable user-override of homepage', () => {
      // First set own dashboard to something.
      hp.setHomepage(dashboardID2, dashboardName2);

      // Then disable the option
      hp.toggleUserOverride(true);

      cy.get('iframe.home-iframe').should('not.exist');
    });

    it('let an admin user enable user-override of homepage', () => {
      hp.toggleUserOverride(false);

      verifyHomeDashboard(dashboardName2);
    });
  });
});
