import { logins } from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as connections from 'lib/connections';
import * as dashboard from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import { CSSselector } from 'locators/common/CSSselector';

function get_time() {
  const a = new Date();
  return a.getTime();
}
describe('Dashboard Bugs Verification.', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const databaseName = 'main';
  const tableName = 'cereals';
  const datasetName1 = 'Cereals';
  const datasetName2 = 'Test_Cereals_' + get_time();
  const dashName = 'db_' + get_time();
  let dashId: string;

  const db: Dashboard = {
    title: 'TopK' + get_time(),
    subTitle: 'TopK_subtitle' + get_time(),
    sheets: [
      {
        sheet_name: 'sheet1',
        sheet_number: 1,
        visuals: [
          {
            name: 'visual1',
            connection_name: connectionName,
            dataset_name: datasetName1,
            visual_type: '.chart-icon-table',
            visual_title: 'visual1',
            linked: 'false',
            data: {
              fields: [
                {
                  field_name: 'cereal_name',
                  field_shelf: ShelfTypes.DIMENSIONS,
                },
                {
                  field_name: 'manufacturer_code',
                  field_shelf: ShelfTypes.DIMENSIONS,
                },
                {
                  field_name: 'cold_or_hot',
                  field_shelf: ShelfTypes.DIMENSIONS,
                },
                {
                  field_name: 'Record Count',
                  field_shelf: ShelfTypes.AGGREGATES,
                  sort: SortOptions.DESCENDING,
                },
              ],
            },
          },
        ],
      },
    ],
  };

  before(() => {
    logins.silent();
  });

  const dashboardsTableRows =
    'div[data-test-id="delete-dataset-modal"] table[data-test-id="datasetDeleteModalDashboardsTable"] > tbody > tr';
  const dashboardIdColumn = `${dashboardsTableRows} td:nth-child(3)`;
  const relatedVisualsColumn = `${dashboardsTableRows} td:nth-child(4) span[data-test-id=applist]`;

  context('ARC-7846: Issues with change dataset for visual then delete dataset', () => {
    it('1 : See that visuals and their containing dashboards are correctly listed even after switch dataset', () => {
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName2);
      functions.addNewDB(dashName);
      functions.createNewVisual(connectionName, datasetName1);
      dashboard.saveDashboard();
      dashboard.editVisual();
      combo_functions.switchDataSet(datasetName2, connectionName);
      dashboard.saveDashboard();

      cy.url()
        .then(($id) => {
          dashId = $id.split('builder/')[1].split('#')[0];
        })
        .then(() => {
          cy.log(dashId);
          functions.navigate_to_data();
          connections.selectConnectionByName(connectionName);
          // Verify dashboard and visual list while deleting newly created dataset
          cy.get(CSSselector.data.dataset_table_row + ' .dataset-title').each(($datasetel) => {
            if ($datasetel.text() === datasetName2) {
              cy.wrap($datasetel)
                .parents('tr')
                .find('span.cell-wrapper a')
                .should('have.text', '1');
              cy.wrap($datasetel)
                .parents('tr')
                .invoke('attr', 'data-href')
                .then((dataHref) => {
                  const datasetId = dataHref?.split('/').pop();
                  cy.intercept(
                    'GET',
                    `/arc/datasets/possible_deleted_reports?dataset_id=${datasetId}`
                  ).as('getData');
                });

              cy.wrap($datasetel)
                .parents('tr')
                .find('span.fa-trash-o')
                .click();
              cy.get('div[data-test-id=delete-dataset-modal]')
                .parent()
                .should('be.visible');

              // have to wait for the server:
              cy.get(dashboardsTableRows, { timeout: 30000 }).should('have.length', 1);

              cy.get(dashboardIdColumn).should('have.text', dashId);
              const visualId = String(Number(dashId) - 1);
              cy.get(relatedVisualsColumn).should('have.text', visualId);

              cy.get('input[data-test-id=viz-delete-dataset-confirm]').type('DELETE');
              cy.get('button[data-test-id=delete-confirmation-yes]')
                .eq(1)
                .should('be.not.disabled')
                .click();
              cy.get(`${CSSselector.data.dataset_table_row}  .dataset-title`)
                .contains(datasetName2)
                .should('not.exist');
            }
          });
        });
    });

    it('2 : Verify that visuals and their containing dashboards are correctly listed even if not within workspace user has permission to access', () => {
      // Create user with Database admin user
      // as the System admin has the sys_misc
      // permission which allowing the user to
      // see the workspaces
      BackendAPI.users.create({
        username: 'user3',
        password: '123456',
        roleIds: [DefaultRoleIds.DatabaseAdmin],
      });

      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName2);
      functions.addNewDB(dashName);
      functions.createNewVisual(connectionName, datasetName1);
      dashboard.saveDashboard();
      dashboard.editVisual();
      combo_functions.switchDataSet(datasetName2, connectionName);
      dashboard.saveDashboard();
      cy.url()
        .then(($id) => {
          dashId = $id.split('builder/')[1].split('#')[0];
        })
        .then(() => {
          cy.log(dashId);
          logins.silent('user3', '123456');
          functions.navigate_to_data();
          connections.selectConnectionByName(connectionName);
          // Verify dashboard and visual list while deleting newly created dataset
          cy.get(CSSselector.data.dataset_table_row + ' .dataset-title').each(($datasetel) => {
            if ($datasetel.text() === datasetName2) {
              cy.wrap($datasetel)
                .parents('tr')
                .find('span.fa-trash-o')
                .click();
              cy.get('div[data-test-id=delete-dataset-modal]')
                .parent()
                .should('be.visible');

              // have to wait for the server:
              cy.get(dashboardsTableRows, { timeout: 30000 }).should('have.length', 1);

              cy.get(dashboardIdColumn).should('have.text', dashId);
              // related visuals info is not shown for users without view permissions
              cy.get(relatedVisualsColumn).should('have.text', '');

              cy.get('input[data-test-id=viz-delete-dataset-confirm]').type('DELETE');
              cy.get('button[data-test-id=delete-confirmation-yes]')
                .eq(1)
                .should('be.not.disabled')
                .click();

              // Check it is deleted
              cy.get(`${CSSselector.data.dataset_table_row}  .dataset-title`)
                .contains(datasetName2)
                .should('not.exist');
            }
          });
        });
    });
  });

  context('ARC-8530: Visual throwing JS console error when no limit is specified.', () => {
    it('1 : Create Dashboard with Record count in decending order and Top or Bottom K set.', () => {
      dashboard.createDashboard(db);
      dashboard.editVisual();

      const shelfDropdownLocator = '#aggregates_shelf .dropdown';
      cy.get(shelfDropdownLocator).each(($el) => {
        $el.click();
      });
      cy.get('#properties-panel ul.properties-menu.noselect li')
        .contains('Order and Top K')
        .click({
          force: true,
        });
      cy.get('#properties-panel .properties-menu.noselect li.has-submenu.order-functions.open')
        .find('.shelf-order-top')
        .find('input')
        .type('15')
        .parent()
        .click();
      cy.get(shelfDropdownLocator).each(($el) => {
        $el.click();
      });
      cy.get('#viz-refresh').click({ force: true });
      cy.get('div.dataTables_scrollBody > table > tbody > tr').should('have.length', 15);
      cy.get(dashboard.locators.limit).clear();
      dashboard.saveVisual();
      cy.get('div.dataTables_scrollBody > table > tbody > tr').should('have.length', 15);
      cy.get('#main-modal i.fa-exclamation-circle').should('be.empty');
    });
  });
});
