// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8160
import { jobs } from 'fixtures/locators/jobs';
import { notifications } from 'fixtures/locators/notifications';
import { logins } from 'lib/arctest';
import { BackendAPI, VIZAPPS_ADMIN_USER_ID } from 'lib/backend-api';
import * as connections from 'lib/connections';
import * as dashboard from 'lib/dashboard';
import { ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as ddfunctions from 'lib/deriveddata';
import * as functions from 'lib/functions';
import { ensureCheckboxState, saveSiteSettings, siteSettingsSearch } from 'lib/settings';
import { visualsPageFunctions } from 'lib/visuals-page/visuals-page-functions';
import { CSSselector } from 'locators/common/CSSselector';

let url: string = '';

// Get dashboard URL and split from baseURL and return rest of the URL
function dashboardUrl(): any {
  cy.get(CSSselector.dashboard.more_options).click();
  cy.get(dashboard.locators.get_url).click({ force: true });
  cy.get(dashboard.locators.dash_url_modal)
    .should('be.visible')
    .find('.ui-draggable-handle > h4')
    .should('have.text', 'Dashboard URL');
  cy.wait(3000);
}

/**
 * Check the export modal is displayed and export json file.
 * @param dashboardName : name of dashboard
 * @param dashId : id of a dashboard
 */
function exportDash(dashboardName: string, dashId: any) {
  cy.intercept('GET', 'arc/migration/export/*').as('export');

  cy.get(dashboard.locators.export_file_name)
    .clear({ force: true })
    .type(dashboardName + '_export');
  cy.get(dashboard.locators.export_button).should('have.text', 'Export');

  cy.request(
    `${
      Cypress.config().baseUrl
    }/arc/migration/export/?dry_run=false&dashboards=[${dashId}]&appgroups=[]&visuals=[]&filename=${dashboardName}`
  ).then((response) => {
    // these assertions log the whole body and response which can be large (500K)
    // making the assertions more targeted might reduce Cypress memory consumption
    // (and hopefully prevent crashes):
    // expect(response.body).to.have.length.above(0);
    // expect(response).to.have.property('headers');

    expect(response.body.length).to.be.above(0);
    expect(response.headers).to.exist;
    cy.writeFile(`${dashboardName}.json`, response.body);
  });
  cy.get(dashboard.locators.export_close_button).click();
}

// Generate random integer number
function getRandomInt(max: number) {
  return Math.floor(Math.random() * Math.floor(max));
}

describe('ARC-8160: ArcViz Automation - Multi Sheets Dashboard regression tests - task 2.', () => {
  const timeStamp = new Date().getTime();
  const ConnectionType1 = 'arcengine';
  const emailTo = 'SShah@phenixqa.com';
  const DBName = `Multisheet-Dashboard${timeStamp}`;
  const DBName1 = `Cabrides_Dashboard${timeStamp}`;
  let dashId = 'dashIDNotFound';

  before(() => {
    logins.silent();
  });

  afterEach(() => {
    BackendAPI.users.getById(VIZAPPS_ADMIN_USER_ID).then((user) =>
      BackendAPI.users.updateProfile('vizapps_admin', {
        user_data: {
          ...user.profile,
          homeapp: -1,
        },
      })
    );
  });

  it('ARC-8099 : Multi-sheet dashboard creation and cloning', () => {
    cy.fixture('visuals.json').then((dash) => {
      // verify that "Download as Image/PDF" is checked and Email is configured
      functions.navigate_to_visuals();
      functions.addNewDB(DBName);
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);
      functions.createNewVisual(dash.data[1].dataconnection, dash.data[1].dataset);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      dashboard.saveDashboard();

      cy.url().then((pageUrl) => {
        dashId = pageUrl.split('/arc/apps/builder/')[1];
        cy.log(`dashId=${dashId}`);
      });

      dashboard.addSheet();
      cy.get("a[data-target='#dash-viz-tab']").click();
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);
      functions.createNewVisual(dash.data[1].dataconnection, dash.data[1].dataset);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      dashboard.saveDashboard();

      dashboard.addSheet();
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);
      functions.createNewVisual(dash.data[1].dataconnection, dash.data[1].dataset);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      dashboard.saveDashboard();

      // verify dashboard URL
      cy.get("a[data-target='#dash-viz-tab']").click();
      dashboardUrl();
      cy.document().then((doc) => {
        url = (doc.getElementById('link-modal-link') as HTMLInputElement).value;
        cy.log('test selected AppId : ' + url);
        url = url.split(`${Cypress.config().baseUrl}` + '/arc/apps/app/')[1];
        expect(url).contain(dashId);
        expect(url).contain('sheet=3');
        cy.get(dashboard.locators.dash_url_modal_buttons)
          .contains('Close')
          .click();
      });
      cy.get(CSSselector.Sheet2).click();
      dashboardUrl();
      cy.document().then((doc) => {
        url = (doc.getElementById('link-modal-link') as HTMLInputElement).value;
        cy.log('test selected AppId : ' + url);
        url = url.split(`${Cypress.config().baseUrl}` + '/arc/apps/app/')[1];
        expect(url).contain(dashId);
        expect(url).contain('sheet=2');
        cy.get(dashboard.locators.dash_url_modal_buttons)
          .contains('Close')
          .click();
      });
      cy.get('.sheet-handle:nth-child(1)').click();
      dashboardUrl();
      cy.document().then((doc) => {
        url = (doc.getElementById('link-modal-link') as HTMLInputElement).value;
        cy.log('test selected AppId : ' + url);
        url = url.split(`${Cypress.config().baseUrl}` + '/arc/apps/app/')[1];
        expect(url).contain(dashId);
        expect(url).contain('sheet=1');
        cy.get(dashboard.locators.dash_url_modal_buttons)
          .contains('Close')
          .click();
      });

      dashboard.dbCloneMoveoptionSelection();
      cy.get(CSSselector.Dashboard_title).should('have.text', 'Clone of ' + DBName);
      cy.get(dashboard.locators.dashboard_sheets).should('have.length', 3);
      dashboard.addSheet();
      cy.get(dashboard.locators.dashboard_sheets).should('have.length', 4);
      cy.get(dashboard.locators.dashboard_sheets + ':nth-child(4)')
        .should('be.visible')
        .should('have.class', 'active')
        .should('contain', 'Sheet 4');

      cy.get("a[data-target='#dash-viz-tab']").click();
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);

      dashboard.moveSheet(4, 'left');
      dashboard.deleteSheet(3);
      cy.get(dashboard.locators.dashboard_sheets).should('have.length', 3);

      dashboard.addSheet();
      cy.get(dashboard.locators.dashboard_sheets).should('have.length', 4);
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);
      dashboard.saveDashboard();
    });
  });

  it('ARC-8101 : Verifying different options on dashboard ellipsis option', () => {
    cy.intercept('POST', '/arc/reports/myapps/*').as('addedAsFavorite');
    cy.intercept('DELETE', '/arc/reports/report/*').as('deleteDash');
    cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
    cy.intercept('POST', '/arc/reports/userdata/homeapp').as('setAsHomepage');
    cy.intercept('GET', '/arc/reports/appgroup').as('appgroupReportLoaded');
    cy.intercept('GET', '/arc/jobs/api/jobs/*/status').as('sendEmail');
    cy.intercept('POST', '/arc/apps/snapshot/*').as('snapshot');
    cy.intercept('GET', '/arc/apps/snapshot_ready/*').as('snapshotReady');
    // verify that "Download as Image/PDF" is checked and Email is configured
    functions.goto_arcviz_settings();

    const needsSave1 = ensureCheckboxState('Enable "Download as Image/PDF" by default', true);
    const needsSave2 = ensureCheckboxState('Permit emails', true);

    siteSettingsSearch('email');
    cy.get(dashboard.locators.test_email_btn).click();
    cy.get(notifications.alert_success).should('be.visible');
    cy.get(notifications.notification_content).should('contain.text', 'Email Status: 1 sent');
    // wait for message to disappear
    cy.get(notifications.alert_success).should('not.exist');

    needsSave1.then((b1) => {
      needsSave2.then((b2) => {
        if (b1 || b2) {
          saveSiteSettings();
        }
      });
    });

    // open dashboard in edit mode and verify add_to_favorites options on ellipsis
    functions.navigate_to_visuals();
    cy.get(CSSselector.Visuals_all).click({ force: true });
    cy.get(CSSselector.listview_Visualspage).click();

    functions.typeInSearchField(DBName);
    cy.get(`[data-test-id="${DBName}"]`).then(($el) => {
      dashId = $el.data('test-report-id');

      cy.get(CSSselector.gridview_Visualspage).click();

      dashboard.openDashboardforEdit(DBName);

      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();

      cy.get(dashboard.locators.add_to_favorites).click();

      cy.get(notifications.notification_content, { timeout: 30000 }).should(
        'contain',
        'Dashboard successfully added to Favorites.'
      );

      cy.wait('@addedAsFavorite')
        .its('response.statusCode')
        .should('be.eq', 200);

      functions.navigate_to_visuals();

      cy.get(CSSselector.apps.favourites_tab).click({ force: true });

      cy.contains(DBName);

      // open dashboard in edit mode and verify set as Homepage options on ellipsis
      dashboard.openDashboardforEdit(DBName);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click({
        force: true,
      });

      cy.get(dashboard.locators.set_as_homepage).click();
      cy.get(notifications.notification_content, { timeout: 30000 }).should(
        'contain',
        'Homepage has been set'
      );
      cy.wait('@setAsHomepage')
        .its('response.statusCode')
        .should('be.eq', 200);
      cy.get(CSSselector.homepage).click();
      cy.wait('@appgroupReportLoaded')
        .its('response.statusCode')
        .should('be.eq', 200);
      cy.get('iframe').then(($iframe) => {
        const $body = $iframe.contents().find('body');
        cy.wrap($body)
          .find(dashboard.locators.dashboard_title)
          .should('have.text', DBName);
      });

      // reset the homepage so that the other test are not disturbed:
      cy.visit('/arc/apps/homepage');
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });

      // open dashboard in edit mode and verify export option on ellipsis
      functions.navigate_to_visuals();
      dashboard.openDashboardforEdit(DBName);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click({
        force: true,
      });
      cy.get(dashboard.locators.export).click();
      exportDash(DBName, dashId);
    });

    // verify New Dashboard option on ellipsis
    cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click({ force: true });
    cy.get(dashboard.locators.new_dashboard).click();
    cy.get(CSSselector.Dashboard_title).should('have.text', '');

    // try to prevent memory exhaustion:
    cy.reload();

    // verify Email option on ellipsis
    dashboard.openDashboardforEdit(DBName);
    functions.click_email();
    cy.get(jobs.email_modal.email_tab.emailTo)
      .click()
      .type(emailTo)
      .type('{enter}');
    cy.get(jobs.email_modal.submit).click();
    cy.wait('@sendEmail')
      .its('response.statusCode')
      .should('be.eq', 200);
    cy.get(notifications.notification_content, { timeout: 30000 }).should(
      'contain',
      'Email successfully sent'
    );

    // verify Delete Dashbaord option on ellipsis
    cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click({ force: true });
    cy.get(dashboard.locators.delete_dashboard).click();
    cy.get(CSSselector.workspace_regression.Deletedashboardconfirm)
      .should('have.text', 'Delete')
      .click();
    cy.wait('@deleteDash')
      .its('response.statusCode')
      .should('be.eq', 200);
  });

  it('ARC-8094 : Actions performed with multisheet dashboard', function() {
    functions.checkArcenginePresence(this);

    cy.intercept('POST', '/arc/reports/reports_accel').as('acceleration');
    cy.fixture('configs/connections/connections').then((connectionData) => {
      const arcadia = {
        connection_type: 'Arcadia',
        connection_name: 'arcengine',
        hostname: 'ec2-35-154-163-180.ap-south-1.compute.amazonaws.com',
        port: '31050',
        username: 'admin',
        password: '',
      };
      connections.createConnection(arcadia);
    });
    const dsName = 'cab-rides-' + getRandomInt(100000000);
    const dddetails = {
      field_name1: 'srcarea',
      field_name2: 'rides',
    };
    functions.createNewDataset('default', 'cabrides', dsName);
    functions.addNewDB(DBName1);
    functions.createNewVisual(ConnectionType1, dsName);
    dashboard.editVisual();
    ddfunctions.addField(ShelfTypes.DIMENSIONS, dddetails.field_name1);
    ddfunctions.addField(ShelfTypes.AGGREGATES, dddetails.field_name2);
    dashboard.saveVisual();
    dashboard.closeVisual(false);
    dashboard.saveDashboard();
    functions.searchDashboard(DBName1);
    visualsPageFunctions.selectReportInGrid(DBName1);
    cy.get(dashboard.locators.visual_tab_ellipsis).click();
    cy.get(dashboard.locators.smart_acceleration).click();
    cy.wait('@acceleration')
      .its('response.statusCode')
      .should('be.eq', 200);
    cy.get(dashboard.locators.recommendation).should('have.text', 'Recommendation');
  });
});
