/**
 *
 * Author: Bhargav Thakkar
 *
 * Smoke test suite for Dataset Management feature. Covers following test
 * 1. Create join using pair and delete the same
 * 2. Create join through SQL expression and delete the same
 * 3. Create segment and delete
 * 4. Clone a field and delete the same
 *
 * Test Plan Wiki page: https://arcadiadata.atlassian.net/wiki/spaces/QA/pages/35291150/Datasets+General
 */

import { logins } from 'lib/arctest';
import * as dsFunction from 'lib/dataset';
import { Field } from 'lib/dataset';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as dsPermissions from 'lib/permissions';

describe('Issues for Dataset Managment Features.', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const databaseName = 'main';
  const sourceTable = 'trips';
  const datasetName = 'dsManage_TestIssues';
  const targetTable = 'trips_detail';
  const targetTable2 = 'census_pop';

  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
    functions.createDatasetFromConnection(connectionName, databaseName, sourceTable, datasetName);
  });

  beforeEach(() => {
    cy.visit('/arc/apps/data');
    dsPermissions.selectDataset(connectionName, datasetName);
  });

  context('Field Tab - Customer Issues', () => {
    const fieldName = 'pickup_neighborhood';

    const iField: Field = {
      DisplayName: 'From',
    };

    it('ARC-11132: Field names with reserved keywords (i.e. FROM) causing validation errors', () => {
      cy.get(dsManage.locators.fields).click();
      cy.get(dsManage.locators.editField).click();

      dsFunction.cloneField(fieldName);
      cy.log('Field cloned successfully');

      dsFunction.editFieldParameters('Copy of ' + fieldName, iField);

      cy.get('#save-attr-btn').click();

      cy.get(dsManage.locators.editField).click();

      dsFunction.editField(iField.DisplayName);
    });
  });

  context('Data Model Tab - Bugs', () => {
    it('VIZ-1194: starting a second join then canceling on Table Browser modal brings up the Edit Join modal with details of the first join', () => {
      cy.log("create first join, but don't save it");
      cy.get(dsManage.locators.dataModel).click();
      cy.get(dsManage.locators.editJoin).click();

      cy.get(dsManage.locators.addJoin(databaseName, sourceTable)).click();

      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('be.visible');
      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('not.be.visible');

      cy.get(dsManage.locators.databaseName).select(databaseName, { force: true });
      cy.get(dsManage.locators.tablename).select(targetTable, { force: true });

      cy.get(dsManage.locators.joinSelect).click();

      cy.get(dsManage.locators.editJoinLoadingSpinner).should('be.visible');
      cy.get(dsManage.locators.editJoinLoadingSpinner).should('not.be.visible');

      cy.get(dsManage.locators.applyJoin).click();

      // wait before saving join else UI doesn't show saved join.
      cy.wait(2000);

      cy.log('start creating second join');
      cy.get(dsManage.locators.addJoin(databaseName, sourceTable)).click();

      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('be.visible');
      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('not.be.visible');

      cy.log('Click on Cancel button on Table Browser modal');
      cy.get(dsManage.locators.joinAddTbDsCancel).click();

      cy.get(dsManage.locators.joinAddTbDsCancel).should('not.be.visible');

      cy.log(
        'wait for Table Browser modal to close and give time for unwanted Edit Join modal to appear'
      );
      cy.wait(10000);

      cy.log('Verify no Edit Join modal is shown');
      cy.get(dsManage.locators.editJoinModal).should('not.be.visible');

      cy.log('Verify joins are visible instead');
      cy.get(dsManage.locators.vizJoinDiv).should('be.visible');
    });

    it('VIZ-1195: disable create join button until all the data required arrives and selected, to avoid sending invalid (null) data', () => {
      cy.get(dsManage.locators.dataModel).click();
      cy.get(dsManage.locators.editJoin).click();

      cy.get(dsManage.locators.addJoin(databaseName, sourceTable)).click();

      cy.intercept({ url: '/**', middleware: true }, (req) => {
        req.on('response', (res) => {
          // Wait for delay in milliseconds before sending the response
          // to the client to simulate slow network
          res.setDelay(1000);
        });
      });

      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('be.visible');
      cy.get(dsManage.locators.joinSelect).should('be.disabled');

      cy.get(dsManage.locators.joinAddTbDsLoadingSpinner).should('not.be.visible');
      cy.get(dsManage.locators.joinSelect).should('not.be.disabled');

      cy.get(dsManage.locators.joinSelect).click();

      functions.loadingSpinnerCheck(dsManage.locators.editJoinLoadingSpinner);

      cy.get(dsManage.locators.editJoinTargetLbl).should('not.have.text', `${databaseName}.null`);
      cy.get(dsManage.locators.editJoinTargetLbl).should('have.text', `${sourceTable}_1`);

      cy.get(dsManage.locators.applyJoin).click();

      // wait before saving join else UI doesn't show saved join.
      cy.wait(2000);

      cy.get(dsManage.locators.joinTableBox)
        .last()
        .should('not.contain', 'null');
      cy.get(dsManage.locators.joinTableBox)
        .last()
        .should('contain', `${sourceTable}_1`);
    });

    it('VIZ-1195: upon slow network ensure new join is added and Edit Join modal is opened with correct data', () => {
      // Create first join. Should open Edit Join modal
      cy.get(dsManage.locators.dataModel).click();
      cy.get(dsManage.locators.editJoin).click();

      cy.intercept({ url: '/**', middleware: true }, (req) => {
        req.on('response', (res) => {
          // Wait for delay in milliseconds before sending the response
          // to the client to simulate slow network
          res.setDelay(1000);
        });
      });

      cy.get(dsManage.locators.addJoin(databaseName, sourceTable)).click();

      functions.loadingSpinnerCheck(dsManage.locators.joinAddTbDsLoadingSpinner);

      cy.get(dsManage.locators.databaseName).select(databaseName, { force: true });
      cy.get(dsManage.locators.tablename).select(targetTable, { force: true });

      cy.get(dsManage.locators.joinSelect).click();

      functions.loadingSpinnerCheck(dsManage.locators.editJoinLoadingSpinner);

      cy.get(dsManage.locators.applyJoin).click();

      // wait before saving join else UI doesn't show saved join.
      cy.wait(2000);

      // Create second join. Should open Edit Join modal with correct join table info
      cy.get(dsManage.locators.addJoin(databaseName, sourceTable)).click();

      functions.loadingSpinnerCheck(dsManage.locators.joinAddTbDsLoadingSpinner);

      cy.get(dsManage.locators.databaseName).select(databaseName, { force: true });
      cy.get(dsManage.locators.tablename).select(targetTable2, { force: true });

      cy.get(dsManage.locators.joinSelect).click();

      functions.loadingSpinnerCheck(dsManage.locators.editJoinLoadingSpinner);

      // Previously, new join was added and Edit Join modal is opened however with previously created join's table names
      cy.get(dsManage.locators.editJoinTargetLbl).should('not.have.text', targetTable);
      cy.get(dsManage.locators.editJoinTargetLbl).should(
        'have.text',
        `${databaseName}.${targetTable2}`
      );

      cy.get(dsManage.locators.applyJoin).click();

      cy.get(dsManage.locators.joinTableBox)
        .last()
        .should('not.contain', targetTable);
      cy.get(dsManage.locators.joinTableBox)
        .last()
        .should('contain', targetTable2);
    });
  });
});
