import * as arctest from 'lib/arctest';
import * as customSettingLocator from 'lib/customsettings/locators';
import * as dashboards from 'lib/dashboard';
import * as dsFunction from 'lib/dataset';
import * as functions from 'lib/functions';
import * as dsPermissions from 'lib/permissions';
import * as visual from 'lib/visual';
import {
  checkNthColorPaletteIsSelected,
  hexToRgb,
  navigateToFieldDatasetMenuAndPressEdit,
  removeElementFromColorShelf,
  settingsShouldBeDisabled,
  settingsShouldBeEnabled,
  testColorAndOpacityValueOfNthBarOnBarVisual,
  testFixedMapColorAndOpacity,
  unCheckCustomColorSetting,
} from './utility';

describe('Custom Dataset Color Setting Test', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const databaseName = 'main';
  const sourceTable = 'cereals';
  const datasetName = 'CerealsReColored';
  const {
    useDatasetColorSettingCheckbox,
    reverseColorPaletteCheckbox,
    opacitySliderInput,
    fixDimensionColorsCheckbox,
    tab,
  } = visual.locators.style.colors;

  const affectedSettings = [
    reverseColorPaletteCheckbox,
    opacitySliderInput,
    fixDimensionColorsCheckbox,
  ];
  const indexOfDesiredColorPalette = 12;
  const red = { hexColor: '#E34B57', rgb: 'rgb(227, 75, 87)' };
  const orange = { hexColor: '#EB8128', rgb: 'rgb(235, 129, 40)' };
  const customOpacityValue = 50;
  const customColoredField = 'cold_or_hot';
  const customReverseBoolean = true;
  const customMapAcrossVisualValue = false;
  const customFixedColorSetting = new Map<string, { rgb: string; hexColor: string }>([
    ['C', red],
    ['H', orange],
  ]);

  context(
    'Custom Dataset level field colors settings should be applied on visual by default (only if one element in the color_shelf)',
    () => {
      before(() => {
        arctest.logins.silent();
        cy.visit('/arc/apps/home');
        functions.deleteDatasetFromConnection(connectionName, datasetName);
        functions.createDatasetFromConnection(
          connectionName,
          databaseName,
          sourceTable,
          datasetName
        );
        cy.visit('/arc/apps/data');
        dsPermissions.selectDataset(connectionName, datasetName);
        navigateToFieldDatasetMenuAndPressEdit();

        // GIVEN
        dsFunction.setCustomColorSettingOfADatasetDimensionField(
          customColoredField,
          customReverseBoolean,
          indexOfDesiredColorPalette,
          customOpacityValue,
          customMapAcrossVisualValue,
          customFixedColorSetting
        );
      });

      describe('Set the cold_or_hot color and see it reflected in the visual', () => {
        // HAPPY scenario
        it('Bar visual. Only one (dimension type) element added to the color_shelf, which has custom color => custom color should be used by default. By uncheck it, dependent settings should be enabled', () => {
          // WHEN
          dashboards.createBarVisual(
            'E2E custom color 1',
            connectionName,
            datasetName,
            {
              x_shelfs: [customColoredField],
              y_shelfs: ['protein_grams'],
              color_shelfs: [customColoredField],
            },
            '#chart-trellis-groupedbars'
          );

          // THEN
          dashboards.chooseVisualTab('colors');
          cy.get(useDatasetColorSettingCheckbox).should('be.checked');
          cy.get(useDatasetColorSettingCheckbox).should('be.enabled');

          // Check custom color setting (opacity, colorpalette etc) are presented by value but they are disabled
          settingsShouldBeDisabled(affectedSettings);
          cy.get(reverseColorPaletteCheckbox).should(
            `be.${customReverseBoolean ? 'checked' : 'unchecked'}`
          );

          const { colorpaletteSelectionNthItem } = customSettingLocator.locators.colorSettings;
          cy.get(colorpaletteSelectionNthItem(indexOfDesiredColorPalette)).should(
            'have.class',
            'settings-color-selected'
          );
          cy.get(opacitySliderInput).should('have.value', customOpacityValue);
          cy.get(fixDimensionColorsCheckbox).should(
            `be.${customFixedColorSetting ? 'checked' : 'unchecked'}`
          );

          // TODO compare visuals visually:)!
          // C
          testColorAndOpacityValueOfNthBarOnBarVisual(
            0,
            customFixedColorSetting.get('C')?.rgb,
            customOpacityValue
          );
          // H
          testColorAndOpacityValueOfNthBarOnBarVisual(
            1,
            customFixedColorSetting.get('H')?.rgb,
            customOpacityValue
          );

          // WHEN
          unCheckCustomColorSetting();

          // THEN
          settingsShouldBeEnabled(affectedSettings);
        });

        // HAPPY scenario
        it('Bar visual. More than one element is present in the color_shelf at the beginning, but when we delete everything except the one with custom color settings, the colors reflect the custom setting', () => {
          // WHEN
          dashboards.createBarVisual(
            'E2E custom color 2',
            connectionName,
            datasetName,
            {
              x_shelfs: [customColoredField],
              y_shelfs: ['protein_grams'],
              color_shelfs: ['manufacturer', customColoredField],
            },
            '#chart-trellis-groupedbars'
          );

          // THEN
          dashboards.chooseVisualTab('colors');
          cy.get(useDatasetColorSettingCheckbox).should('be.not.visible');
          settingsShouldBeEnabled(affectedSettings);
          // index 1 is the default colorpalette
          checkNthColorPaletteIsSelected(1);

          // WHEN
          removeElementFromColorShelf('manufacturer');
          cy.get('#viz-refresh').dblclick();

          // THEN

          dashboards.chooseVisualTab('colors');
          cy.get(useDatasetColorSettingCheckbox).should('be.checked');
          cy.get(useDatasetColorSettingCheckbox).should('be.enabled');

          // Check custom color setting (opacity, colorpalette etc) are presented with corresponding value but they are disabled
          settingsShouldBeDisabled(affectedSettings);
          cy.get(reverseColorPaletteCheckbox).should(
            `be.${customReverseBoolean ? 'checked' : 'unchecked'}`
          );
          checkNthColorPaletteIsSelected(indexOfDesiredColorPalette);
          cy.get(opacitySliderInput).should('have.value', customOpacityValue);
          cy.get(fixDimensionColorsCheckbox).should(
            `be.${customFixedColorSetting ? 'checked' : 'unchecked'}`
          );

          // TODO compare visuals visually:)!
          // Check bars color represents the fixed colors
          // C
          testColorAndOpacityValueOfNthBarOnBarVisual(
            0,
            customFixedColorSetting.get('C')?.rgb,
            customOpacityValue
          );
          // H
          testColorAndOpacityValueOfNthBarOnBarVisual(
            1,
            customFixedColorSetting.get('H')?.rgb,
            customOpacityValue
          );
        });

        // Edge case
        it('A bar visual which had a custom colored field in the color_shelf changed to another visual. Settings should be still aligned with the custom color settings', () => {
          // WHEN
          dashboards.createBarVisual(
            'E2E custom color 3',
            connectionName,
            datasetName,
            {
              x_shelfs: [customColoredField],
              y_shelfs: ['protein_grams'],
              color_shelfs: [customColoredField],
            },
            '#chart-trellis-groupedbars'
          );

          // THEN
          [
            visual.locators.visualTypeSelectors.area,
            visual.locators.visualTypeSelectors.line_visual,
            visual.locators.visualTypeSelectors.bar_visual,
          ].forEach((id) => {
            dashboards.chooseVisualTab('data');
            cy.get(id).click();
            cy.get('#viz-refresh').click();
            dashboards.chooseVisualTab('colors');
            cy.get(useDatasetColorSettingCheckbox).should('be.checked');
            cy.get(useDatasetColorSettingCheckbox).should('be.enabled');

            // Check custom color setting (opacity, colorpalette etc) are presented by value but they are disabled
            settingsShouldBeDisabled(affectedSettings);
            cy.get(reverseColorPaletteCheckbox).should(
              `be.${customReverseBoolean ? 'checked' : 'unchecked'}`
            );
            checkNthColorPaletteIsSelected(indexOfDesiredColorPalette);
            cy.get(opacitySliderInput).should('have.value', customOpacityValue);
            cy.get(fixDimensionColorsCheckbox).should(
              `be.${customFixedColorSetting ? 'checked' : 'unchecked'}`
            );

            const indexOfCColorMap = 1;
            const indexOfHColorMap = 2;
            testFixedMapColorAndOpacity(
              indexOfCColorMap,
              hexToRgb(red.hexColor),
              customOpacityValue
            );
            testFixedMapColorAndOpacity(
              indexOfHColorMap,
              hexToRgb(orange.hexColor),
              customOpacityValue
            );
            // TODO when we start to use the new library compare visuals visually:)!
          });
        });

        [
          visual.locators.visualTypeSelectors.bar_line_visual,
          visual.locators.visualTypeSelectors.boxplot,
          visual.locators.visualTypeSelectors.histogram,
          visual.locators.visualTypeSelectors.gauge,
        ].forEach((visualType) => {
          it(`A bar visual which had a custom colored field in the color_shelf changed to ${visualType} visual. After changing visual which has no color shelf (link visual in this current example) -> custom settings shouldn't not be present, all settings should be enabled`, () => {
            // WHEN
            dashboards.createBarVisual(
              'E2E custom color 3',
              connectionName,
              datasetName,
              {
                x_shelfs: [customColoredField],
                y_shelfs: ['protein_grams'],
                color_shelfs: [customColoredField],
              },
              '#chart-trellis-groupedbars'
            );

            // THEN
            dashboards.chooseVisualTab('data');
            cy.get(visualType).click();
            cy.get('#viz-refresh').click();
            dashboards.chooseVisualTab('colors');
            cy.get(useDatasetColorSettingCheckbox).should('not.exist');

            // Check custom color setting (opacity, colorpalette etc) are presented by value but they are disabled
            cy.get(`${tab} [id*="${visual.locators.settingUserControl}"]`).should(
              'be.not.disabled'
            );
          });
        });

        it("Custom setting is available on table visuals's column setting by default", () => {
          // WHEN
          dashboards.createTableVisual(
            // TABLE
            'E2E custom color on table visual 3',
            connectionName,
            datasetName,
            {
              x_shelfs: [customColoredField],
              y_shelfs: ['protein_grams'],
            }
          );

          // WHEN
          dashboards.chooseVisualTab('data');

          const {
            backgroundColorMenu,
            backgroundColorMenuEnableButton,
            backgroundColorMenuColorShowPiece,
          } = customSettingLocator.locators.propertiesPanel;
          cy.get(
            customSettingLocator.locators.shelves.appShelves.dimensionItem(customColoredField)
          ).click();
          cy.get(backgroundColorMenu).click();
          cy.wait(2000);
          cy.get(backgroundColorMenuEnableButton).click();
          cy.get(backgroundColorMenuColorShowPiece).click();

          // THEN
          // Check custom color setting (opacity, colorpalette etc) are presented by value
          cy.get(reverseColorPaletteCheckbox).should(
            `be.${customReverseBoolean ? 'checked' : 'unchecked'}`
          );
          cy.get(
            `[data_setting_id='color_palette'] .settings-palette:nth-child(${indexOfDesiredColorPalette})`
          ).should('have.class', 'settings-color-selected');
          cy.get(opacitySliderInput).should('have.value', customOpacityValue);
          cy.get(fixDimensionColorsCheckbox).should(
            `be.${customFixedColorSetting ? 'checked' : 'unchecked'}`
          );
        });
      });
    }
  );
});
