import { BackendAPI } from 'lib/backend-api';
import * as connections from 'lib/connections';
import * as functions from 'lib/functions';
import { silent } from 'lib/logins';
import { CSSselector } from 'locators/common/CSSselector';

const importedDashboardIds: string[] = [];
let importedDatasetId: string = '';
let testerUserId: number;
const testerUsername = 'delete-dataset-tester';
const testerPassword = 'deleteDatasetTester';
let testerRoleId: number;
const testerRole = 'delete-dataset-role';

function removeDefaultGroupsAndRole(username: string, password: string) {
  // first log in with the new user
  // which will get the viz_quest_groups
  // and Database_admin role
  silent(username, password);
  BackendAPI.users.getByName(username).then((user) => {
    BackendAPI.groups.getByName('viz_guest_group').then((vizGuestGroup) => {
      if (vizGuestGroup) {
        BackendAPI.groups.update({
          id: vizGuestGroup.id,
          userIds: vizGuestGroup.users.map(({ id }) => id).filter((id) => id !== user.id),
        });
      }
    });

    BackendAPI.roles.getByName('Database admin').then((databaseAdminRole) => {
      BackendAPI.roles.update({
        id: databaseAdminRole.id,
        users: databaseAdminRole.users.filter((name) => name !== user.username),
      });
    });
  });
}

describe('Dataset Management Smoke - Delete Dataset Modal', () => {
  context('Delete Dataset', () => {
    const dataconnectionName = 'samples';
    const dataset = 'NYC Taxicab Rides - Dataset Delete Modal test';
    const jsonFolder = 'dataset-delete-modal-jsons/';
    const imports = [
      {
        fileName: `${jsonFolder}public-1-prev-linked-correct-dataset-1-not-correct-dataset-dashboard.json`,
        workspaceName: 'Public',
        needsForDeletion: true,
      },
      {
        fileName: `${jsonFolder}public-1visual-correct-dataset-dashboard.json`,
        workspaceName: 'Public',
        needsForDeletion: false,
      },
      {
        fileName: `${jsonFolder}private-1visual-correct-dataset-dashboard.json`,
        workspaceName: 'Private',
        needsForDeletion: false,
      },
      {
        fileName: `${jsonFolder}public-1correct-dataset-1-not-correct-dataset-dashboard.json`,
        workspaceName: 'Public',
        needsForDeletion: true,
      },
      {
        fileName: `${jsonFolder}private-1correct-dataset-1-not-correct-dataset-dashboard.json`,
        workspaceName: 'Private',
        needsForDeletion: true,
      },
      {
        fileName: `${jsonFolder}public-1linked-correct-dataset-dashboard.json`,
        workspaceName: 'Public',
        needsForDeletion: false,
      },
      {
        fileName: `${jsonFolder}private-1linked-correct-dataset-dashboard.json`,
        workspaceName: 'Private',
        needsForDeletion: false,
      },
    ];

    const importedFilesLength = imports.length;
    const datasetDeleteModal = 'div[data-test-id="delete-dataset-modal"]';
    const dashboardsTableRows = `${datasetDeleteModal} table[data-test-id="datasetDeleteModalDashboardsTable"] > tbody > tr`;
    const visualsTableRows = `${datasetDeleteModal} table[data-test-id="datasetDeleteModalVisualsTable"] > tbody > tr`;

    before(() => {
      silent();

      // Import each dashboard
      const [dashboardToImportWithDataset, ...dashboardsToImport] = imports;

      BackendAPI.dashboards
        .import({
          fileName: dashboardToImportWithDataset.fileName,
          dataconnectionName,
          workspaceName: dashboardToImportWithDataset.workspaceName,
          randomised: false,
        })
        .then(({ dashboard, datasets }) => {
          const { id, name } = dashboard;
          cy.log(`Dashboard imported: ${name}`);

          if (dashboardToImportWithDataset.needsForDeletion) {
            importedDashboardIds.push(id);
          }

          datasets.forEach((ds) => {
            if (ds.name === 'NYC Taxicab Rides - Dataset Delete Modal test') {
              importedDatasetId = ds.id.toString();
            }
          });

          BackendAPI.roles
            .create({
              name: testerRole,
              privs: [
                {
                  ptype: 'dataset',
                  perms: ['ds_manage', 'ds_appedit', 'ds_appview'],
                  dcid: '1',
                  dslist: [importedDatasetId],
                },
              ],
            })
            .then((role) => {
              cy.log(`Role created: ${role.name} with dslist: ${importedDatasetId}`);

              BackendAPI.users.create({
                username: testerUsername,
                password: testerPassword,
                roleIds: [role.id],
              });
              testerRoleId = role.id;
            })
            .then((user) => {
              cy.log(`User created: ${user.id}`);

              BackendAPI.users.update({
                id: user.id,
                groupIds: [],
              });
              testerUserId = user.id;
            });
        });

      dashboardsToImport.map(({ fileName, workspaceName, needsForDeletion }) =>
        BackendAPI.dashboards
          .import({
            fileName,
            dataconnectionName,
            workspaceName,
            randomised: false,
          })
          .then(({ dashboard, datasets }) => {
            const { id, name } = dashboard;
            cy.log(`Dashboard imported: ${name}`);

            if (needsForDeletion) {
              importedDashboardIds.push(id);
            }
          })
      );
    });

    after(() => {
      // dashboard cleanup
      // make sure we do the cleanup with the vizapps_admin
      silent();
      importedDashboardIds.forEach((dashboardId: string) => {
        BackendAPI.dashboards.delete(Number(dashboardId)).then(() => {
          cy.log(`Dashboard with id ${dashboardId} deleted successfully`);
        });
      });
      // user cleanup
      BackendAPI.users.delete(testerUserId).then(() => {
        cy.log(`User with id ${testerUserId} deleted successfully`);
      });
      // role cleanup
      BackendAPI.roles.delete(testerRoleId).then(() => {
        cy.log(`Role with id ${testerRoleId} deleted successfully`);
      });
    });

    it('1 - should count the dashboards and visuals in the Dataset Delete Modal with admin', () => {
      functions.navigate_to_data();

      connections.selectConnectionByName(dataconnectionName);

      cy.get(CSSselector.data.dataset_table_row + ' .dataset-title').each(($datasetel) => {
        if ($datasetel.text() === dataset) {
          // check the dashboard count is matching
          cy.wrap($datasetel)
            .parents('tr')
            .find('span.cell-wrapper a')
            .should('have.text', importedFilesLength.toString());

          // click on the trash icon
          cy.wrap($datasetel)
            .parents('tr')
            .find('span.fa-trash-o')
            .click();

          // the modal should be visible
          cy.get(datasetDeleteModal)
            .parent()
            .should('be.visible');

          // have to wait for the server:
          // the length should be grater than or equal to 1
          cy.get(dashboardsTableRows, { timeout: 30000 })
            .its('length')
            .should('be.gte', 1);

          // check the badges count
          // dashboards badge
          cy.get(`${datasetDeleteModal} button.badge`)
            .eq(0)
            .should('have.text', '4');
          // visuals badge
          cy.get(`${datasetDeleteModal} button.badge`)
            .eq(1)
            .should('have.text', '3');

          // check the rows in the dashboardsTable
          cy.get(dashboardsTableRows)
            .its('length')
            .should('eq', 4);

          // check the rows in the visualsTable
          cy.get(visualsTableRows)
            .its('length')
            .should('eq', 3);
        }
      });
    });

    it('2 - should log in the non-admin user and count the dashboards and visuals in the Dataset Delete Modal', () => {
      // silent(testerUsername, testerPassword);
      removeDefaultGroupsAndRole(testerUsername, testerPassword);

      cy.visit('/arc/apps/apps');
      functions.navigate_to_data();

      connections.selectConnectionByName(dataconnectionName);

      cy.get(CSSselector.data.dataset_table_row + ' .dataset-title').each(($datasetel) => {
        if ($datasetel.text() === dataset) {
          // check the dashboard count is matching
          cy.wrap($datasetel)
            .parents('tr')
            .find('span.cell-wrapper a')
            .should('have.text', (importedFilesLength - 3).toString());

          // click on the trash icon
          cy.wrap($datasetel)
            .parents('tr')
            .find('span.fa-trash-o')
            .click();

          // the modal should be visible
          cy.get(datasetDeleteModal)
            .parent()
            .should('be.visible');

          // have to wait for the server:
          // the length should be grater than or equal to 1
          cy.get(dashboardsTableRows, { timeout: 30000 })
            .its('length')
            .should('be.gte', 1);

          // check the badges count
          // dashboards badge
          cy.get(`${datasetDeleteModal} button.badge`)
            .eq(0)
            .should('have.text', '4');
          // visuals badge
          cy.get(`${datasetDeleteModal} button.badge`)
            .eq(3)
            .should('have.text', '3');

          // check the rows in the dashboardsTable
          cy.get(dashboardsTableRows)
            .its('length')
            .should('eq', 4);

          // check the rows in the visualsTable
          cy.get(visualsTableRows)
            .its('length')
            .should('eq', 3);

          cy.get(dashboardsTableRows)
            .filter(':contains("Restricted")')
            .should('have.length', 2);

          cy.get(visualsTableRows)
            .filter(':contains("Restricted")')
            .should('have.length', 2);

          cy.get('input[data-test-id=viz-delete-dataset-confirm]').type('DELETE');
          cy.get('button[data-test-id=delete-confirmation-yes]')
            .eq(1)
            .should('be.not.disabled')
            .click();

          // Retry the check for the non-existence of the dataset
          cy.get(`${CSSselector.data.dataset_table_row} .dataset-title`, { timeout: 12000 }).should(
            'have.length',
            0
          );
        }
      });
    });
  });
});
