/**
 *
 * Author: Bhargav Thakkar
 *
 * Smoke test suite for Dataset Management feature. Covers following test
 * 1. Create join using pair and delete the same
 * 2. Create join through SQL expression and delete the same
 * 3. Create segment and delete
 * 4. Clone a field and delete the same
 *
 * Test Plan Wiki page: https://arcadiadata.atlassian.net/wiki/spaces/QA/pages/35291150/Datasets+General
 */

import * as arctest from 'lib/arctest';
import { selectConnectionByName } from 'lib/connections';
import * as dsFunction from 'lib/dataset';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as dsPermissions from 'lib/permissions';
import { CSSselector } from 'locators/common/CSSselector';

describe('Dataset Management Smoke Test Suite.', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const databaseName = 'main';
  const sourceTable = 'trips';
  const datasetName = 'DatasetManageTestSet1';
  const targetTable = 'trips_detail';

  before(() => {
    arctest.logins.silent();
    cy.visit('/arc/apps/home');
    functions.createDatasetFromConnection(connectionName, databaseName, sourceTable, datasetName);
  });

  beforeEach(() => {
    cy.visit('/arc/apps/data');
    dsPermissions.selectDataset(connectionName, datasetName);
  });

  context('Dataset Management Smoke - Joins', () => {
    it('ARC-8055: Add a join pair for a data set', () => {
      const joinColumns = {
        pickup_neighborhood: 'pickup_neighborhood',
      };
      dsManage.addJoin(databaseName, sourceTable, databaseName, targetTable, joinColumns, 1);

      cy.get('[data-test-id="data-btn"]').click();

      dsManage.deleteJoin();

      cy.log('Verify that join deleted succesfully');
      cy.get(dsManage.locators.vizJoinImage).should('not.exist');
      // verify that join is removed
      cy.get(`div[data-original-title="${databaseName}.${targetTable}"].join-table-box i`).should(
        'not.exist'
      );

      cy.get(dsManage.locators.showdata).click();
      cy.get('[data-test-id=sample-table-loading-spinner]').should('be.visible');
      cy.get('[data-test-id=sample-table-loading-spinner]').should('be.not.visible');
      cy.get('[data-test-id="datasets-data-table"] th').should('contain', sourceTable);
      cy.get('[data-test-id="datasets-data-table"] th').should('not.contain', targetTable);
    });

    it('Create join using expression editor', () => {
      const joinExprStr = '[pickup_boro] = [trips_detail pickup_neighborhood]';

      dsManage.createJoinSQLExpr(
        databaseName,
        sourceTable,
        databaseName,
        targetTable,
        joinExprStr,
        1
      );

      cy.get('[data-test-id="data-btn"]').click();

      dsManage.deleteJoin();

      cy.log('Verify that join deleted succesfully');
      cy.get(dsManage.locators.vizJoinImage).should('not.exist');
      // verify that join is removed
      cy.get(`div[data-original-title="${databaseName}.${targetTable}"].join-table-box i`).should(
        'not.exist'
      );

      cy.get(dsManage.locators.showdata).click();
      cy.get('[data-test-id=sample-table-loading-spinner]').should('be.visible');
      cy.get('[data-test-id=sample-table-loading-spinner]').should('be.not.visible');
      cy.get('[data-test-id="datasets-data-table"] th').should('contain', sourceTable);
      cy.get('[data-test-id="datasets-data-table"] th').should('not.contain', targetTable);
    });
  });

  context('Dataset Management Smoke - Segments', () => {
    const segmentName = 'SegSmokeT1_' + Math.floor(Math.random() * 1000);
    const segmentGroup = 'SmokeGroup1';
    const fltrExpression = '[total_amount] > 100';

    it('ARC-8055: Add Segment for a dataset', () => {
      dsManage.createNewSegment(
        connectionName,
        datasetName,
        segmentName,
        fltrExpression,
        segmentGroup
      );

      // verify the segment count is correct
      cy.get('#dataset-segments-count').should('contain', '1');

      // verify that segment existing
      cy.get('#segments-table').should('contain', segmentName);

      dsManage.deleteSegment(segmentName);

      // verify that segment is deleted
      cy.get('#segments-table').should('not.contain', segmentName);

      // verify the segment count is correct
      cy.get('#dataset-segments-count').should('contain', '0');
    });

    /**
     * This can be used as a basic upgrade verification test.
     */
    it('Verify sample segments', () => {
      cy.visit('/arc/apps/data');

      // older VIZ versions have two ''Restaurant Inspection SF' datasets in the samples
      // one of them has segments, the other does not.
      // dsPermissions.selectDataset('samples', 'Restaurant Inspection SF');

      // this dataset has only one instance:
      dsPermissions.selectDataset('samples', 'US State Populations Over Time');
      cy.get(dsManage.locators.segments).click();
      cy.get(dsManage.locators.segListTableRows).should('have.length', 4);
    });
  });

  context('Dataset Managment - Smoke for Fields tab', () => {
    const fieldName = 'pickup_neighborhood';

    it('Clone a field', () => {
      cy.get(dsManage.locators.fields).click();
      cy.get(dsManage.locators.editField).click();

      dsFunction.cloneField(fieldName);
      cy.log('Field cloned successfully');

      // Reload the dataset page and navigate to field tab
      cy.visit('/arc/apps/data');

      dsPermissions.selectDataset(connectionName, datasetName);

      cy.get(dsManage.locators.fields).click();
      cy.get(dsManage.locators.editField).click();

      dsFunction.deleteField('Copy of ' + fieldName);
    });
  });

  context(
    'Dataset Managment - semicolon at the end of the query should not be a problem during creation',
    () => {
      it('Save query with semicolon at the end', () => {
        selectConnectionByName('samples');
        cy.get(CSSselector.data.new_dataset_button).click();
        cy.get('[data-test-id="dataset-create-modal"] [data-test-id="loading-spinner"]').should(
          'not.exist'
        );

        cy.get(CSSselector.data.new_dataset_modal.dataset_title)
          .click()
          .type('main.cereals e2e test dataset')
          .type('{enter}');

        cy.get(CSSselector.data.new_dataset_modal.sourceType_select)
          .click()
          .find('input')
          .type('sql')
          .type('{enter}');

        const SEMICOLON = ';';
        cy.get('[data-test-id="dataset-create-modal"]')
          .find('.ace_editor')
          .then((aceEditor) => {
            functions.waitForAceEditorReady(aceEditor);
          })
          .type(`select * from main.cereals${SEMICOLON}`);

        cy.intercept('POST', 'arc/datasets/dataset').as('dataset_created');
        cy.get(CSSselector.data.new_dataset_modal.create_button).click();
        cy.wait('@dataset_created')
          .its('response.statusCode')
          .should('eq', 200);
      });
    }
  );
});
