import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as ddfunctions from 'lib/deriveddata';
import * as functions from 'lib/functions';

const data = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'World Life Expectancy for derived data',
  databaseName: 'main',
  tableName: 'world_life_expectancy',
};

const tableDashboard: Dashboard = {
  title: 'Derived data test',
  subTitle: 'table visual test',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'table-visual',
          connection_name: data.connectionName,
          dataset_name: data.datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'country',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'fips_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'un_region',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'un_subregion',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'gdp_per_capita',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'population',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'life_expectancy',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: undefined,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Dashboard - Derived data test.', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
    functions.deleteDatasetFromConnection(data.connectionName, data.datasetName);
    functions.createDatasetFromConnection(
      data.connectionName,
      data.databaseName,
      data.tableName,
      data.datasetName
    );
    dashboards.createDashboard(tableDashboard);
  });

  it('Test 1: Create Dimension as a field in derived data section', () => {
    const dddetails = {
      field_shelf: ShelfTypes.DIMENSIONS,
      field_name: 'country',
      dd_name: 'dim_in_dd_section',
      dd_type: 'viz-expression-only',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.removeField(ShelfTypes.DIMENSIONS, dddetails.field_name);
    ddfunctions.addField(ShelfTypes.DIMENSIONS, dddetails.dd_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 2: Create Dimension as a field in derived data section as subquery', () => {
    const dddetails = {
      field_shelf: ShelfTypes.DIMENSIONS,
      field_name: 'un_region',
      dd_name: 'dim_in_dd_subquery',
      dd_type: 'viz-derived-data',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.removeField(ShelfTypes.DIMENSIONS, dddetails.field_name);
    ddfunctions.addField(ShelfTypes.DIMENSIONS, dddetails.dd_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 3: Create Aggregate as a field in derived data section', () => {
    const dddetails = {
      field_shelf: ShelfTypes.AGGREGATES,
      field_name: 'sum(population)',
      dd_name: 'mea_in_dataset_field',
      dd_type: 'viz-expression-only',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.removeField(ShelfTypes.AGGREGATES, dddetails.field_name);
    ddfunctions.addField(ShelfTypes.AGGREGATES, dddetails.dd_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 4: Create Aggregate as a field in derived data section as subquery', () => {
    const dddetails = {
      field_shelf: ShelfTypes.AGGREGATES,
      field_name: 'life_expectancy',
      dd_name: 'mea_in_dd_subquery',
      dd_type: 'viz-derived-data',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.removeField(ShelfTypes.AGGREGATES, dddetails.field_name);
    ddfunctions.addField(ShelfTypes.AGGREGATES, dddetails.dd_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 5: Create Dimension as a field in data set', () => {
    const dddetails = {
      field_shelf: ShelfTypes.DIMENSIONS,
      field_name: 'fips_code',
      dd_name: 'dim_in_ds',
      dd_type: 'ds-expression-only',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.addField(ShelfTypes.DIMENSIONS, dddetails.dd_name);
    ddfunctions.removeField(ShelfTypes.DIMENSIONS, dddetails.field_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 6: Create Aggregate as a field in dataset', () => {
    const dddetails = {
      field_shelf: ShelfTypes.AGGREGATES,
      field_name: 'sum(gdp_per_capita)',
      dd_name: 'mea_in_ds_field',
      dd_type: 'ds-expression-only',
    };
    ddfunctions.saveExpression(dddetails);
    ddfunctions.addField(ShelfTypes.AGGREGATES, dddetails.dd_name);
    ddfunctions.removeField(ShelfTypes.AGGREGATES, dddetails.field_name);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });

  it('Test 8: Delete a derived data fields', () => {
    // listed only the dd and dd sub-query dd fields
    // as for the ds dd needs to be deleted
    // at the dataset
    const derivedDataFields = [
      { shelf_type: ShelfTypes.DIMENSIONS, shelf_name: 'dim_in_dd_section' },
      { shelf_type: ShelfTypes.DIMENSIONS, shelf_name: 'dim_in_dd_subquery' },
      { shelf_type: ShelfTypes.AGGREGATES, shelf_name: 'mea_in_dataset_field' },
      { shelf_type: ShelfTypes.AGGREGATES, shelf_name: 'mea_in_dd_subquery' },
    ];

    derivedDataFields.forEach((field) => {
      ddfunctions.removeField(field.shelf_type, field.shelf_name);
    });

    // derivedDataFields.forEach((field) => {
    //   if (field.shelf_type === ShelfTypes.DIMENSIONS) {
    //     ddfunctions.deleteDerivedDataDimField(field.shelf_name);
    //   } else {
    //     ddfunctions.deleteDerivedDataAggField(field.shelf_name);
    //   }
    // });

    cy.wait(3000);
    dashboards.saveVisual();
    dashboards.closeVisual(false);
  });
});
