import * as admAPI from 'lib/adminapi';
import * as arctest from 'lib/arctest';
import { locators as datasetLocator } from 'lib/arctest/screen/data/data';
import { BackendAPI } from 'lib/backend-api';
import * as connection from 'lib/connections';
import { locators } from 'lib/direct_access/locators';
import * as functions from 'lib/functions';
import { areFilesEqual } from 'tests/csv_excel_download/utils';

/**
 * Write the SQL query and run it.
 * @param connectionName : Name of a connection
 * @param query : sql query which will be executed in the SQL editor
 */
function runSQLQuery(connectionName: string, query: string, checkLimitBox = false) {
  connection.selectConnectionByName(connectionName);
  cy.get(locators.dataPageThreeDotDropDownButton).click();
  cy.get(locators.direct_access).click();

  cy.get(locators.limitCheckbox, { timeout: 30_000 }).should('be.visible');

  if (checkLimitBox) {
    cy.get(locators.limitCheckbox).check();
  } else {
    cy.get(locators.limitCheckbox).uncheck();
  }

  cy.get(locators.ace_editor).then((aceEditor) => {
    functions.waitForAceEditorReady(aceEditor);
  });

  cy.get(locators.ace_editor).click();
  cy.get(locators.ace_editor).type(query);
  cy.get(locators.run_button).click();
}

describe('Direct Access.', () => {
  const tableColumns = ['cereal_name', 'calories', 'protein_grams'];
  let validQuery =
    'select cereal_name, calories, protein_grams from cereals where calories IN (120) AND protein_grams = 3';
  validQuery = validQuery.toLowerCase().trim(); // try to avoid cypress bug with remove as much letter diff combination as possible
  const nonPermittedQuery = 'delete from cereals where calories = 120';
  const connectionName = Cypress.env('smokeConnectionName');
  const connTypes = ['sqlite', 'hive', 'arcengine'];
  let rtrnConnType: string;

  before(() => {
    arctest.logins.silent();
    BackendAPI.connections.getByName(connectionName).then((con) => {
      rtrnConnType = con.type;
    });
  });

  context('ARC-11287: Automate smoke test for Direct Access', () => {
    it('1: Run select query and verify table data, and download results', () => {
      cy.visit('/');
      cy.log('Smoke Test for Conn Type = ' + rtrnConnType);
      cy.log('Smoke Test for Conn  Index = ' + connTypes.indexOf(rtrnConnType));
      if (connTypes.indexOf(rtrnConnType) > -1) {
        runSQLQuery(connectionName, validQuery);
        cy.get(locators.table_title).then(($text) => {
          expect($text.text().trim()).to.eq(validQuery);
        });
        cy.get(locators.table_header).each(($el, index) => {
          assert.equal($el.text().trim(), tableColumns[index]);
          // We have multiple whitespaces in the table header is it normal (actual text value is: 'cereal_name\n\n  \n    \n  \n')
        });
        cy.get(locators.table_rows).should('have.length', 6);
      } else {
        cy.log('Skipping this test - Connection type not supported for this feature');
      }
    });

    it('2: Download results', () => {
      cy.get(locators.more_option_btn).click();
      cy.get(locators.howManyRowToDownloadInputField)
        .type('100')
        .trigger('change');
      cy.get(locators.downLoadResultAsCsv).click();
      const expectedFile = 'cypress/tests/direct_access/expected_results/result1.csv';
      // how much time we need to have the download created?
      cy.wait(3000);
      areFilesEqual(expectedFile, false);
    });

    it('3: Save The Query', () => {
      cy.get(locators.more_option_btn).click();
      cy.get(locators.saveQueryButton).click();
      cy.get(locators.queryNavigationTabs.savedQueriesTab).click();
      cy.get(`${locators.queryNavigationTabs.savedQueryCard}:contains(${validQuery})`, {
        timeout: 10000,
      }).should('exist');
    });

    it('4: Should be listed in the Query History tab', () => {
      cy.get(locators.queryNavigationTabs.queryHistoryTab).click();
      cy.get(`${locators.queryNavigationTabs.queryHistoryEntry}:contains(${validQuery})`, {
        timeout: 10000,
      }).should('exist');
    });

    it('5. Should be able to Create Dataset', () => {
      if (connTypes.indexOf(rtrnConnType) > -1) {
        cy.get(locators.createDatasetButton).click();
        cy.get(datasetLocator.dataset_create_modal, { timeout: 8000 }).should('be.visible');
        cy.get(`${datasetLocator.dataset_create_modal} .ace_editor`).contains(validQuery);
        // No point to check dataset creation here enough that the component filled correctly
        cy.get(datasetLocator.dataset_create_modal)
          .find('button:contains(Close)')
          .click();
      }
    });

    it('6: Run delete query and verify error', () => {
      if (connTypes.indexOf(rtrnConnType) > -1) {
        runSQLQuery(connectionName, nonPermittedQuery);
        cy.get(locators.table_title).then(($text) => {
          expect($text.text().trim()).to.eq(nonPermittedQuery);
        });
        cy.get(locators.output_panel).should(
          'contain',
          `The requested query is not permitted. ${nonPermittedQuery}`
        );
      } else {
        cy.log('Skipping this test - Connection type not supported for this feature');
      }
    });
  });
});
