import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import { chooseVisualTab } from 'lib/dashboard';
import * as drillinto from 'lib/drillinto';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

describe('DRILL INTO test suite - Bug ARC-11588.', () => {
  let barBuilderUrl: string;
  let groupedBarBuilderUrl: string;

  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });

    BackendAPI.dashboards
      .import({
        fileName: 'drillinto1_1_dash_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        const { id } = dashboard;
        barBuilderUrl = `/arc/apps/builder/${id}`;
      });

    BackendAPI.dashboards
      .import({
        fileName: 'groupedbar_dash_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        const { id } = dashboard;
        groupedBarBuilderUrl = `/arc/apps/builder/${id}`;
      });
  });

  function navigateBackward() {
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('navigated');
    cy.get('.arc-viz-title-container .viz-params-btn.viz-params-backward').click();
    cy.wait('@navigated');
  }

  function navigateForward() {
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('navigated');
    cy.get('.arc-viz-title-container .viz-params-btn.viz-params-forward').click();
    cy.wait('@navigated');
  }

  // Bar graph visual edit mode tests
  context('DRILL INTO Bar graph visual test suite', () => {
    it('Test 1: DRILL_INTO a bar visual in visual edit mode', () => {
      cy.visit(barBuilderUrl);
      functions.waitForVisualsLoaded();
      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });
      drillinto.drillIntoBar('cereal_name', 'Post');

      chooseVisualTab('data');

      // verify the cereal_name field is added in dimension shelf
      let xShelfExp;
      xShelfExp = { manufacturer: '', cereal_name: '' };
      drillinto.verifyShelf('x', xShelfExp);
      let filtersShelfExp;
      filtersShelfExp = { "manufacturer in ('Post')": '' };
      drillinto.verifyShelf('filters', filtersShelfExp);
      // close the visual discarding changes
      visual.closeVisual(true);
    });

    it('Test 2: DRILL_INTO a bar visual in visual edit mode with pills in color shelf', () => {
      BackendAPI.dashboards
        .import({
          fileName: 'drillinto1_2_dash_prettyfied.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
        })
        .then(({ dashboard }) => {
          const { id } = dashboard;
          const barWithColorBuilderUrl = `/arc/apps/builder/${id}`;
          cy.visit(barWithColorBuilderUrl);
          functions.waitForVisualsLoaded();
        });

      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });
      drillinto.drillIntoBar('cold_or_hot', 'General Mills', true);

      chooseVisualTab('data');

      // verify the cold_or_hot field is added in dimension shelf
      let xShelfExp;
      xShelfExp = { manufacturer: '', cold_or_hot: '' };
      drillinto.verifyShelf('x', xShelfExp);
      let colorShelfExp;
      colorShelfExp = { cereal_name: '' };
      drillinto.verifyShelf('color', colorShelfExp);
      let filtersShelfExp;
      filtersShelfExp = {
        "manufacturer in ('General Mills')": '',
        "cereal_name in ('Apple_Cinnamon_Cheerios')": '',
      };
      drillinto.verifyShelf('filters', filtersShelfExp);
      visual.closeVisual(true);
    });

    it('Test 3: DRILL_INTO a bar visual in visual edit mode with pills in filter shelf', () => {
      BackendAPI.dashboards
        .import({
          fileName: 'drillinto1_3_dash_prettyfied.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
        })
        .then(({ dashboard }) => {
          const { id } = dashboard;
          const barWithFilterBuilderUrl = `/arc/apps/builder/${id}`;
          cy.visit(barWithFilterBuilderUrl);
          functions.waitForVisualsLoaded();
        });

      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });

      drillinto.drillIntoBar('cereal_name', 'General Mills');

      chooseVisualTab('data');

      // verify the cereal_name field is added in dimension shelf
      let xShelfExp;
      xShelfExp = { manufacturer: '', cereal_name: '' };
      drillinto.verifyShelf('x', xShelfExp);
      let filtersShelfExp;
      filtersShelfExp = {
        "manufacturer in ('General Mills')": '',
        "manufacturer in ('American Home Food Products', 'General Mills', 'Kelloggs', 'Nabisco', 'Post', 'Quaker Oats', 'Ralston Purina')":
          '',
      };
      drillinto.verifyShelf('filters', filtersShelfExp);
      visual.closeVisual(true);
    });

    it('Test 4: DRILL_INTO a bar visual in visual edit mode with multiple drill into', () => {
      cy.visit(barBuilderUrl);
      functions.waitForVisualsLoaded();
      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });

      drillinto.drillIntoBar('cereal_name', 'General Mills');
      functions.waitForVisualsLoaded();
      drillinto.drillIntoBar('cold_or_hot', 'Cheerios');

      // verify the cereal_name and cold_or_hot fields are added in dimension shelf
      let xShelfExp;
      xShelfExp = { manufacturer: '', cold_or_hot: '', cereal_name: '' };
      drillinto.verifyShelf('x', xShelfExp);
      let filtersShelfExp;
      filtersShelfExp = {
        "manufacturer in ('General Mills')": '',
        "cereal_name in ('Cheerios')": '',
      };
      drillinto.verifyShelf('filters', filtersShelfExp);
      visual.closeVisual(true);
    });

    // Should be fixed/removed by VIZ-2652
    it.skip('Test 5: DRILL_INTO a bar visual in visual edit mode with undo/redo', () => {
      cy.visit(barBuilderUrl);
      functions.waitForVisualsLoaded();
      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });

      drillinto.drillIntoBar('cereal_name', 'General Mills');
      drillinto.drillIntoBar('cold_or_hot', 'Cheerios');

      // undo the changes done by DRILL INTO
      navigateBackward();

      let xShelfExp;
      xShelfExp = { manufacturer: '', cereal_name: '' };
      drillinto.verifyShelf('x', xShelfExp);
      let filtersShelfExp;
      filtersShelfExp = { "manufacturer in ('General Mills')": '' };
      drillinto.verifyShelf('filters', filtersShelfExp);

      drillinto.drillIntoBar('cold_or_hot', 'Cheerios');

      navigateBackward();
      navigateBackward();

      // undo the changes done by DRILL INTO
      navigateForward();
      navigateForward();

      // verify the cereal_name field is added in dimension shelf
      xShelfExp = { manufacturer: '', cold_or_hot: '', cereal_name: '' };
      drillinto.verifyShelf('x', xShelfExp);
      filtersShelfExp = {
        "manufacturer in ('General Mills')": '',
        "cereal_name in ('Cheerios')": '',
      };
      drillinto.verifyShelf('filters', filtersShelfExp);
      visual.closeVisual(true);
    });

    it('Test 6: DRILL_INTO a bar visual in visual edit mode with cloned dimension attribute', () => {
      BackendAPI.dashboards
        .import({
          fileName: 'drillinto8_dash_prettyfied.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
        })
        .then(({ dashboard }) => {
          const { id } = dashboard;
          const barWithClonedDimBuilderUrl = `/arc/apps/builder/${id}`;
          cy.visit(barWithClonedDimBuilderUrl);
          functions.waitForVisualsLoaded();
        });

      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });
      drillinto.drillIntoBar('cereal_name_cloned', 'American Home ');

      chooseVisualTab('data');

      // verify the cereal_name_cloned field is added in dimension shelf
      const xShelfExp = { manufacturer: '', cereal_name_cloned: '' };
      drillinto.verifyShelf('x', xShelfExp);
      const filtersShelfExp = { "manufacturer in ('American Home Food Products')": '' };
      drillinto.verifyShelf('filters', filtersShelfExp);
      // close the visual discarding changes
      visual.closeVisual(true);
    });
  });

  // Bar graph dashboard view mode tests
  // Should be fixed/removed by VIZ-2652
  context.skip('DRILL INTO Bar graph visual tests in Dashboard view mode', () => {
    it('Test 1: DRILL_INTO a bar visual in DB view mode', () => {
      cy.visit(barBuilderUrl);
      functions.waitForVisualsLoaded();
      // drillinto by cereal_name
      drillinto.drillIntoBarDBView('cereal_name');
      drillinto.drillIntoBarDBView('cold_or_hot');
      // verify the bars are equal to 9
      cy.wait(1000);
      cy.get('#main span.viz-params-btn.viz-params-backward').click();

      // undo the changes done by DRILL INTO
      cy.get('#main span.viz-params-btn.viz-params-forward').click();
      cy.get('#main span.viz-params-btn.viz-params-backward').click();
      cy.get('#main span.viz-params-btn.viz-params-refresh').click();
    });
  });

  // Grouped bar tests
  context('DRILL INTO Grouped Bar graph visual test suite in visual edit mode', () => {
    it('Test 1: DRILL_INTO a grouped bar visual in visual edit mode', () => {
      cy.visit(groupedBarBuilderUrl);
      functions.waitForVisualsLoaded();
      const visualNumber = 1;
      cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
        force: true,
      });

      cy.get('g.arc-bar-group:nth-of-type(4) rect:nth-of-type(3)').click();
      drillinto.drillIntoField('cold_or_hot');

      chooseVisualTab('data');

      // verify the cold_or_hot field is added in dimension shelf
      const xShelfExp = { manufacturer: '', cold_or_hot: '' };
      drillinto.verifyShelf('x', xShelfExp);
      const filtersShelfExp = {
        "manufacturer in ('General Mills')": '',
        "cereal_name in ('Cheerios')": '',
      };
      drillinto.verifyShelf('filters', filtersShelfExp);
      // close the visual discarding changes
      visual.closeVisual(true);
    });
  });

  context('DRILL INTO Grouped Bar graph visual tests in Dashboard view mode', () => {
    it('Test 1: DRILL_INTO a grouped bar visual in DB view mode', () => {
      cy.visit(groupedBarBuilderUrl);
      functions.waitForVisualsLoaded();

      cy.get('g.arc-bar-group:nth-of-type(4) rect:nth-of-type(3)').click();

      drillinto.drillIntoField('cold_or_hot');

      cy.get('.arc-bar-group rect').should('have.length', 1);
    });

    // Should be fixed/removed by VIZ-2652
    it.skip('Test 2: DRILL_INTO a grouped bar visual in DB view mode undo/redo/reset', () => {
      cy.visit(groupedBarBuilderUrl);
      functions.waitForVisualsLoaded();

      cy.get('g.arc-bar-group:nth-of-type(4) rect:nth-of-type(3)').click();

      cy.get('div.contextmenu-foot div.dropdown-on-hover').trigger('mouseover');
      cy.wait(1000);
      cy.get('.dropdown-on-hover li a').each(($el, index, $list) => {
        if ($el.text() === 'cold_or_hot') {
          cy.get(`.dropdown-on-hover li a[data-col=${index}]`).click();
        }
      });
      cy.get('.arc-bar-group rect').should('have.length', 1);
      cy.get('#main span.viz-params-btn.viz-params-backward').click();
      cy.get('.arc-bar-group rect').should('have.length', 77);

      // undo the changes done by DRILL INTO
      cy.get('#main span.viz-params-btn.viz-params-forward').click();
      cy.get('.arc-bar-group rect').should('have.length', 1);
      cy.get('#main span.viz-params-btn.viz-params-refresh').click();
    });
  });
});
