import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder/builder';
import * as dashboards from 'lib/dashboard/dashboard_functions';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import {
  addEmailTemplate,
  checkNewlyCreateEmailTemplateExists,
  deleteEmailTemplate,
  deleteLatestJob,
  fillAndSendEmailModal,
  getLatestJobLog,
  verifyEmailInMailHog,
} from './util';

const dataConnectionData = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'World Life Expectancy for email template test',
  databaseName: 'main',
  tableName: 'world_life_expectancy',
  filterName1: 'country',
  filterValue1: ['Afghanistan', 'Albania', 'Algeria'],
};

const tableDashboard: Dashboard = {
  title: 'Email send out test',
  subTitle: 'table visual - email send out test',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'table-visual',
          connection_name: dataConnectionData.connectionName,
          dataset_name: dataConnectionData.datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'country',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'population',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz end to end UI test', () => {
  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
    cy.visit('/arc/apps/');
  });

  describe('Email template test', () => {
    describe('As user "admin", he should be able to create and save an email template using the html template editor', () => {
      describe('create email template', () => {
        it('should delete the previously created email template', () => {
          cy.fixture('emails.json').then((data) => {
            deleteEmailTemplate(data.template.name);
          });
        });
        it('should let the user create a new email template', () => {
          cy.fixture('emails.json').then((data) => {
            addEmailTemplate(data.template);
          });
        });

        it('should check the newly created email template exists', () => {
          cy.fixture('emails.json').then((data) => {
            checkNewlyCreateEmailTemplateExists(data.template.name);
          });
        });
      });
    });
  });

  describe('Create connection and dashboard for email template send out', () => {
    describe('Delete previously created connection and create a new one', () => {
      it('should delete and re-create connection', () => {
        functions.deleteDatasetFromConnection(
          dataConnectionData.connectionName,
          dataConnectionData.datasetName
        );
        functions.createDatasetFromConnection(
          dataConnectionData.connectionName,
          dataConnectionData.databaseName,
          dataConnectionData.tableName,
          dataConnectionData.datasetName
        );
      });
      it('delete previous job', () => {
        deleteLatestJob();
      });
      it('should create dashboard', () => {
        dashboards.createDashboard(tableDashboard);
        arctest.addFilter([dataConnectionData.filterName1]);
        arctest.chooseOneOrMultiplePicklistItem(dataConnectionData.filterValue1);
        dashboards.saveDashboard();
        functions.click_email();
        fillAndSendEmailModal();
        cy.wait(2000);
      });
      it('should get the scheduled email log and compare the job data with the received email body', () => {
        getLatestJobLog(tableDashboard.title).then((receivedJobData) => {
          verifyEmailInMailHog(tableDashboard.title, receivedJobData);
        });
      });
    });
  });
});
