import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import { createConnection, selectConnectionByName } from 'lib/connections';
import { extractTargetDb } from 'lib/data/util';
import * as functions from 'lib/functions';
import { createNewDataset, navigate_to_data } from 'lib/functions';
import { getDatasetId, selectDataset } from 'lib/permissions/dataset_permissions';
import { setSiteSettingCheckbox } from 'lib/settings';
import { CSSselector } from 'locators/common/CSSselector';
import * as util from '../util';

const currentTime = new Date().getTime();

const hrPartDs = {
  databaseName: 'extracts',
  tableName: `hr_partitioned_test_${currentTime}`,
  datasetName: `hr_partitioned_test_${currentTime}`,
};

const impalaConn = {
  connection_type: 'Impala',
  connection_name: 'impala-test',
  hostname: 'szhou-datavizimpala-3.vpc.cloudera.com',
  port: '21050',
  username: '',
  password: '',
  advanced: {
    conn_mode: 'binary_con_mode',
    sock_type: 'normal_sock_type',
    con_authtype: 'con_authtype_radio_nosasl',
  },
};

const fullExtract = {
  srcConnName: Cypress.env('smokeConnectionName'),
  srcDsName: 'Cereals',
  tgtConnName: extractTargetDb.connection_name,
  tgtDbName: extractTargetDb.test_db_name,
  tgtTableName: `cdv_1_${currentTime}`,
  dims: {
    fields: [
      {
        fieldName: 'cereal_name',
        aggAction: null,
        labelToVerify: 'cereal_name',
      },
    ],
  },
  rowCount: '77',
  fieldName: 'cereal_name',
  fieldVal: '100%_Bran',
};

const emailExtract = {
  srcConnName: Cypress.env('smokeConnectionName'),
  srcDsName: 'Cereals',
  tgtConnName: extractTargetDb.connection_name,
  tgtDbName: extractTargetDb.test_db_name,
  tgtTableName: `cdv_2_${currentTime}`,
  dims: {
    fields: [
      {
        fieldName: 'cereal_name',
        aggAction: null,
        labelToVerify: 'cereal_name',
      },
    ],
  },
  advanced: {
    emailSuccess: true,
    emailFail: true,
    emailTo: 'a@gmail.com',
  },
};

const partExtract = {
  srcConnName: impalaConn.connection_name,
  srcDsName: hrPartDs.datasetName,
  tgtConnName: impalaConn.connection_name,
  tgtDbName: 'extracts',
  tgtTableName: `cdv_3_${currentTime}`,
  dims: {
    fields: [
      {
        fieldName: 'employee_name',
        aggAction: null,
        labelToVerify: 'employee_name',
      },
    ],
  },
  aggs: {
    fields: [
      {
        fieldName: 'salary',
        aggAction: null,
        labelToVerify: 'salary',
      },
    ],
  },
  parts: {
    fields: [
      {
        fieldName: 'employmentstatus_col',
        aggAction: null,
        labelToVerify: 'employmentstatus_col',
      },
    ],
  },
  rowCount: '75',
  fieldName: 'employee_name',
  fieldVal: 'test name',
  newFieldVal: 'test name 2',
};

const moreAggsExtract = {
  srcConnNameImpala: impalaConn.connection_name,
  srcDbNameImpala: 'default',
  srcTableNameImpala: 'hr_partitioned',
  srcDsNameImpala: 'hr_partitioned_more_aggs',
  srcConnNameSqlite: Cypress.env('smokeConnectionName'),
  srcDsNameSqlite: 'Cereals',
  tgtConnName: extractTargetDb.connection_name,
  tgtDbName: extractTargetDb.test_db_name,
  tgtTableName: `cdv_4_${currentTime}`,
  dims: {
    fields: [
      {
        fieldName: 'cereal_name',
        aggAction: 'shelf-agg-groupconcat',
        labelToVerify: 'group_concat(cereal_name)',
      },
      {
        fieldName: 'manufacturer',
        aggAction: null,
        labelToVerify: 'manufacturer',
      },
    ],
    duplicates: [
      {
        fieldName: 'cereal_name',
        aggAction: 'shelf-agg-max',
        labelToVerify: 'max(cereal_name)',
      },
      {
        fieldName: 'cereal_name',
        aggAction: null,
        labelToVerify: 'cereal_name',
      },
      {
        fieldName: 'manufacturer',
        aggAction: 'shelf-agg-max',
        labelToVerify: 'max(manufacturer)',
      },
      {
        fieldName: 'manufacturer',
        aggAction: null,
        labelToVerify: 'manufacturer',
      },
    ],
  },
  aggs: {
    fields: [
      {
        fieldName: 'Record Count',
        aggAction: null,
        labelToVerify: 'Record Count',
      },
      {
        fieldName: 'calories',
        aggAction: 'shelf-agg-sum',
        labelToVerify: 'sum(calories)',
      },
      {
        fieldName: 'cups_per_serving',
        aggAction: 'shelf-agg-avg',
        labelToVerify: 'avg(cups_per_serving)',
      },
      {
        fieldName: 'display_shelf',
        aggAction: null,
        labelToVerify: 'display_shelf',
      },
    ],
    duplicates: [
      {
        fieldName: 'calories',
        aggAction: 'shelf-agg-sum',
        labelToVerify: 'sum(calories)',
      },
    ],
  },
};

describe('Extracts tests', () => {
  before(() => {
    logins.silent();

    // enable extract feature
    setSiteSettingCheckbox('Enable Data Extracts', true);
    setSiteSettingCheckbox('Permit emails', true);
    createConnection(extractTargetDb);
    createConnection(impalaConn);
  });

  after(() => {
    // clean up full extracts ->
    navigate_to_data();
    selectDataset(fullExtract.srcConnName, fullExtract.srcDsName);
    util.deleteExtract(fullExtract);

    // clean up partitioned extracts ->
    // cleanup impala src table -- update row back
    selectConnectionByName(partExtract.srcConnName);
    util.navigate_to_sql();
    util.updateRowInImpalaSrcTable(
      hrPartDs,
      partExtract.newFieldVal,
      partExtract.fieldVal,
      partExtract
    );

    // cleanup impala db -- delete new extracts table
    util
      .runSqlOnSQLPage(`drop table ${partExtract.tgtDbName}.${partExtract.tgtTableName}`)
      .then(() => {
        util.ensureTableIsDropped(`${partExtract.tgtDbName}.${partExtract.tgtTableName}`);
      });

    // drop the extract source table:
    util
      .runSqlOnSQLPage(`drop table if exists ${hrPartDs.databaseName}.${hrPartDs.tableName}`)
      .then(() => {
        util.ensureTableIsDropped(`${hrPartDs.databaseName}.${hrPartDs.tableName}`);
      });
  });

  it('Full extracts test', () => {
    // samples connection is read-only in linux installations
    // we must create a separate dataconnection for extracts
    navigate_to_data();
    selectDataset(fullExtract.srcConnName, fullExtract.srcDsName);
    util.createExtract(fullExtract);
    util.runExtract(fullExtract.tgtDbName, fullExtract.tgtTableName, 'Success');
    selectConnectionByName(fullExtract.tgtConnName);
    util.checkDataExracted(fullExtract);
  });

  it('Partitioned extracts test', () => {
    // create a new table that can be safely updated
    selectConnectionByName(partExtract.srcConnName);
    util.navigate_to_sql();
    util.createSourcePartitionedTable(hrPartDs.databaseName, hrPartDs.tableName);
    // create new src dataset
    navigate_to_data();
    selectConnectionByName(partExtract.srcConnName);
    createNewDataset(hrPartDs.databaseName, hrPartDs.tableName, hrPartDs.datasetName);

    // select src dataset
    navigate_to_data();
    cy.get(CSSselector.data.connection_list).should('be.visible');
    selectDataset(partExtract.srcConnName, partExtract.srcDsName);

    // create partitioned extract
    util.createExtract(partExtract);

    // run partitioned extract, should succeed
    util.runExtract(partExtract.tgtDbName, partExtract.tgtTableName, 'Success');

    // check data extracted into extract table
    util.checkDataExracted(partExtract);

    // update row in impala src table
    util
      .updateRowInImpalaSrcTable(
        hrPartDs,
        partExtract.fieldVal,
        partExtract.newFieldVal,
        partExtract
      )
      .then(() => {
        // run partitioned extract, should succeed
        navigate_to_data();
        selectDataset(partExtract.srcConnName, partExtract.srcDsName);
        util.runExtract(partExtract.tgtDbName, partExtract.tgtTableName, 'Success');

        // check data extracted into extract table
        util.checkDataExracted(partExtract);
      });
  });

  it('Email extract status upon refresh test', () => {
    navigate_to_data();
    selectDataset(emailExtract.srcConnName, emailExtract.srcDsName);
    util.createExtract(emailExtract);

    // run extract, should succeed
    util.runExtract(emailExtract.tgtDbName, emailExtract.tgtTableName, 'Success');

    // verify success email is received in mailhog
    util.verifyEmailInMailHog(emailExtract, 'SUCCESS');

    // delete extract's target table
    const query = `drop table if exists ${emailExtract.tgtDbName}.${emailExtract.tgtTableName}`;

    // use the direct SQL access UI for dropping the table:
    selectConnectionByName(emailExtract.tgtConnName);
    util.navigate_to_sql();
    util.runSqlOnSQLPage(query);

    navigate_to_data();
    selectDataset(emailExtract.srcConnName, emailExtract.srcDsName);

    // run extract, should fail
    util.runExtract(emailExtract.tgtDbName, emailExtract.tgtTableName, 'Error');

    // verify error email is received in mailhog
    util.verifyEmailInMailHog(emailExtract, 'FAIL');
  });

  it('Test more aggregation support on the extract editor', () => {
    cy.intercept('POST', '/arc/datasets/extract').as('createExtract');
    cy.intercept('GET', '/arc/jobs/datacopylogs*').as('getExtracts');

    // when source is impala
    selectConnectionByName(moreAggsExtract.srcConnNameImpala);
    createNewDataset(
      moreAggsExtract.srcDbNameImpala,
      moreAggsExtract.srcTableNameImpala,
      moreAggsExtract.srcDsNameImpala
    );
    selectDataset(moreAggsExtract.srcConnNameImpala, moreAggsExtract.srcDsNameImpala);

    cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
    cy.get(CSSselector.dataset_detail.create_extract_btn).click();

    // verify dropdown for impala contains the impala functions
    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${CSSselector.dataset_detail.extract_col_action_dropdown}:first`)
      .click();

    const firstAggFnList = `${CSSselector.dataset_detail.extract_agg_funcs}:first`;
    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-ndv"]`)
      .parent()
      .should('not.have.class', 'arc-invisible');

    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-stddev"]`)
      .parent()
      .should('not.have.class', 'arc-invisible');

    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-variance-pop"]`)
      .parent()
      .should('not.have.class', 'arc-invisible');

    functions.modalClick(CSSselector.dataset_detail.cancel_extract_form);

    // when source is sqlite
    navigate_to_data();
    selectDataset(moreAggsExtract.srcConnNameSqlite, moreAggsExtract.srcDsNameSqlite);

    // create extract
    cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
    cy.get(CSSselector.dataset_detail.create_extract_btn).click();

    util.selectTargetFields(moreAggsExtract);

    // verify dropdown for sqlite doesn't contain the impala functions
    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${CSSselector.dataset_detail.extract_col_action_dropdown}:first`)
      .click();

    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-ndv"]`)
      .parent()
      .should('have.class', 'arc-invisible');

    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-stddev"]`)
      .parent()
      .should('have.class', 'arc-invisible');

    cy.get(CSSselector.dataset_detail.new_extract_aggs)
      .find(`${firstAggFnList} [agg-action="shelf-agg-variance-pop"]`)
      .parent()
      .should('have.class', 'arc-invisible');

    // select duplicated fields and apply various aggregate functions to both dims and aggs
    util.selectFields('dims', moreAggsExtract);
    util.selectFields('aggs', moreAggsExtract);

    // remove duplicate field
    cy.get(CSSselector.dataset_detail.new_extract_dims)
      .find(`[data-value='cereal_name']`)
      .should('have.length', 3);

    cy.get(CSSselector.dataset_detail.new_extract_dims)
      .find(CSSselector.dataset_detail.extract_col_label)
      .contains(/^cereal_name$/)
      .parent(CSSselector.dataset_detail.extract_col_div)
      .find(CSSselector.dataset_detail.extract_col_action_dropdown)
      .click();

    cy.get(`${CSSselector.dataset_detail.extract_col_action_dropdown}[aria-expanded="true"]`)
      .siblings(CSSselector.dataset_detail.extract_col_action_menu)
      .find(CSSselector.dataset_detail.extract_remove_col)
      .scrollIntoView()
      .click();

    cy.get(CSSselector.dataset_detail.new_extract_dims)
      .find(`[data-value='cereal_name']`)
      .should('have.length', 2);

    functions.modalClick(CSSselector.dataset_detail.new_extract_submit);

    // verify extract is created
    cy.wait('@createExtract', { timeout: 500_000 });
    cy.wait('@getExtracts');
    cy.get(CSSselector.dataset_detail.extracts_table_container).should('be.visible');
    cy.contains(`${moreAggsExtract.tgtDbName}.${moreAggsExtract.tgtTableName}`)
      .scrollIntoView()
      .should('be.visible');

    // check editor shows extract column values properly
    const colLabels = [
      'group_concat(cereal_name)',
      'manufacturer',
      'max(cereal_name)',
      'max(manufacturer)',
      'manufacturer',
      'Record Count',
      'sum(calories)',
      'avg(cups_per_serving)',
      'display_shelf',
      'sum(calories)',
    ];
    util.checkExtractColsInEditor(
      `${moreAggsExtract.tgtDbName}.${moreAggsExtract.tgtTableName}`,
      10,
      colLabels
    );

    // ensure target table is not re-created and lost data upon editing just schedule on extract editor
    util.verifyUIRecogniseColsChange('sum(calories)');

    // close modals
    functions.modalClick(CSSselector.dataset_detail.edit_extract_confirm_cancel);
    cy.get(CSSselector.dataset_detail.edit_extract_confirmation_modal, { timeout: 100_000 }).should(
      'not.be.visible'
    );
    functions.modalClick(CSSselector.dataset_detail.cancel_extract_form);
  });

  it('Verify UI displays correctly extracts created through CLI', () => {
    getDatasetId(fullExtract.srcConnName, fullExtract.srcDsName).then((datasetId) => {
      BackendAPI.connections.getByName(fullExtract.tgtConnName).then((con) => {
        verifyCliCreatedExtract(datasetId, con.id);
      });
    });
  });

  function verifyCliCreatedExtract(datasetId: number, targetConnectionId: number) {
    // There are syntax differences in aliases of extracts created through cli and UI. This test
    // verifies that UI recognises those differences and displays extracts created through cli
    // properly
    const cliExtractTgtTableName = 'cdv_cli_de';
    cy.intercept(
      { url: `/arc/datasets/extract?dataset_id=${datasetId}&details=true`, middleware: true },
      (req) => {
        // stub request response with an extra extract with column info like an extract created through cli
        req.on('response', (res) => {
          const cliExtract = {
            id: 9,
            extract_type: 'refresh_full',
            srcDatasetId: datasetId,
            srcDatasetName: fullExtract.srcDsName,
            tgtDataconnectionName: fullExtract.srcConnName,
            tgtDataconnectionId: targetConnectionId,
            tgtDbname: fullExtract.tgtDbName,
            tgtTablename: cliExtractTgtTableName,
            extract_config: {
              username: 'vizapps_admin',
              source: {
                dimData: [
                  {
                    expr: '[manufacturer_code]',
                    alias: 'manufacturer_code',
                    type: 'String',
                  },
                  {
                    expr: '[cold_or_hot]',
                    alias: 'cold_or_hot',
                    type: 'String',
                  },
                ],
                aggData: [
                  {
                    expr: 'sum(1)',
                    alias: 'sum(1)',
                    type: 'Integer',
                  },
                  {
                    expr: 'count(distinct([cereal_name]))',
                    alias: 'count(distinct([cereal_name]))',
                    type: 'Integer',
                  },
                  {
                    expr: 'count([cereal_name])',
                    alias: 'count([cereal_name])',
                    type: 'Integer',
                  },
                  {
                    expr: 'avg([protein_grams])',
                    alias: 'avg([protein_grams])',
                    type: 'Real',
                  },
                  {
                    expr: 'max([calories])',
                    alias: 'max([calories])',
                    type: 'Integer',
                  },
                  {
                    expr: 'min([calories])',
                    alias: 'min([calories])',
                    type: 'Integer',
                  },
                ],
              },
              target: {
                partitionData: [],
              },
              notification_email: {
                email_success: null,
                email_fail: null,
                to_addresses: [],
                email_subject: null,
              },
              rewrite_properties: {
                rewrite_status: 'disabled_until_refreshed',
              },
            },
          };

          res.body.push(cliExtract);
          res.send(res);
        });
      }
    );

    navigate_to_data();
    selectDataset(fullExtract.srcConnName, fullExtract.srcDsName);
    cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();

    // check editor shows extract created through cli column values properly
    const colLabels = [
      'countdistinct(cereal_name)',
      'count(cereal_name)',
      'manufacturer_code',
      'cold_or_hot',
      'Record Count',
      'max(calories)',
      'min(calories)',
      'avg(protein_grams)',
    ];
    util.checkExtractColsInEditor(
      `${fullExtract.tgtDbName}.${cliExtractTgtTableName}`,
      8,
      colLabels
    );

    // ensure target table is not re-created and lost data upon editing just schedule on extract editor
    util.verifyUIRecogniseColsChange('max(calories)');
  }
});
