import { recurse } from 'cypress-recurse';
import { modalClick, waitForAceEditorReady } from 'lib/functions';
import { selectDataset } from 'lib/permissions/dataset_permissions';
import { CSSselector } from 'locators';

/**
 * creates extract with Manual schedule
 * method assumes you have navigated to correct dataset
 * @export
 * @param {*} extractData
 */
export function createExtract(extractData: any) {
  cy.intercept('POST', '/arc/datasets/extract').as('createExtract');
  cy.intercept('GET', '/arc/jobs/datacopylogs*').as('getExtracts');

  // create extract
  cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
  cy.get(CSSselector.dataset_detail.create_extract_btn).click();

  selectTargetFields(extractData);

  // dims
  selectFields('dims', extractData);

  // aggs
  selectFields('aggs', extractData);

  // partitions
  selectFields('parts', extractData);

  // email details
  if (extractData.hasOwnProperty('advanced')) {
    cy.get(CSSselector.dataset_detail.extract_form_advanced_nav).click();

    if (extractData.advanced.hasOwnProperty('emailSuccess') && extractData.advanced.emailSuccess) {
      cy.get(CSSselector.dataset_detail.extract_email_success_status).check();
    }

    if (extractData.advanced.hasOwnProperty('emailFail') && extractData.advanced.emailFail) {
      cy.get(CSSselector.dataset_detail.extract_email_fail_status).check();
    }

    if (extractData.advanced.hasOwnProperty('emailTo')) {
      cy.get('#email-to-tokenfield').type(`${extractData.advanced.emailTo}{enter}`);
    }

    if (
      extractData.advanced.hasOwnProperty('emailSubject') &&
      extractData.advanced.emailSubject.length > 0
    ) {
      cy.get(CSSselector.dataset_detail.extract_email_subject)
        .clear()
        .type(extractData.advanced.emailSubject);
    }
  }

  modalClick(CSSselector.dataset_detail.new_extract_submit);

  // verify extract is created
  cy.wait('@createExtract', { timeout: 500_000 });
  cy.wait('@getExtracts');
  cy.get(CSSselector.dataset_detail.extracts_table_container).should('be.visible');
  cy.contains(`${extractData.tgtDbName}.${extractData.tgtTableName}`)
    .scrollIntoView()
    .should('be.visible');
}

/**
 * runs extract and checks status
 * method assumes you have navigated to correct dataset
 * @export
 * @param {string} extractTgtDbName
 * @param {string} extractTgtTableName
 * @param {string} status
 */
export function runExtract(extractTgtDbName: string, extractTgtTableName: string, status: string) {
  cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
  cy.get(CSSselector.dataset_detail.extracts_table_container)
    .should('be.visible')
    .contains(`${extractTgtDbName}.${extractTgtTableName}`)
    .parent(CSSselector.dataset_detail.extract_table_row)
    .find(CSSselector.dataset_detail.extract_run_now_btn)
    .scrollIntoView()
    .click();

  // wait for 'Run Extract' button event listeners to be attached,
  // since they close confirmation modal on save
  cy.wait(500);
  cy.get('button')
    .contains('Run Extract')
    .click();

  // data extract's page is repainted frequently, so extract rows are replaced each time.
  // we are checking for expected status repeatedly
  function checkStatus() {
    return cy
      .get(CSSselector.dataset_detail.extracts_table_container)
      .contains(`${extractTgtDbName}.${extractTgtTableName}`)
      .parent(CSSselector.dataset_detail.extract_table_row)
      .scrollIntoView()
      .find(CSSselector.dataset_detail.extract_state);
  }

  // To be sure we moved from a possible already Success (created->edit) state to Running
  cy.wait(1500);

  recurse(checkStatus, ($status: any) => $status.text() === status, {
    log: true,
    limit: 200, // max number of iterations
    timeout: 300_000, // time limit in ms
    delay: 1000, // delay before next iteration, ms
  });
}

/**
 * checks if data extracted into extract table
 * @export
 * @param {*} extractData
 */
export function checkDataExracted(extractData: any) {
  navigate_to_sql();

  const countQuery = `select count(*) from ${extractData.tgtDbName}.${extractData.tgtTableName}`;

  const runSqlQuery = () => {
    return runSqlOnSQLPage(countQuery).then(() => {
      return cy.get(CSSselector.sql_output); // pass sql_output to predicate
    });
  };

  // runSqlQuery until extracting data finished -- result data table is not empty
  recurse(
    runSqlQuery,
    ($sqlOutput: any) => $sqlOutput.find(`:contains(${extractData.rowCount})`).length > 0,
    {
      log: true,
      limit: 100, // max number of iterations
      timeout: 100_000, // time limit in ms
      delay: 1000, // delay before next iteration, ms
    }
  );

  const query =
    `select * from ${extractData.tgtDbName}.${extractData.tgtTableName} ` +
    `where ${extractData.fieldName} like '${extractData.fieldVal}'`;

  runSqlOnSQLPage(query).then(() => {
    cy.get(CSSselector.sql_output, { timeout: 10_000 }).contains(extractData.fieldName);
  });
}

/**
 * deletes extract
 * method assumes you have navigated to correct dataset
 * @export
 * @param {*} extractData
 */
export function deleteExtract(extractData: { tgtDbName: string; tgtTableName: string }) {
  cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
  cy.get(CSSselector.dataset_detail.extracts_table_container)
    .should('be.visible')
    .contains(`${extractData.tgtDbName}.${extractData.tgtTableName}`)
    .parent(CSSselector.dataset_detail.extract_table_row)
    .find(CSSselector.dataset_detail.extract_del_btn)
    .click();

  cy.intercept('GET', '/arc/jobs/datacopylogs*').as('fetchDataset');

  // wait for 'Delete Extract' button event listeners to be attached,
  // since they close confirmation modal on save
  cy.wait(1000);
  cy.get('button')
    .contains('Delete Extract')
    .click();

  cy.get('#discard-changes-modal').should('be.not.visible');

  cy.wait('@fetchDataset');
  // verify extract is deleted
  cy.get(CSSselector.dataset_detail.extracts_table_container, { timeout: 5_000 })
    .contains(`${extractData.tgtDbName}.${extractData.tgtTableName}`)
    .should('not.exist');
}

/**
 * verifies if extract refresh status email is received in mailhog
 * method assumes you have navigated to correct dataset's extract tab
 * @export
 * @param {*} extractData
 * @param {string} status
 */
export function verifyEmailInMailHog(extractData: any, status: string) {
  cy.get(CSSselector.dataset_detail.extracts_table_container)
    .contains(`${extractData.tgtDbName}.${extractData.tgtTableName}`)
    .parent(CSSselector.dataset_detail.extract_table_row)
    .invoke('data', 'id')
    .then((extractId) => {
      cy.mhGetAllMails()
        .mhFirst()
        .mhGetSubject()
        .should('eq', `[${status}] [Cloudera Data Visualization] Data Extract: ${extractId}`);
      cy.mhGetAllMails()
        .mhFirst()
        .mhGetBody()
        .should('contain', `"extractId": ${String(extractId)}`);
    });
  cy.mhGetAllMails()
    .mhFirst()
    .mhGetBody()
    .should('contain', `"tgtTablename": "${extractData.tgtTableName}"`);
  cy.mhGetAllMails()
    .mhFirst()
    .mhGetRecipients()
    .should('contain', `${extractData.advanced.emailTo}`);
  if (status === 'FAIL') {
    cy.mhGetAllMails()
      .mhFirst()
      .mhGetBody()
      .should(
        'contain',
        `Error:\r\ntable "${extractData.tgtTableName}" does not exist` // Postgres
        // `Error:\r\nCatalog Error: Table with name ${extractData.tgtTableName} does not exist!` // DuckDB
        // `Error:\r\nno such table: ${extractData.tgtDbName}.${extractData.tgtTableName}` // SQLite
      );
  }
}

function selectAllDimension() {
  cy.get(CSSselector.dataset_detail.new_extract_dims)
    .find('input.select-all-fields')
    .check();
}

function selectAllMeasures() {
  cy.get(CSSselector.dataset_detail.new_extract_aggs)
    .find('input.select-all-fields')
    .check();
}

export function copyDataViaExtract(extractData: any) {
  selectDataset(extractData.srcConnName, extractData.srcDsName);

  cy.intercept('POST', '/arc/datasets/extract').as('createExtract');
  cy.intercept('GET', '/arc/jobs/datacopylogs*').as('getExtracts');

  // create extract
  cy.get(CSSselector.dataset_detail.extracts_tab_nav_btn).click();
  cy.get(CSSselector.dataset_detail.create_extract_btn).click();

  selectTargetFields(extractData);

  selectAllDimension();
  selectAllMeasures();
  modalClick(CSSselector.dataset_detail.new_extract_submit);

  // verify extract is created
  cy.wait('@createExtract', { timeout: 500_000 });
  cy.wait('@getExtracts');
  cy.get(CSSselector.dataset_detail.extracts_table_container).should('be.visible');
  cy.contains(`${extractData.tgtDbName}.${extractData.tgtTableName}`)
    .scrollIntoView()
    .should('be.visible');

  runExtract(extractData.tgtDbName, extractData.tgtTableName, 'Success');
}

// Please note Record Count and partition fields should always have
// aggAction value set to null since they don't support aggregates,
// also they don't support duplication
export function selectFields(fieldType: string, extractData: any) {
  if (extractData.hasOwnProperty(fieldType)) {
    const fieldTypeDiv = `new_extract_${fieldType}`;

    // fields
    if (
      extractData[fieldType].hasOwnProperty('fields') &&
      extractData[fieldType].fields.length > 0
    ) {
      extractData[fieldType].fields.forEach((field: any) => {
        cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
          .find(`[data-value='${field.fieldName}']`)
          .check();

        if (fieldType !== 'parts') {
          selectAgg(field, fieldTypeDiv);
        }
      });
    }

    if (fieldType === 'parts') {
      return;
    }

    // duplicated fields
    if (
      extractData[fieldType].hasOwnProperty('duplicates') &&
      extractData[fieldType].duplicates.length > 0
    ) {
      extractData[fieldType].duplicates.forEach((field: any) => {
        // duplicate
        cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
          .find(`[data-value='${field.fieldName}']:last`)
          .parent(CSSselector.dataset_detail.extract_col_div)
          .find(CSSselector.dataset_detail.extract_col_action_dropdown)
          .click();

        cy.get(`${CSSselector.dataset_detail.extract_col_action_dropdown}[aria-expanded="true"]`)
          .siblings(CSSselector.dataset_detail.extract_col_action_menu)
          .find(CSSselector.dataset_detail.extract_duplicate_col)
          .scrollIntoView()
          .click();

        cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
          .find(`[data-value='${field.fieldName}']:last`)
          .check();

        selectAgg(field, fieldTypeDiv);
      });
    }
  }

  function selectAgg(field: any, fieldTypeDiv: string) {
    if (field.fieldName === 'Record Count') {
      return;
    }

    // select agg
    cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
      .find(`[data-value='${field.fieldName}']:last`)
      .parent(CSSselector.dataset_detail.extract_col_div)
      .find(CSSselector.dataset_detail.extract_col_action_dropdown)
      .click();

    if (field.aggAction) {
      cy.get(`${CSSselector.dataset_detail.extract_col_action_dropdown}[aria-expanded="true"]`)
        .siblings(CSSselector.dataset_detail.extract_col_action_menu)
        .find(`[agg-action="${field.aggAction}"]`)
        .click();
    } else {
      // make sure to unselect agg if any selected
      // (select sum to unselect it)
      cy.log(field.fieldName);
      cy.get(`${CSSselector.dataset_detail.extract_col_action_dropdown}[aria-expanded="true"]`)
        .siblings(CSSselector.dataset_detail.extract_col_action_menu)
        .find(`[agg-action="shelf-agg-sum"]`)
        .click();

      cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
        .find(`[data-value='${field.fieldName}']:last`)
        .parent(CSSselector.dataset_detail.extract_col_div)
        .find(CSSselector.dataset_detail.extract_col_action_dropdown)
        .click();

      cy.get(`${CSSselector.dataset_detail.extract_col_action_dropdown}[aria-expanded="true"]`)
        .siblings(CSSselector.dataset_detail.extract_col_action_menu)
        .find(`[agg-action="shelf-agg-sum"]`)
        .click();
    }

    cy.get((CSSselector.dataset_detail as any)[fieldTypeDiv])
      .find(CSSselector.dataset_detail.extract_col_label)
      .contains(field.labelToVerify);
  }
}

/**
 * selects target connection, database and table names on the extract modal
 * method assumes you have opened the extract modal
 * @export
 * @param {*} extractData
 */
export function selectTargetFields(extractData: any) {
  cy.get(CSSselector.dataset_detail.new_extract_tgt_dc_chosen).click();
  cy.get(CSSselector.dataset_detail.new_extract_tgt_dc_chosen_results)
    .contains(extractData.tgtConnName)
    .click();

  cy.get(CSSselector.dataset_detail.new_extract_tgt_db_chosen, { timeout: 100_000 }).click();
  cy.get(CSSselector.dataset_detail.new_extract_tgt_db_chosen_results)
    .contains(extractData.tgtDbName)
    .should('have.length.at.least', 1)
    .click();
  cy.get(CSSselector.dataset_detail.new_extract_tgt_table_name).type(extractData.tgtTableName);
}

/**
 * update row in impala src table
 * method assumes you have navigated to sql tab
 * @export
 * @param {*} hrPartDs
 * @param {string} oldEmpName
 * @param {string} newEmpName
 * @param {*} partExtract
 */
export function updateRowInImpalaSrcTable(
  hrPartDs: any,
  oldEmpName: string,
  newEmpName: string,
  partExtract: any
) {
  const query = `
    insert overwrite table ${hrPartDs.databaseName}.${hrPartDs.tableName}
    partition (employmentstatus_col)
    select case when employee_name like '${oldEmpName}' then '${newEmpName}' else employee_name end as employee_name,
    fromdiversityjobfair,
    'position',
    hispaniclatino,
    dateofhire,
    dateoftermination,
    termreason,
    employmentstatus,
    recruitmentsource,
    record_count,
    empid,
    salary,
    engagementsurvey,
    empsatisfaction,
    absences,
    employmentstatus_col
    from  ${hrPartDs.databaseName}.${hrPartDs.tableName}
  `;

  return runSqlOnSQLPage(query).then(() => {
    partExtract.fieldVal = newEmpName;
  });
}

export function createSourcePartitionedTable(dbName: string, tableName: string) {
  const query = `create table ${dbName}.${tableName} partitioned by (employmentstatus_col) as select * from default.hr_partitioned`;
  return runSqlOnSQLPage(query);
}

export function runSqlOnSQLPage(query: string) {
  cy.get(CSSselector.sql_tab)
    .find('.ace_editor')
    .then((aceEditor) => {
      waitForAceEditorReady(aceEditor);
    })
    .then((aceEditor) => {
      cy.window()
        .its('ace' as any)
        .invoke('edit', aceEditor.attr('id'))
        .invoke('setValue', query);
    });

  cy.get(CSSselector.run_sql_button).click();

  // wait for run to finish
  return cy
    .get(CSSselector.sql_output)
    .find(CSSselector.loading_bar, { timeout: 60_000 })
    .should('not.exist', { timeout: 60_000 });
}

export function navigate_to_sql() {
  cy.intercept('GET', '/arc/datasets/dataconnection').as('dataconnectionsLoaded');
  cy.get(CSSselector.topnav_sql).click();
  cy.wait('@dataconnectionsLoaded');
}

/**
 * verifies extract editor shows extract column values properly
 * method assumes you are on dataset's extract tab
 * @export
 * @param {string} tgtTable
 * @param {number} colsCnt
 * @param {string[]} colLabels
 */
export function checkExtractColsInEditor(tgtTable: string, colsCnt: number, colLabels: string[]) {
  cy.get(`${CSSselector.dataset_detail.extracts_table_container} [title="${tgtTable}"]`)
    .parent(CSSselector.dataset_detail.extract_table_row)
    .find('.edit')
    .scrollIntoView()
    .click();

  cy.get(CSSselector.dataset_detail.extract_col_div)
    .find(`${CSSselector.dataset_detail.extract_col_checkbox}:checked`)
    .should('have.length', colsCnt);

  colLabels.forEach((colLabel) => {
    cy.get(CSSselector.dataset_detail.extract_col_div)
      .find(CSSselector.dataset_detail.extract_col_label)
      .contains(colLabel);
  });
}

/**
 * verify target table is not re-created and lost data upon editing just schedule on extract editor
 * method assumes you opened extract editor
 * @export
 * @param {string} colLabel
 */
export function verifyUIRecogniseColsChange(colLabel: string) {
  cy.get(CSSselector.dataset_detail.extract_refresh).select('Quarterly');
  cy.get(CSSselector.dataset_detail.new_extract_submit).click();
  cy.get(CSSselector.dataset_detail.edit_extract_confirmation_modal).should('be.visible');
  cy.get(CSSselector.dataset_detail.edit_extract_warning).should('not.be.visible');

  modalClick(CSSselector.dataset_detail.edit_extract_confirm_cancel);

  cy.get(CSSselector.dataset_detail.edit_extract_confirmation_modal, { timeout: 100_000 }).should(
    'not.be.visible'
  );

  cy.get(CSSselector.dataset_detail.extract_col_div)
    .find(CSSselector.dataset_detail.extract_col_label)
    .contains(colLabel)
    .siblings(CSSselector.dataset_detail.extract_col_checkbox)
    .uncheck();

  cy.get(CSSselector.dataset_detail.new_extract_submit).click();
  cy.get(CSSselector.dataset_detail.edit_extract_warning).should('be.visible');
}

export function ensureTableIsDropped(dbTableName: string) {
  cy.get(CSSselector.sql_output, { timeout: 10_000 }).contains('Table has been dropped');
  cy.reload();
  cy.get(CSSselector.sql_tables_loading_spinner).should('be.visible');
  cy.get(CSSselector.sql_tables_loading_spinner).should('not.be.visible');
  cy.get(`${CSSselector.sql_tab_cli} [data-table="${dbTableName}"]`).should('not.exist');
}
