// creating the following:
// 2 users with system admin role: user1 and user 2
// 2 groups: user_group1(user1 and user2) and user_group2(user1)
// 3 segments in cereals dataset as follows:
// 1. name: Segment1, Group: G1, Filters: [calories] > 90
// 2. name: Segment2, Group: G1, Filters: [calories] > 100
// 3. name: Segment3, Filters: [calories] < 90

import { logins } from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import { Dashboard } from 'lib/dashboard/dashboards_interfaces';
import * as dsManage from 'lib/datasetManagement';
import * as fa from 'lib/filter_association';
import * as functions from 'lib/functions';
import * as dataset from 'lib/permissions';
import * as user_groups from 'lib/users_groups';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const datasetName2 = 'World Life Expectancy';
const segmentName = ['Segment1', 'Segment2', 'Segment3'];
const segmentGroup = ['G1', 'G1'];
const fltrExpression = ['[calories] > 90', '[calories] > 100', '[calories] < 90'];
const dashboard: Dashboard = {
  title: 'filterassocation_sqlquery_visual_dashboard',
  subTitle: 'filterassocation_sqlquery_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'sqlqueryvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-rawtable',
          visual_title: 'simple_sqlquery_visual',
          linked: 'false',
        },
      ],
    },
  ],
};

describe('Filter Association tests.', () => {
  context('ARC-10387', () => {
    before(() => {
      BackendAPI.users.create({
        username: 'user4',
        password: '123456',
        roleIds: [DefaultRoleIds.Analyst, DefaultRoleIds.DatabaseAdmin],
      });

      BackendAPI.users.create({
        username: 'user2',
        password: '123456',
        roleIds: [DefaultRoleIds.Analyst, DefaultRoleIds.DatabaseAdmin],
      });

      logins.silent();
      cy.visit('/arc/apps/apps');
      user_groups.createNewGroup('user_group1', ['user4', 'user2']);
      cy.wait(500);
      user_groups.createNewGroup('user_group2', ['user4']);

      for (let i = 0; i < 2; i++) {
        dsManage.createNewSegment(
          connectionName,
          datasetName,
          segmentName[i],
          fltrExpression[i],
          segmentGroup[i]
        );
      }
      dsManage.createNewSegment(connectionName, datasetName, segmentName[2], fltrExpression[2]);
    });

    it('ARC-10316: Create/Update/Edit Filter Associations', () => {
      // Create filter association and verify that it is created
      functions.navigate_to_data();
      dataset.selectDataset(connectionName, datasetName);
      fa.createFilterAssociation('FA1', ['vizapps_admin'], [], 'Segment1');
      fa.saveFilterAssociation();
      cy.get(fa.locators.fa_table + ' tbody tr td').should('be.visible');

      // Verify that modifications of user, group and segment in filter association are reflected in filter association data table.
      cy.get(fa.locators.fa_data.edit.replace('index', '1')).click();
      cy.get(fa.locators.fa_modal).should('be.visible');
      cy.get(fa.locators.segment)
        .contains('Segment2')
        .parent()
        .find('input')
        .check();
      cy.get(fa.locators.segment)
        .contains('Segment1')
        .parent()
        .find('input')
        .uncheck();
      fa.addUser(['user4']);
      fa.addGroup(['user_group1']);
      cy.intercept('get', '/arc/datasets/filterassociation*').as('updateFilterAssociation');
      cy.get(fa.locators.save).click();
      cy.wait('@updateFilterAssociation');
      cy.get(fa.locators.fa_data.users.replace('index', '1')).should('contain', 'user4');
      cy.get(fa.locators.fa_data.users.replace('index', '1')).should('contain', 'vizapps_admin');
      cy.get(fa.locators.fa_data.groups.replace('index', '1')).should('have.text', 'user_group1');

      // Verify the cancel button of filter association modal
      cy.get(fa.locators.fa_data.edit.replace('index', '1')).click();
      cy.get(fa.locators.fa_modal).should('be.visible');
      cy.wait(500);
      cy.get(fa.locators.save)
        .prev()
        .should('have.text', 'Cancel')
        .click();
      cy.get(fa.locators.fa_modal).should('not.be.visible');

      logins.silent('user4', '123456');

      functions.addNewDB('Filter Association Dashboard');
      functions.createNewVisual(Cypress.env('smokeConnectionName'), 'Cereals');
      cy.get('div.dataTables_scroll div.dataTables_scrollBody tbody tr').each(($el, i) => {
        const row = i + 1;
        cy.wrap($el)
          .get(':nth-child(' + row + ') > td:nth-child(4)')
          .invoke('text')
          .then(parseFloat)
          .should('be.greaterThan', 90);
      });
      dashboards.saveDashboard();

      // verify that filter association is properly applied on SQL type visual.
      dashboards.createDashboard(dashboard);
      cy.get('div.dataTables_scroll div.dataTables_scrollBody tbody tr').each(($el, i) => {
        const row = i + 1;
        cy.wrap($el)
          .get(':nth-child(' + row + ') > td:nth-child(4)')
          .invoke('text')
          .then(parseFloat)
          .should('be.greaterThan', 90);
      });
      dashboards.saveDashboard();
      logins.silent();
    });

    it("ARC-10408: Verification of moving users and groups from 'Available to select' column to 'Selected' column and vice versa", () => {
      // verification of moving single user and group
      functions.navigate_to_data();
      dataset.selectDataset(connectionName, datasetName);
      fa.createFilterAssociation('FA2', ['vizapps_admin'], [], 'Segment1');
      fa.saveFilterAssociation();
      cy.get(fa.locators.fa_data.edit.replace('index', '2')).click();
      fa.addUser(['user4']);
      cy.get(fa.locators.users.selected_users + ' label').should('contain', 'user4');
      fa.addGroup(['user_group1']);
      cy.get(fa.locators.groups.selected_groups + ' label').should('contain', 'user_group1');
      fa.removeUser(['user4']);
      cy.get(fa.locators.users.available_users + ' label').should('contain', 'user4');
      fa.removeGroup(['user_group1']);
      cy.get(fa.locators.groups.available_groups + ' label').should('contain', 'user_group1');

      // verification of moving multiple user and group
      fa.addUser(['user4', 'user2']);
      cy.get(fa.locators.users.selected_users + ' label')
        .should('contain', 'user4')
        .should('contain', 'user2');
      // cy.get(fa.locators.users.selected_users + ' label').should('contain','user2');
      fa.addGroup(['user_group1', 'user_group2']);
      cy.get(fa.locators.groups.selected_groups + ' label')
        .should('contain', 'user_group1')
        .should('contain', 'user_group2');
      // cy.get(fa.locators.groups.selected_groups + ' label').should('contain','user_group2');
      fa.removeUser(['user4', 'user2']);
      cy.get(fa.locators.users.available_users + ' label')
        .should('contain', 'user4')
        .should('contain', 'user2');
      // cy.get(fa.locators.users.available_users + ' label').should('contain','user2');
      fa.removeGroup(['user_group1', 'user_group2']);
      cy.get(fa.locators.groups.available_groups + ' label')
        .should('contain', 'user_group1')
        .should('contain', 'user_group2');
      // cy.get(fa.locators.groups.available_groups + ' label').should('contain','user_group2');

      // verification of moving user and group using search feature in filter association
      cy.get(fa.locators.groups.search).type('user_group1');
      cy.get(fa.locators.groups.available_groups + ' label').should('contain', 'user_group1');
      fa.addGroup(['user_group1']);
      cy.get(fa.locators.groups.selected_groups + ' label').should('contain', 'user_group1');
      cy.get(fa.locators.users.user_tab).click();
      cy.get(fa.locators.users.search).type('user4');
      cy.get(fa.locators.users.available_users + ' label').should('contain', 'user4');
      fa.addUser(['user4']);
      cy.get(fa.locators.users.selected_users + ' label').should('contain', 'user4');
      cy.wait(500);
      cy.get(fa.locators.save)
        .prev()
        .should('have.text', 'Cancel')
        .click();
      cy.get(fa.locators.fa_modal).should('not.be.visible');

      // verification of moving user and group by clicking on 'Select All' using search feature in filter association
      fa.createFilterAssociation('FA3');
      cy.get(fa.locators.users.user_tab).click();
      cy.get(fa.locators.users.search).type('user');
      cy.get(fa.locators.users.select_all + ' input').check();
      cy.get(fa.locators.users.add).click();
      cy.get(fa.locators.users.selected_users + ' label')
        .should('contain', 'user4')
        .should('contain', 'user2');

      cy.get(user_groups.locators.group_tab).click();
      cy.get(fa.locators.groups.search).type('user_group');
      cy.get(fa.locators.groups.select_all + ' input').check();
      cy.get(fa.locators.groups.add).click();
      cy.wait(200);
      cy.get(fa.locators.groups.selected_groups + ' label')
        .should('contain', 'user_group1')
        .should('contain', 'user_group2');
      cy.wait(500);
      cy.get(fa.locators.save)
        .prev()
        .should('have.text', 'Cancel')
        .click();
      cy.get(fa.locators.fa_modal).should('not.be.visible');
    });

    it('ARC-10407: Add/Edit segments, users and groups in Filter Association', () => {
      // verify that without any segments in dataset, 'No segments available' should be displayed on FA modal
      functions.navigate_to_data();
      dataset.selectDataset(connectionName, datasetName2);
      fa.createFilterAssociation('FA3');
      cy.get('#filterassociation-modal-datasets-segments-container').should(
        'contain',
        'No segments available'
      );
      cy.wait(500);
      cy.get(fa.locators.save)
        .prev()
        .should('have.text', 'Cancel')
        .click();
      cy.get(fa.locators.fa_modal).should('not.be.visible');

      // verify the grouped and ungrouped segments on FA modal
      functions.navigate_to_data();
      dataset.selectDataset(connectionName, datasetName);
      fa.createFilterAssociation('FA3');
      cy.get('#filterassociation-modal-datasets-segments-container').as('segments');
      cy.get('@segments')
        .find(' > ul')
        .should('have.length', 2);
      cy.get('@segments')
        .find('div')
        .contains('G1')
        .next()
        .find('li')
        .should('have.length', 2)
        .find('input')
        .should('have.attr', 'type', 'checkbox');
      cy.get('@segments')
        .find('div')
        .contains('Not Grouped')
        .next()
        .find('li')
        .should('have.length', 1)
        .find('input')
        .should('have.attr', 'type', 'checkbox');
      cy.wait(500);
      cy.get(fa.locators.save)
        .prev()
        .should('have.text', 'Cancel')
        .click();
      cy.get(fa.locators.fa_modal).should('not.be.visible');

      // verify that applied segment is displayed in the FA table accordingly.
      fa.createFilterAssociation('FA3', [], [], 'Segment1');
      fa.saveFilterAssociation();
      cy.get(fa.locators.fa_data.segments.replace('index', '3')).should('have.text', 'Segment1');

      // verify that if no user and no group is applied in FA then users and groups column should be blank.
      cy.get(fa.locators.fa_data.users.replace('index', '3')).should('have.text', '');
      cy.get(fa.locators.fa_data.groups.replace('index', '3')).should('have.text', '');

      // verify that after adding one user and one group in FA, it should be displayed in users and groups column
      cy.get(fa.locators.fa_data.edit.replace('index', '3')).click();
      fa.addUser(['user4']);
      cy.get(user_groups.locators.group_tab).click();
      fa.addGroup(['user_group1']);
      cy.intercept('GET', '/arc/datasets/filterassociation*').as('updateFilterAssociation');
      cy.get(fa.locators.save).click();
      cy.wait('@updateFilterAssociation');
      cy.get(fa.locators.fa_data.users.replace('index', '3')).should('have.text', 'user4');
      cy.get(fa.locators.fa_data.groups.replace('index', '3')).should('have.text', 'user_group1');

      // verify that after removing user and group from FA, it should not be displayed in users and groups column
      cy.get(fa.locators.fa_data.edit.replace('index', '3')).click();
      fa.removeUser(['user4']);
      cy.get(user_groups.locators.group_tab).click();
      fa.removeGroup(['user_group1']);
      cy.intercept('GET', '/arc/datasets/filterassociation*').as('updateFilterAssociation');
      cy.get(fa.locators.save).click();
      cy.wait('@updateFilterAssociation');
      cy.get(fa.locators.fa_data.users.replace('index', '3')).should('have.text', '');
      cy.get(fa.locators.fa_data.groups.replace('index', '3')).should('have.text', '');

      // Verify that filter association is deleted successfully.
      cy.get(fa.locators.fa_data.delete.replace('index', '1')).click();
      cy.get(fa.locators.delete_popup).click();
      cy.wait('@updateFilterAssociation');
    });
  });
});
