import * as arctest from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import * as filter from 'lib/filter';
import { CSSselector } from 'locators/common/CSSselector';

describe('Date Filter Tests.', () => {
  let visualBuilderUrl: string;
  let visualViewUrl: string;

  before(() => {
    arctest.logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'cypress_dashboard_date_filters_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        visualBuilderUrl = `/arc/apps/builder/${dashboard.id}`;
        visualViewUrl = `/arc/apps/app/${dashboard.id}`;
        cy.visit(visualViewUrl);
      });
  });

  function verifyDateSelected(startDate: string, endDate: string) {
    cy.get('.arc-viz-filter-container')
      .find('.filter-pill')
      .trigger('mouseover');
    cy.get('.header-label')
      .contains('inspection_date')
      .parent('tr')
      .find('.viz-tooltip-string')
      .should('include.text', `[inspection_date] between '${startDate}' and '${endDate}'`);
    cy.get('.arc-viz-filter-container')
      .find('.filter-pill')
      .should('contain', 'inspection_date');

    // Hack: for some reason cypress detect the .filter-pill detached from the DOM, so we make the tooltip disappear this way
    cy.get('.subnav-report').trigger('mouseover');

    filter.verifyCurrentParameters(['inspection_date.start']);
    filter.verifyCurrentParameters([startDate]);
    filter.verifyCurrentParameters(['inspection_date.end']);
    filter.verifyCurrentParameters([endDate]);
  }

  context('Date filter tests', () => {
    afterEach(() => {
      cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
      cy.get(CSSselector.dashboard.alert_container).should(($el) => {
        expect($el).to.have.css('display', 'none');
      });
    });

    it('change filter values for Relative dates in edit mode', () => {
      filter.applyDateFilterToVisual(
        '2016-05-01 14:00:23',
        'Years',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-01-01 00:00:00', '2017-12-31 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Quarters',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-07-01 00:00:00', '2017-09-30 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Months',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-06-01 00:00:00', '2017-06-30 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Days',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-05-02 00:00:00', '2017-05-02 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Hours',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-05-01 15:00:00', '2017-05-01 15:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Minutes',
        'Relative dates',
        visualBuilderUrl
      );
      // verify
      verifyDateSelected('2017-05-01 14:01:00', '2017-05-01 14:01:59');
    });

    it('change filter values for Relative dates in view mode', () => {
      // same test as above
      filter.applyDateFilterToVisual(
        '2016-05-01 14:00:23',
        'Years',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-01-01 00:00:00', '2017-12-31 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Quarters',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-07-01 00:00:00', '2017-09-30 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Months',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-06-01 00:00:00', '2017-06-30 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Days',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-05-02 00:00:00', '2017-05-02 23:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Hours',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-05-01 15:00:00', '2017-05-01 15:59:59');

      filter.applyDateFilterToVisual(
        '2017-05-01 14:00:23',
        'Minutes',
        'Relative dates',
        visualViewUrl
      );
      // verify
      verifyDateSelected('2017-05-01 14:01:00', '2017-05-01 14:01:59');
    });

    it('change filter values for fixed dates and quick filters in edit mode', () => {
      filter.applyDateFilterToVisual(
        '2016-11-01',
        '2017-01-31',
        'Fixed dates',
        visualBuilderUrl,
        'inspection_date',
        false
      );
      // verify
      verifyDateSelected('2016-11-01 00:00:00', '2017-01-31 00:00:00');

      filter.applyDateFilterToVisual('6H', '', 'Quick filters', visualBuilderUrl);
      filter.verifyCurrentParameters(['inspection_date.end']);

      filter.applyDateFilterToVisual('12H', '', 'Quick filters', visualBuilderUrl);
      filter.verifyCurrentParameters(['inspection_date.end']);
    });

    it('change filter values for fixed dates and quick filters in view mode', () => {
      filter.applyDateFilterToVisual('2016-11-01', '2017-01-31', 'Fixed dates', visualViewUrl);
      // verify
      verifyDateSelected('2016-11-01 00:00:00', '2017-01-31 00:00:00');

      filter.applyDateFilterToVisual('6H', '', 'Quick filters', visualViewUrl);
      filter.verifyCurrentParameters(['inspection_date.end']);

      filter.applyDateFilterToVisual('12H', '', 'Quick filters', visualViewUrl);
      filter.verifyCurrentParameters(['inspection_date.end']);
    });

    it('change the display settings', () => {
      filter.applyDateFilterToVisual('', '', '', visualBuilderUrl);

      // validate the changes
      filter.configureDateFilterBasic('inspection_date', 'Inspection Date', 'ipdate', true);

      filter.configureDateFilterStyle('700', true);
      filter.configureDateFilterScope('app_visibility', 'current_dataset');
    });
  });
});
