import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import { FilterScopeTypes, FilterVisibilityTypes } from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import * as functions from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';

function checkTableContent(visualSelectors: string[], expectedContent: string[][]) {
  for (const visualSelector of visualSelectors) {
    arctest.verifyTableContents(visualSelector, expectedContent);
  }
}

describe('Filter Scope Test', () => {
  const filterValue = '100%_Bran';
  const filterProp = 'cereal_name';
  const ds1VisualSelectorsSheet1 = [
    arctest.locators.visuals + ':nth-child(1)',
    arctest.locators.visuals + ':nth-child(3)',
  ];
  const ds1VisualSelectorsSheet2 = [arctest.locators.visuals + ':nth-child(1)'];
  const ds2VisualSelectorsSheet1 = [arctest.locators.visuals + ':nth-child(2)'];
  const ds2VisualSelectorsSheet2 = [arctest.locators.visuals + ':nth-child(2)'];
  const filteredCerealsTable = [['100%_Bran', 'C', 'Nabisco', '70']];
  const unfilteredCerealsTable = [
    ['100%_Bran', 'C', 'Nabisco', '70'],
    ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
    ['All-Bran', 'C', 'Kelloggs', '70'],
    ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
    ['Almond_Delight', 'C', 'Ralston Purina', '110'],
  ];
  let dashboardId: string;

  before(() => {
    logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'scopeTestsForFilters_prettyfied',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => (dashboardId = dashboard.id));
  });

  beforeEach(() => {
    cy.intercept('GET', `/arc/apps/builder/${dashboardId}`).as(`dashboard ${dashboardId} loaded`);
    cy.visit(`/arc/apps/builder/${dashboardId}`);
    cy.wait(`@dashboard ${dashboardId} loaded`);
    cy.get('.sheet-title')
      .contains('Sheet 1')
      .click();
    functions.waitForVisualsLoaded();
    filter.removeFilterValues(1, [filterValue]);
  });

  context('Testing filter scope settings', () => {
    it('1 : Check dashboard filter with SHEET scope on CURRENT dataset', () => {
      // default setting of filter scopes
      cy.get(CSSselector.dashboard.title).should('contain', 'Filter Scope Tests on Multi Sheets_');

      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking DS1 related tables on sheet1');
      checkTableContent(ds1VisualSelectorsSheet1, filteredCerealsTable);
      cy.log('Checking DS2 related tables on sheet1');
      checkTableContent(ds2VisualSelectorsSheet1, unfilteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking DS1 related tables on sheet2');
      checkTableContent(ds1VisualSelectorsSheet2, unfilteredCerealsTable);
      cy.log('Checking DS2 related tables on sheet1');
      checkTableContent(ds2VisualSelectorsSheet2, unfilteredCerealsTable);
    });

    it('2 : Check dashboard filter with SHEET scope on ALL datasets', () => {
      const allVisualSelectorsFromSheet1 = ds1VisualSelectorsSheet1.concat(
        ds2VisualSelectorsSheet1
      );
      const allVisualSelectorsFromSheet2 = ds1VisualSelectorsSheet2.concat(
        ds2VisualSelectorsSheet2
      );
      filter.configureFilterScope(FilterVisibilityTypes.SHEET, FilterScopeTypes.ALL);
      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, unfilteredCerealsTable);
    });

    it('3 : Check dashboard filter with SHEET scope set to EXPLICIT ', () => {
      filter.setExplicitFilter(3, filterProp);
      dashboards.saveDashboard();
      filter.setExplicitFilter(2, filterProp);
      dashboards.saveDashboard();

      const ds1RelatedVisualOnSheet1NonExplicit = ds1VisualSelectorsSheet1[0];
      const ds1RelatedVisualOnSheet1Explicit = ds1VisualSelectorsSheet1[1];
      const ds2RelatedVisualOnSheet1Explicit = ds2VisualSelectorsSheet1[0];

      filter.configureFilterScope(FilterVisibilityTypes.SHEET, FilterScopeTypes.EXPLICIT);
      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking DS1 related non explicit tables on sheet1');
      checkTableContent([ds1RelatedVisualOnSheet1NonExplicit], unfilteredCerealsTable);
      cy.log('Checking DS1 related explicit tables on sheet1');
      checkTableContent([ds1RelatedVisualOnSheet1Explicit], filteredCerealsTable);
      cy.log('Checking DS2 related explicit tables on sheet1');
      checkTableContent([ds2RelatedVisualOnSheet1Explicit], filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();
      cy.wait(2000);

      const ds1RelatedVisualOnSheet2NonExplicit = ds1VisualSelectorsSheet2[0];
      const ds2RelatedVisualOnSheet2Explicit = ds2VisualSelectorsSheet2[0];

      filter.setExplicitFilter(2, filterProp);
      dashboards.saveDashboard();

      cy.log('Checking DS1 related non explicit tables on sheet2');
      checkTableContent([ds1RelatedVisualOnSheet2NonExplicit], unfilteredCerealsTable);
      cy.log('Checking DS2 related explicit tables on sheet2');
      checkTableContent([ds2RelatedVisualOnSheet2Explicit], unfilteredCerealsTable);

      filter.removeExplicitFilter(2, filterProp);
      dashboards.saveDashboard();

      cy.get('.sheet-title')
        .contains('Sheet 1')
        .click();
      functions.waitForVisualsLoaded();
      cy.wait(2000);

      filter.removeExplicitFilter(3, filterProp);
      dashboards.saveDashboard();
      filter.removeExplicitFilter(2, filterProp);
      dashboards.saveDashboard();
    });

    it('4 : Check dashboard filter with APP scope on CURRENT dataset', () => {
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.CURRENT);
      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking DS1 related tables on sheet1');
      checkTableContent(ds1VisualSelectorsSheet1, filteredCerealsTable);
      cy.log('Checking DS2 related tables on sheet1');
      checkTableContent(ds2VisualSelectorsSheet1, unfilteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking DS1 related tables on sheet2');
      checkTableContent(ds1VisualSelectorsSheet2, filteredCerealsTable);
      cy.log('Checking DS2 related tables on sheet2');
      checkTableContent(ds2VisualSelectorsSheet2, unfilteredCerealsTable);
    });

    it('5 : Check dashboard filter with APP scope on ALL dataset', () => {
      const allVisualSelectorsFromSheet1 = ds1VisualSelectorsSheet1.concat(
        ds2VisualSelectorsSheet1
      );
      const allVisualSelectorsFromSheet2 = ds1VisualSelectorsSheet2.concat(
        ds2VisualSelectorsSheet2
      );
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.ALL);
      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, filteredCerealsTable);
    });

    it('6 : Check dashboard filter with APP scope set to EXPLICIT', () => {
      filter.setExplicitFilter(3, filterProp);
      dashboards.saveDashboard();
      filter.setExplicitFilter(2, filterProp);
      dashboards.saveDashboard();

      const ds1RelatedVisualOnSheet1NonExplicit = ds1VisualSelectorsSheet1[0];
      const ds1RelatedVisualOnSheet1Explicit = ds1VisualSelectorsSheet1[1];
      const ds2RelatedVisualOnSheet1Explicit = ds2VisualSelectorsSheet1[0];

      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.EXPLICIT);
      filter.applyDropdownFilterToVisual([filterValue]);

      cy.log('Checking DS1 related non explicit tables on sheet1');
      checkTableContent([ds1RelatedVisualOnSheet1NonExplicit], unfilteredCerealsTable);
      cy.log('Checking DS1 related explicit tables on sheet1');
      checkTableContent([ds1RelatedVisualOnSheet1Explicit], filteredCerealsTable);
      cy.log('Checking DS2 related explicit tables on sheet1');
      checkTableContent([ds2RelatedVisualOnSheet1Explicit], filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();
      cy.wait(2000);

      const ds1RelatedVisualOnSheet2NonExplicit = ds1VisualSelectorsSheet2[0];
      const ds2RelatedVisualOnSheet2Explicit = ds2VisualSelectorsSheet2[0];

      filter.setExplicitFilter(2, filterProp);
      cy.log('Checking DS1 related non explicit tables on sheet2');
      checkTableContent([ds1RelatedVisualOnSheet2NonExplicit], unfilteredCerealsTable);
      cy.log('Checking DS2 related explicit tables on sheet2');
      checkTableContent([ds2RelatedVisualOnSheet2Explicit], filteredCerealsTable);
    });

    // TODO: Skiped beacause of a known bug that is tracked on VIZ-2236
    it.skip('7 : EXPLICIT filter should still work after hit apply on the filter-modal ', () => {
      filter.setExplicitFilter(2, filterProp);
      dashboards.saveDashboard();

      filter.applyDropdownFilterToVisual([filterValue]);
      dashboards.saveDashboard();

      const ds1RelatedVisualsOnSheet1NonExplicit = ds1VisualSelectorsSheet1;
      const ds2RelatedVisualOnSheet1Explicit = ds2VisualSelectorsSheet1[0];

      filter.configureFilterScope(FilterVisibilityTypes.SHEET, FilterScopeTypes.EXPLICIT);
      dashboards.saveDashboard();

      cy.log('Checking DS1 related non explicit tables on sheet1');
      checkTableContent(ds1RelatedVisualsOnSheet1NonExplicit, unfilteredCerealsTable);
      cy.log('Checking DS2 related explicit tables on sheet1');
      checkTableContent([ds2RelatedVisualOnSheet1Explicit], filteredCerealsTable);

      cy.log('Edit the filter shelf');
      cy.get(CSSselector.visual_edit_specific(2)).click({
        force: true,
      });
      cy.get('[data-test-id="visual-data-tab-a"]').click();
      cy.get(CSSselector.filter_field).click();
      cy.get(
        '[data-test-id="properties-panel"] [data-test-id="properties-menu-select-values"]'
      ).click({ force: true });
      cy.wait(1200);
      cy.get('[data-test-id="filter-modal-apply-btn"]').click();
      cy.wait(1200);
      cy.get('[data-test-id="viz-refresh"]').click();
      dashboards.saveDashboard();

      cy.log('Checking DS1 related non explicit tables on sheet1');
      checkTableContent(ds1RelatedVisualsOnSheet1NonExplicit, unfilteredCerealsTable);
      cy.log('Checking DS2 related non explicit tables on sheet1');
      checkTableContent([ds2RelatedVisualOnSheet1Explicit], filteredCerealsTable);
    });

    it('8 : Check dashboard after removing an APP scope filter ', () => {
      const allVisualSelectorsFromSheet1 = ds1VisualSelectorsSheet1.concat(
        ds2VisualSelectorsSheet1
      );
      const allVisualSelectorsFromSheet2 = ds1VisualSelectorsSheet2.concat(
        ds2VisualSelectorsSheet2
      );
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.ALL);
      filter.applyDropdownFilterToVisual([filterValue]);
      dashboards.saveDashboard();

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, filteredCerealsTable);

      cy.get(CSSselector.filters.remove_filter_specific(1)).click({ force: true });
      cy.wait(3000);
      cy.get('.discard-changes-yes').click({ force: true });
      cy.wait(1000);
      dashboards.saveDashboard();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, unfilteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 1')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, unfilteredCerealsTable);

      arctest.addFilter([filterProp]);
      filter.applyDropdownFilterToVisual([filterValue]);
      dashboards.saveDashboard();
    });

    // TODO: Skiped beacause of a known bug that is tracked on VIZ-2235
    it.skip('9 : Check dashboard after reset an APP scope filter to SHEET scope from sheet2', () => {
      const allVisualSelectorsFromSheet1 = ds1VisualSelectorsSheet1.concat(
        ds2VisualSelectorsSheet1
      );
      const allVisualSelectorsFromSheet2 = ds1VisualSelectorsSheet2.concat(
        ds2VisualSelectorsSheet2
      );
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.ALL);
      filter.applyDropdownFilterToVisual([filterValue]);
      dashboards.saveDashboard();

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 2')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, filteredCerealsTable);

      cy.log('Reseting the scope to SHEET on sheet2');
      filter.configureFilterScope(FilterVisibilityTypes.SHEET, FilterScopeTypes.ALL);
      dashboards.saveDashboard();

      cy.log('Checking all tables on sheet2');
      checkTableContent(allVisualSelectorsFromSheet2, filteredCerealsTable);

      cy.get('.sheet-title')
        .contains('Sheet 1')
        .click();
      functions.waitForVisualsLoaded();

      cy.log('Checking all tables on sheet1');
      checkTableContent(allVisualSelectorsFromSheet1, unfilteredCerealsTable);

      arctest.addFilter(['cereal_name']);
      filter.applyDropdownFilterToVisual([filterValue]);
      dashboards.saveDashboard();
    });
  });
});
