import * as arctest from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import { FilterScopeTypes, FilterVisibilityTypes } from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

describe('Dropdown Filter Tests.', () => {
  let visualBuilderUrl: string;
  let visualViewUrl: string;
  let dashboardId: string;

  before(() => {
    arctest.logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'dashboardFilterTest_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        dashboardId = dashboard.id;
        visualViewUrl = `/arc/apps/app/${dashboardId}`;
        visualBuilderUrl = `/arc/apps/builder/${dashboardId}`;
      });
  });

  /**
   * View the dashboard without saving the changes,
   * assumes that you are on the the dashboard edit screen
   * and have made some changes which you dont want to save
   *
   */
  function discardChangesAndViewDashboard() {
    cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
    cy.get(CSSselector.dashboard.alert_container).should(($el) => {
      expect($el).to.have.css('display', 'none');
    });

    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  }

  context('1. Change filter values edit mode', () => {
    it('1: change value', () => {
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualBuilderUrl);
      filter.verifyAppliedFilter(['All-Bran', 'Apple_Cinnamon_Cheerios']);
      discardChangesAndViewDashboard();
    });

    it.skip('2: forward back controls', () => {
      // VIZ-2288 - filter navigation buttons are removed in edit mode
      // do they meet expectation on click (back, forward, current params, reset)
      // do current params stick on click
      cy.log('1.2: applying All-Bran, Apple_Cinnamon_Cheerios');
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualBuilderUrl);
      cy.log('Going back');
      filter.filterControlBack();
      cy.log('Verifying All-Bran');
      filter.verifyAppliedFilter(['All-Bran']);
      cy.wait(1000);
      cy.log('Going forward');
      filter.filterControlForward();
      cy.log('Verifying Apple_Cinnamon_Cheerios');
      filter.verifyAppliedFilter(['Apple_Cinnamon_Cheerios']);
      cy.log('Verifying Apple_Cinnamon_Cheerios');
      filter.verifyCurrentParameters(['Apple_Cinnamon_Cheerios']);
      discardChangesAndViewDashboard();
    });

    it('3: reset controls', () => {
      filter.applyDropdownFilterToVisual(
        ['All-Bran', 'Apple_Cinnamon_Cheerios', 'Special_K'],
        visualBuilderUrl
      );
      filter.verifyAppliedFilter(['Special_K']);
      filter.verifyCurrentParameters(['Special_K']);
      filter.filterControlReset();
      filter.verifyCurrentParameters(['No parameters']);

      discardChangesAndViewDashboard();
    });
  });

  context('2. Change filter values view mode', () => {
    afterEach(() => {
      cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
      cy.get(CSSselector.dashboard.alert_container).should(($el) => {
        expect($el).to.have.css('display', 'none');
      });
    });

    it('1: change value', () => {
      // VIEW
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualViewUrl);
      filter.verifyAppliedFilter(['All-Bran', 'Apple_Cinnamon_Cheerios']);
    });

    it('2: forward back controls', () => {
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualViewUrl);
      filter.filterControlBack();
      filter.verifyAppliedFilter(['All-Bran']);
      filter.verifyCurrentParameters(['All-Bran']);
      filter.filterControlForward();
      filter.verifyAppliedFilter(['Apple_Cinnamon_Cheerios']);
      filter.verifyCurrentParameters(['Apple_Cinnamon_Cheerios']);

      filter.applyDropdownFilterToVisual(
        ['All-Bran', 'Apple_Cinnamon_Cheerios', 'Special_K'],
        visualViewUrl
      );
      filter.verifyAppliedFilter(['Special_K']);
      filter.verifyCurrentParameters(['Special_K']);
      filter.filterControlReset();
      filter.verifyCurrentParameters(['No parameters']);
    });

    it('3: reset controls', () => {
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualViewUrl);
      filter.filterControlBack();
      filter.verifyAppliedFilter(['All-Bran']);
      filter.verifyCurrentParameters(['All-Bran']);
      filter.filterControlForward();
      filter.verifyAppliedFilter(['Apple_Cinnamon_Cheerios']);
      filter.verifyCurrentParameters(['Apple_Cinnamon_Cheerios']);

      filter.applyDropdownFilterToVisual(
        ['All-Bran', 'Apple_Cinnamon_Cheerios', 'Special_K'],
        visualViewUrl
      );
      filter.verifyAppliedFilter(['Special_K']);
      filter.verifyCurrentParameters(['Special_K']);
      filter.filterControlReset();
      filter.verifyCurrentParameters(['No parameters']);
    });
  });

  context('3. Dashboard filter settings', () => {
    beforeEach(() => {
      cy.visit(visualBuilderUrl);
    });

    afterEach(() => {
      cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
      cy.get(CSSselector.dashboard.alert_container).should(($el) => {
        expect($el).to.have.css('display', 'none');
      });
    });

    const dropdownDefaultFilterSettings = {
      width: '',
      item_count: '1000',
      textbox_parameter: false,
      upload_manually: true,
      single_select: false,
      all_option: false,
      display_dropdown: false,
      multi_param: false,
      hide_empty: true,
      remember_selections: false,
      apply_together: false,
    };

    const dropdownFilterSettings = {
      width: '500',
      item_count: '20',
      textbox_parameter: false,
      upload_manually: false,
      single_select: true,
      all_option: true,
      display_dropdown: true,
      multi_param: true,
      hide_empty: false,
      remember_selections: false,
      apply_together: false,
    };

    const enterManuallyFilterSettings = {
      width: '400',
      item_count: '30',
      textbox_parameter: false,
      upload_manually: true,
      single_select: false,
      all_option: false,
      display_dropdown: false,
      multi_param: false,
      hide_empty: false,
      remember_selections: true,
      apply_together: false,
    };

    it('1: test output parameters and filter values', () => {
      cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');

      // change output paramter to opname1
      filter.configureFilterValues('cereal_name', 'cereal_name1', 'opname1', '');

      // update expression
      // since this is the 1st visual
      visuals.editSpecificVisual(1);
      cy.get('[data-test-id="visual-data-tab-a"]').click();
      cy.get(CSSselector.filter_field).click();
      visuals.verifySQLExpression("[cereal_name] in (<<opname1.data:'Grape-Nuts'>>)");
      cy.get(CSSselector.refresh_visual).click({ force: true });
      visuals.closeVisual(false);
      dashboards.saveDashboard();

      // apply filter and verify for opname1 in VIEW mode
      filter.applyDropdownFilterToVisual(
        ['All-Bran', 'Apple_Cinnamon_Cheerios', 'Special_K'],
        visualViewUrl
      );

      filter.verifyCurrentParameters([
        'opname1.data',
        'All-Bran',
        'Apple_Cinnamon_Cheerios',
        'Special_K',
      ]);

      // apply filter and verify for opname1 in EDIT mode
      cy.get(dashboards.locators.edit_link).click();
      cy.wait(2000);
      filter.applyDropdownFilterToVisual(
        ['All-Bran', 'Apple_Cinnamon_Cheerios', 'Special_K'],
        visualViewUrl
      );

      filter.verifyCurrentParameters([
        'opname1.data',
        'All-Bran',
        'Apple_Cinnamon_Cheerios',
        'Special_K',
      ]);

      discardChangesAndViewDashboard();

      // revert output parameter to opname and verify that too
      cy.intercept('GET', `/arc/reports/report/${dashboardId}?assets=true`).as('reportLoaded');
      cy.intercept('GET', '/arc/reports/customcss').as('customcssLoaded');
      cy.intercept('GET', '/arc/datasets/dataset').as('datasetLoaded');
      cy.intercept('GET', '/arc/datasets/dataconnection').as('dataconnectionLoaded');
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect_parallel');
      cy.visit(visualBuilderUrl);
      cy.wait('@reportLoaded');
      cy.wait('@customcssLoaded');
      cy.wait('@datasetLoaded');
      cy.wait('@dataconnectionLoaded');
      cy.wait('@jsonselect_parallel');
      cy.wait('@jsonselect_parallel');

      filter.configureFilterValues('cereal_name', 'cereal_name', 'opname', '');
      //  reset expression to original
      visuals.editSpecificVisual(1);
      cy.get('[data-test-id="visual-data-tab-a"]').click();
      cy.get(CSSselector.filter_field).click();
      visuals.verifySQLExpression("[cereal_name] in (<<opname.data:'Grape-Nuts'>>)");
      cy.get('[data-test-id="visual-data-tab-a"]').click();
      cy.get(CSSselector.refresh_visual).click({ force: true });
      visuals.closeVisual(false);
      dashboards.saveDashboard();

      // verify above changes
      filter.applyDropdownFilterToVisual(['All-Bran', 'Apple_Cinnamon_Cheerios'], visualBuilderUrl);
      filter.verifyCurrentParameters(['opname.data', 'All-Bran', 'Apple_Cinnamon_Cheerios']);
      discardChangesAndViewDashboard();
    });

    it('2: display settings in view screen', () => {
      filter.configureFilterDisplaySettings(dropdownFilterSettings);
      dashboards.saveDashboard();

      filter.applySingleSelectFilterToVisual([], visualViewUrl);

      // validate the changes in VIEW mode
      if (dropdownFilterSettings.width !== '') {
        cy.get(CSSselector.filters.filter_container)
          .should('have.attr', 'style')
          .and('include', `width: ${dropdownFilterSettings.width}px`);
      }
      cy.get(CSSselector.filters.filter_select_dropdown).click({
        force: true,
      });
      // verify number of items in list +1 to accomodate (All)
      cy.get(CSSselector.filters.filter_picklist).should(
        'have.length',
        parseInt(dropdownFilterSettings.item_count, 10) + 1
      );

      if (dropdownFilterSettings.single_select && !dropdownFilterSettings.display_dropdown) {
        cy.get(CSSselector.filters.filter_picklist).should('have.class', 'radio');
        if (dropdownFilterSettings.all_option) {
          cy.get(CSSselector.filters.filter_picklist).should('contain', '(All)');
        }
      } else if (dropdownFilterSettings.single_select && dropdownFilterSettings.display_dropdown) {
        cy.get(CSSselector.filters.filter_picklist).should('not.have.class', 'radio');
        cy.get(CSSselector.filters.filter_container).should('have.class', 'dropdown');

        if (dropdownFilterSettings.all_option) {
          cy.get(CSSselector.filters.filter_selectlist).should('contain', '(All)');
        }
      }

      cy.get(CSSselector.filters.filter_select_dropdown).click();
      cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');

      filter.applySingleSelectFilterToVisual([], visualBuilderUrl);
      // now check for the enter values link to show up
      filter.configureFilterDisplaySettings(enterManuallyFilterSettings);

      dashboards.saveDashboard();
      filter.applyDropdownFilterToVisual([], visualViewUrl);

      if (enterManuallyFilterSettings.width !== '') {
        cy.get(CSSselector.filters.filter_container)
          .should('have.attr', 'style')
          .and('include', `width: ${enterManuallyFilterSettings.width}px`);
      }
      cy.get(CSSselector.filters.filter_dropdown).click();

      // verify number of items in list. no +1 since (All) is not enabled
      cy.get(CSSselector.filters.filter_picklist).should(
        'have.length',
        parseInt(enterManuallyFilterSettings.item_count, 10)
      );
      cy.get(CSSselector.filters.filter_warning).should(
        'contain',
        enterManuallyFilterSettings.item_count + ' displayed'
      );

      if (enterManuallyFilterSettings.upload_manually) {
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('valueSubmitted');

        cy.get(CSSselector.filters.filter_enter_values).should('contain', 'Enter values');
        cy.get(CSSselector.filters.filter_enter_values).click();
        cy.get(CSSselector.filters.filter_values_modal).type('100%_Bran');
        cy.get(CSSselector.filters.filtervalues_add).click();

        cy.get(CSSselector.filters.filter_dropdown).click();
        cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
        filter.verifyAppliedFilter(['100%_Bran']);
        cy.get(CSSselector.filters.filter_dropdown).click();
      }

      // reset to default
      filter.applyDropdownFilterToVisual([], visualBuilderUrl);
      filter.configureFilterDisplaySettings(dropdownDefaultFilterSettings);
      dashboards.saveDashboard();
    });

    it('3: display settings in edit screen', () => {
      filter.configureFilterDisplaySettings(dropdownFilterSettings);
      dashboards.saveDashboard();

      filter.applySingleSelectFilterToVisual([], visualBuilderUrl);

      // validate the changes in EDIT mode
      if (dropdownFilterSettings.width !== '') {
        cy.get(CSSselector.filters.filter_container)
          .should('have.attr', 'style')
          .and('include', `width: ${dropdownFilterSettings.width}px`);
      }
      cy.get(CSSselector.filters.filter_select_dropdown).click({
        force: true,
      });
      cy.get(CSSselector.filters.filter_picklist).should(
        'have.length',
        parseInt(dropdownFilterSettings.item_count, 10) + 1
      );

      if (dropdownFilterSettings.single_select && !dropdownFilterSettings.display_dropdown) {
        cy.get(CSSselector.filters.filter_picklist).should('have.class', 'radio');
        if (dropdownFilterSettings.all_option) {
          cy.get(CSSselector.filters.filter_picklist).should('contain', '(All)');
        }
      } else if (dropdownFilterSettings.single_select && dropdownFilterSettings.display_dropdown) {
        cy.get(CSSselector.filters.filter_picklist).should('not.have.class', 'radio');
        cy.get(CSSselector.filters.filter_container).should('have.class', 'dropdown');

        if (dropdownFilterSettings.all_option) {
          cy.get(CSSselector.filters.filter_selectlist).should('contain', '(All)');
        }
      }

      cy.get(CSSselector.filters.filter_select_dropdown).click();
      cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');

      // now check for the enter values link to show up
      filter.configureFilterDisplaySettings(enterManuallyFilterSettings);
      dashboards.saveDashboard();

      if (enterManuallyFilterSettings.width !== '') {
        cy.get(CSSselector.filters.filter_container)
          .should('have.attr', 'style')
          .and('include', `width: ${enterManuallyFilterSettings.width}px`);
      }
      cy.get(CSSselector.filters.filter_dropdown).click();

      // verify number of items in list. no +1 since (All) is not enabled
      cy.get(CSSselector.filters.filter_picklist).should(
        'have.length',
        parseInt(enterManuallyFilterSettings.item_count, 10)
      );
      cy.get(CSSselector.filters.filter_warning).should(
        'contain',
        enterManuallyFilterSettings.item_count + ' displayed'
      );

      if (enterManuallyFilterSettings.upload_manually) {
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('valueSubmitted');

        cy.get(CSSselector.filters.filter_enter_values).should('contain', 'Enter values');
        cy.get(CSSselector.filters.filter_enter_values).click();
        cy.get(CSSselector.filters.filter_values_modal).type('100%_Bran');
        cy.get(CSSselector.filters.filtervalues_add).click();

        cy.get(CSSselector.filters.filter_dropdown).click();
        cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
        filter.verifyAppliedFilter(['100%_Bran']);
        cy.get(CSSselector.filters.filter_dropdown).click();
      }

      discardChangesAndViewDashboard();
      // reset to default
      filter.applyDropdownFilterToVisual([], visualBuilderUrl);
      filter.configureFilterDisplaySettings(dropdownDefaultFilterSettings);
      dashboards.saveDashboard();
    });

    it('4: should update data fetched', () => {
      cy.get(CSSselector.filters.configure_filter_specific(1)).click({ force: true });

      cy.get(CSSselector.filters.settings_navtab)
        .contains('Data')
        .click();

      cy.get(CSSselector.filters.data.sort_order_desc).click();

      cy.get(CSSselector.filters.save_settings).click();
    });

    it('5: should change the scope', () => {
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.CURRENT);
    });

    it.skip('6: should apply custom styles');
  });
});

describe('Scope Tests For Filters.', () => {
  const numberOfSheets = 2;

  context('1. Dataset scope', () => {
    interface CompareData {
      [key: string]: number;
    }

    interface FilterCompareData extends CompareData {
      filterIdSheet1: number;
      filterIdSheet2: number;
      datasetIdSheet1: number;
      datasetIdSheet2: number;
    }

    const filterCompareData: FilterCompareData = {
      filterIdSheet1: 0,
      filterIdSheet2: 0,
      datasetIdSheet1: 0,
      datasetIdSheet2: 0,
    };

    before(() => {
      arctest.logins.silent();
      BackendAPI.dashboards
        .import({
          fileName: 'dashboardFilterTest_2_prettyfied.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
        })
        .then(({ dashboard }) => cy.visit(`/arc/apps/builder/${dashboard.id}`));

      // Get the filter and dataset ids here to be able to compare the param_history later
      for (let i = 1; i <= numberOfSheets; i++) {
        arctest.screen.builder.switchToSheet(i);
        cy.window()
          .its('arc.globals.reports' as any)
          .invoke('getAllSheetReports', [i])
          .then((allReports) => {
            const filters: any[] = Object.values(allReports).filter(
              (report: any) => report.report_type() === 'picklist'
            );
            filterCompareData[`filterIdSheet${i}`] = filters[0].report_id();
            filterCompareData[`datasetIdSheet${i}`] = filters[0].dataset_id();
          });
      }
      arctest.screen.builder.switchToSheet(1);
    });

    it('1: Correct dashboard is loaded', () => {
      cy.get(arctest.screen.builder.locators.dashboard_title).should(
        'contain',
        'Scope Tests For Filters'
      );
      cy.get(arctest.screen.builder.locators.dashboard_sheet_handles).should('length', 2);
    });

    it('2: test dashboard state on page load', () => {
      // Check filter widget F1 - should have All-Bran selected
      cy.get(arctest.screen.builder.locators.filter_widgets).should(
        'have.length',
        1,
        'number of filters on load'
      );
      cy.get(arctest.screen.builder.locators.filter_widgets + ' .selected-names');
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($filterWidget, i) => {
        const selector = '#' + $filterWidget.attr('id');
        arctest.screen.builder.verifyTitleFilter(selector, 'F1 from DS 1 - DS - cereal_name');
        arctest.screen.builder.verifyContentsMultiFilter(selector, ['All-Bran']);
      });
      // Check value in current params - it should have scope info
      cy.window()
        .its('arc.globals.params' as any)
        .invoke('getCopy')
        .should('have.length', 0);
      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should('have.length', 1);

      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should((arr) => {
          expect(arr[0]).to.deep.eq([
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.include_nulls',
              value: 'true',
              value_list: ['true'],
              keytype: 1,
              source_visual_id: filterCompareData.filterIdSheet1,
              source_dataset_id: filterCompareData.datasetIdSheet1,
              valuetype: 'STRING',
              scope: {
                scopeType: 'dataset',
                isAppScope: false,
                filterName: 'cereal_name',
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.data',
              value: "'All-Bran'",
              value_list: ['All-Bran'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.exclude',
              value: 'in',
              value_list: ['in'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
          ]);
        });

      cy.get(arctest.screen.builder.locators.visuals).should('length', 3);
      cy.get(arctest.screen.builder.locators.visuals).each(($viz, i, $list) => {
        const selector = '#' + $viz.attr('id');
        cy.get(selector + ' .dataTables_wrapper').each(($table) => {
          switch (i) {
            case 0:
              // Check number of items in V1 - same dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V1 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['All-Bran', 'C', 'Kelloggs', '70'],
              ]);
              break;
            case 1:
              // Check number of items in V2 - different dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V2 from DS 2');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
            case 2:
              // Check number of items in V3 - same dataset but accepts explicit scope params
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V3 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
          }
        });
      });
    });

    it('3: Change value of filter F1 and test dashboard state', () => {
      // change value of filter F1
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($li, i, $list) => {
        const selector = '#' + $li.attr('id');
        cy.get(selector).click();
        cy.get(selector + ' .picklist-list').should('be.visible');
        cy.get(selector + ' .picklist-list li label[title="Apple_Jacks"]').click();
      });

      // verification for (['All-Bran', 'Apple_Jacks'])
      // Check filter widget F1 - should have correct items selected
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($filterWidget, i, $list) => {
        const selector = '#' + $filterWidget.attr('id');
        arctest.screen.builder.verifyTitleFilter(selector, 'F1 from DS 1 - DS - cereal_name');
        arctest.screen.builder.verifyContentsMultiFilter(selector, ['All-Bran', 'Apple_Jacks']);
      });

      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should((arr) => {
          expect(arr[0]).to.deep.eq([
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.include_nulls',
              value: 'true',
              value_list: ['true'],
              keytype: 1,
              source_visual_id: filterCompareData.filterIdSheet1,
              source_dataset_id: filterCompareData.datasetIdSheet1,
              valuetype: 'STRING',
              scope: {
                scopeType: 'dataset',
                isAppScope: false,
                filterName: 'cereal_name',
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.data',
              value: "'All-Bran', 'Apple_Jacks'",
              value_list: ['All-Bran', 'Apple_Jacks'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.exclude',
              value: 'in',
              value_list: ['in'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
          ]);
        });

      cy.get(arctest.screen.builder.locators.visuals).each(($viz, i, $list) => {
        const selector = '#' + $viz.attr('id');
        cy.get(selector + ' .dataTables_wrapper').each(($table) => {
          if (i === 0) {
            // Check number of items in V1 - same dataset
            expect(
              $viz
                .find('.arc-viz-title-container')
                .text()
                .trim()
            ).to.eq('V1 from DS 1');
            arctest.screen.builder.verifyTableContents(selector, [
              ['All-Bran', 'C', 'Kelloggs', '70'],
              ['Apple_Jacks', 'C', 'Kelloggs', '110'],
            ]);
          }
        });
      });
    });

    // verification2('Almond_Delight')
    it('4: Switch to sheet 2 and test dashboard state on load', () => {
      arctest.screen.builder.switchToSheet(2);
      // Check filter widget F2 - should have Almond_Delight selected
      cy.get(arctest.screen.builder.locators.filter_widgets).should(
        'have.length',
        1,
        'number of filters on load'
      );
      cy.get(arctest.screen.builder.locators.filter_widgets + ' .selected-names');
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($filterWidget, i, $list) => {
        const selector = '#' + $filterWidget.attr('id');
        arctest.screen.builder.verifyTitleFilter(selector, 'F2 from DS 1 - cereal_name');
        arctest.screen.builder.verifyContentsMultiFilter(selector, ['Almond_Delight']);
      });

      cy.window()
        .its('arc.globals.params' as any)
        .invoke('getCopy')
        .should('have.length', 0);
      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should('have.length', 1);
      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should((arr) => {
          const expected = [
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.include_nulls',
              value: 'true',
              value_list: ['true'],
              keytype: 1,
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              valuetype: 'STRING',
              scope: {
                isAppScope: false,
                filterName: 'cereal_name',
                filterId: filterCompareData.filterIdSheet2,
                scopeType: 'global',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.data',
              value: "'Almond_Delight'",
              value_list: ['Almond_Delight'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              scope: {
                filterId: filterCompareData.filterIdSheet2,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'global',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.exclude',
              value: 'in',
              value_list: ['in'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              scope: {
                filterId: filterCompareData.filterIdSheet2,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'global',
              },
            },
          ];

          const actual = arr[0];

          // console.log('expected',JSON.stringify(expected,null,2));
          // console.log('actual',JSON.stringify(actual,null,2));
          expect(actual).to.deep.eq(expected);
        });
      cy.get(arctest.screen.builder.locators.visuals).should('length', 2);
      cy.get(arctest.screen.builder.locators.visuals).each(($viz, i, $list) => {
        const selector = '#' + $viz.attr('id');
        cy.get(selector + ' .dataTables_wrapper').each(($table) => {
          switch (i) {
            case 0:
              // Check number of items in V4 - same dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V4 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
            case 1:
              // Check number of items in V5 - same dataset but accepts explicit scope params
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V5 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
          }
        });
      });
    });

    // verification2(['Almond_Delight', 'Basic_4'])
    it('5: Change value of filter F2 and test dashboard state', () => {
      // change value of filter F2
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($li) => {
        $li.find('.selected-names').click();
        // cy.get($li.selector + '.picklist-list').should('be.visible');
        expect($li.find('.picklist-list')).to.be.visible;
        $li.find('.picklist-list li label[title="Basic_4"]').click();
      });

      // verification for (['Almond_Delight', 'Basic_4'])
      // Check filter widget F2 - should have correct items selected
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($filterWidget, i) => {
        const selector = '#' + $filterWidget.attr('id');
        arctest.screen.builder.verifyTitleFilter(selector, 'F2 from DS 1 - cereal_name');
        arctest.screen.builder.verifyContentsMultiFilter(selector, ['Almond_Delight', 'Basic_4']);
      });

      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should((arr) => {
          const expected = [
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.include_nulls',
              value: 'true',
              value_list: ['true'],
              keytype: 1,
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              valuetype: 'STRING',
              scope: {
                scopeType: 'global',
                isAppScope: false,
                filterName: 'cereal_name',
                filterId: filterCompareData.filterIdSheet2,
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.data',
              value: "'Almond_Delight', 'Basic_4'",
              value_list: ['Almond_Delight', 'Basic_4'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              scope: {
                filterId: filterCompareData.filterIdSheet2,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'global',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.exclude',
              value: 'in',
              value_list: ['in'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet2,
              source_dataset_id: undefined,
              scope: {
                filterId: filterCompareData.filterIdSheet2,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'global',
              },
            },
          ];
          const actual = arr[0];

          // console.log('expected',JSON.stringify(expected,null,2));
          // console.log('actual',JSON.stringify(actual,null,2));
          expect(actual).to.deep.eq(expected);
        });

      cy.get(arctest.screen.builder.locators.visuals).each(($viz, i, $list) => {
        const selector = '#' + $viz.attr('id');
        cy.get(selector + ' .dataTables_wrapper').each(($table) => {
          switch (i) {
            case 0:
              // Check number of items in V4 - same dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V4 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
                ['Basic_4', 'C', 'General Mills', '130'],
              ]);
              break;
            case 1:
              // Check number of items in V5 - same dataset but accepts explicit scope params
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V5 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
          }
        });
      });
    });

    // verification1(['All-Bran', 'Apple_Jacks'])
    it('6: Switch back to sheet 1 and test dashboard state', () => {
      arctest.screen.builder.switchToSheet(1);
      cy.get(arctest.screen.builder.locators.filter_widgets + ' .selected-names');
      cy.get(arctest.screen.builder.locators.filter_widgets).each(($filterWidget, i, $list) => {
        const selector = '#' + $filterWidget.attr('id');
        arctest.screen.builder.verifyTitleFilter(selector, 'F1 from DS 1 - DS - cereal_name');
        arctest.screen.builder.verifyContentsMultiFilter(selector, ['All-Bran', 'Apple_Jacks']);
      });

      // Check value in current params - it should have scope info
      cy.window()
        .its('arc.globals.params' as any)
        .invoke('getCopy')
        .should('have.length', 0);
      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should('have.length', 1);

      cy.window()
        .its('arc.globals.ms_dashboard' as any)
        .invoke('params_history')
        .invoke('current_params')
        .should((arr) => {
          expect(arr[0]).to.deep.eq([
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.include_nulls',
              value: 'true',
              value_list: ['true'],
              keytype: 1,
              source_visual_id: filterCompareData.filterIdSheet1,
              source_dataset_id: filterCompareData.datasetIdSheet1,
              valuetype: 'STRING',
              scope: {
                scopeType: 'dataset',
                isAppScope: false,
                filterName: 'cereal_name',
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.data',
              value: "'All-Bran', 'Apple_Jacks'",
              value_list: ['All-Bran', 'Apple_Jacks'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
            {
              isDatasetSegment: undefined,
              key: 'cereal_name.exclude',
              value: 'in',
              value_list: ['in'],
              keytype: 1,
              valuetype: 'STRING',
              source_visual_id: filterCompareData.filterIdSheet1, // id of current visual
              source_dataset_id: filterCompareData.datasetIdSheet1, // id of current visual's dataset
              scope: {
                filterId: filterCompareData.filterIdSheet1,
                datasetId: filterCompareData.datasetIdSheet1,
                filterName: 'cereal_name',
                isAppScope: false,
                scopeType: 'dataset',
              },
            },
          ]);
        });

      cy.get(arctest.screen.builder.locators.visuals).should('length', 3);
      cy.get(arctest.screen.builder.locators.visuals).each(($viz, i, $list) => {
        const selector = '#' + $viz.attr('id');
        cy.get(selector + ' .dataTables_wrapper').each(($table) => {
          switch (i) {
            case 0:
              // Check number of items in V1 - same dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V1 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['Apple_Jacks', 'C', 'Kelloggs', '110'],
              ]);
              break;
            case 1:
              // Check number of items in V2 - different dataset
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V2 from DS 2');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
            case 2:
              // Check number of items in V3 - same dataset but accepts explicit scope params
              expect(
                $viz
                  .find('.arc-viz-title-container')
                  .text()
                  .trim()
              ).to.equal('V3 from DS 1');
              arctest.screen.builder.verifyTableContents(selector, [
                ['100%_Bran', 'C', 'Nabisco', '70'],
                ['100%_Natural_Bran', 'C', 'Quaker Oats', '120'],
                ['All-Bran', 'C', 'Kelloggs', '70'],
                ['All-Bran_with_Extra_Fiber', 'C', 'Kelloggs', '50'],
                ['Almond_Delight', 'C', 'Ralston Purina', '110'],
              ]);
              break;
          }
        });
      });
    });
  });

  // params emitted in app scope are visible across different dashboards in same app

  // Testing receiver
  // Receiving app scope params by a visual
  //    1. App scope explicit params
  //    2. App scope dataset scope params
  // Receiving explicit scope params by a visual

  /*
  describe('Dataset scope filters work on same dataset and not on similar but different dataset', function() {
    it('Dataset scope filters do not work when colname is changed');
  });
  describe('Explicit/Dataset scope Filters do not go across sheets in a dashboard', function(){});
  describe('App scope filters pass from 1 dashboard to visual', function(){});
  */
});
