import { jobs } from 'fixtures/locators/jobs';
import * as apps from 'lib/apps';
import { goToJobSchedulePage } from 'lib/functions';
import { silent } from 'lib/logins';
import {
  checkSuccessfulNotification,
  clickOnFirstJobName,
  configureEmailSettings,
  createEmailJob,
  JobType,
} from './utils';

describe('Job Page - Job details', () => {
  before(() => {
    silent();
  });

  context('Check Scheduled Email Job on Job Detail Page', () => {
    before(() => {
      configureEmailSettings();
      createEmailJob(JobType.Scheduled);
      goToJobSchedulePage();
      cy.wait(2000);
      clickOnFirstJobName('scheduledJobsTable');
    });

    it('Should display general information about the Scheduled Email Job', () => {
      cy.get(jobs.job_name).should('have.text', 'This is an email job created by Cypress');

      cy.get(jobs.job_schedule_status).should('have.text', 'Active');

      cy.get(jobs.job_interval).should('have.text', 'Scheduled');

      cy.get(jobs.job_type).should('have.text', 'Email');

      cy.get(jobs.job_owner).should('have.text', 'vizapps_admin');
    });

    it('Should open the Email Details Dropdown and display the data', () => {
      cy.get(jobs.details_toggle).click();

      cy.get(jobs.email_subject).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_message).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_template).should('contain.text', 'Default Email Template');

      cy.fixture('configs/email/emails.json').then((email) => {
        cy.get(`[data-test-id="to-${email.email_settings.gmail.address}"]`).should(
          'contain',
          email.email_settings.gmail.address
        );

        cy.get(`[data-test-id="cc-${email.email_settings.yahoo.username}"]`).should(
          'contain',
          email.email_settings.yahoo.username
        );
      });
    });

    it('Should not have any logs in the Previous Runs table yet', () => {
      cy.get('h3').contains('Previous Runs');

      cy.get(jobs.select_job_log).should('not.exist');
    });

    it('Should be able to cancel and delete a running job log', () => {
      cy.get(jobs.job_actions).click();
      cy.get(jobs.job_actions_run).click();

      cy.get(`${jobs.previous_runs_table} tbody tr`)
        .first()
        .should('contain.text', 'Running')
        .find(jobs.cancel_run)
        .click();

      checkSuccessfulNotification('Cancelled run');

      cy.get(`${jobs.previous_runs_table} tbody tr`)
        .first()
        .should('contain.text', 'Cancelled')
        .find(jobs.cancel_run)
        .should('be.disabled');

      cy.get(`${jobs.previous_runs_table} thead th input[type="checkbox"]`).click();
      cy.get(jobs.delete_job_logs)
        .should('not.be.disabled')
        .click();
      cy.get(apps.locators.appdeleteconfirmationtxt)
        .filter(':visible')
        .type('DELETE');
      cy.get(apps.locators.appdeleteconfirmationbtn)
        .filter(':visible')
        .click();

      cy.get(`${jobs.previous_runs_table} tbody tr`).should('have.length', 1);
      cy.get(`${jobs.previous_runs_table} tbody tr`).should('contain.text', 'No data.');
    });

    describe('The Actions dropdown', () => {
      it('Should be able to pause/resume scheduling', () => {
        cy.get(jobs.job_schedule_status).should('have.text', 'Active');

        cy.get(jobs.job_actions).click();
        cy.get(jobs.job_actions_pause_resume).click();

        checkSuccessfulNotification('Job(s) successfully paused/resumed');

        cy.get(jobs.job_schedule_status).should('have.text', 'Paused');

        cy.get(jobs.job_actions).click();
        cy.get(jobs.job_actions_pause_resume).click();

        checkSuccessfulNotification('Job(s) successfully paused/resumed');

        cy.get(jobs.job_schedule_status).should('have.text', 'Active');
      });

      it('Should have a Run Now button which creates a new log', () => {
        cy.get(jobs.job_actions).click();
        cy.get(jobs.job_actions_run).click();

        checkSuccessfulNotification('Successfully started to run job(s)');

        cy.get(`${jobs.previous_runs_table} tbody tr`)
          .first()
          .should('contain.text', 'Running')
          .find(jobs.run_details)
          .click();
        cy.wait(500);
        cy.contains('Run Details');
        cy.get(jobs.job_log_trigger_mode).should('have.text', 'Manually');
        cy.get(jobs.close_run_details).click();
      });

      it('Should have a delete button which deletes the job', () => {
        cy.get(jobs.job_actions).click();
        cy.get(jobs.job_actions_delete).click();

        cy.get(apps.locators.appdeleteconfirmationtxt)
          .filter(':visible')
          .type('DELETE');
        cy.get(apps.locators.appdeleteconfirmationbtn)
          .filter(':visible')
          .click();

        cy.url().should('contain', '/arc/jobs/adminlogs');
      });
    });
  });
});
