import { jobs } from 'fixtures/locators/jobs';
import { notifications } from 'fixtures/locators/notifications';
import { createConnection } from 'lib/connections';
import { locators, openDashboard } from 'lib/dashboard';
import { extractTargetDb } from 'lib/data/util';
import { click_email, goto_arcviz_settings, navigate_to_data } from 'lib/functions';
import { selectDataset } from 'lib/permissions';
import { ensureCheckboxState, saveSiteSettings, siteSettingsSearch } from 'lib/settings';
import { createExtract, runExtract } from 'tests/extracts/util';

export enum JobType {
  Adhoc = 'Adhoc',
  Scheduled = 'Scheduled',
  Threshold = 'Threshold',
}

const currentTime = new Date().getTime();

export const extractJob = {
  srcConnName: Cypress.env('smokeConnectionName'),
  srcDsName: 'Cereals',
  tgtConnName: extractTargetDb.connection_name,
  tgtDbName: extractTargetDb.test_db_name,
  tgtTableName: `cdv_2_${currentTime}`,
  dims: {
    fields: [
      {
        fieldName: 'cereal_name',
        aggAction: null,
        labelToVerify: 'cereal_name',
      },
    ],
  },
  advanced: {
    emailSuccess: true,
    emailFail: true,
    emailTo: 'a@gmail.com',
  },
};

export function checkSuccessfulNotification(message: string) {
  cy.contains(message, { timeout: 30000 } as any);
  cy.get(notifications.notification_message, { timeout: 30000 }).should(($message) => {
    expect($message).to.contain(message);
  });
}

export const clickOnFirstJobName = (tab: 'runsTable' | 'scheduledJobsTable') => {
  cy.get(
    `${
      tab === 'runsTable' ? jobs.job_overview.runs_table : jobs.job_overview.jobs_table
    } > tbody > tr`
  )
    .eq(0)
    .find(jobs.job_overview.job_name)
    .click();
  cy.wait(1000);
};

export const checkFirstJobOwner = (tab: 'runsTable' | 'scheduledJobsTable', ownerName: string) => {
  cy.get(
    `${
      tab === 'runsTable' ? jobs.job_overview.runs_table : jobs.job_overview.jobs_table
    } > tbody > tr`
  )
    .eq(0)
    .find(jobs.job_overview.job_owner)
    .then(($span) => {
      expect($span).to.have.text(ownerName);
    });
};

export function createEmailJob(
  type: JobType,
  generalText: string = 'This is an email job created by Cypress'
) {
  openDashboard('Global Threats');
  click_email();
  cy.fixture('configs/email/emails.json').then((email_configs) => {
    cy.get(jobs.email_modal.form).within(() => {
      if (type === JobType.Scheduled || type === JobType.Threshold) {
        cy.get(jobs.email_modal.email_scheduled).check();
        cy.get(jobs.email_modal.schedule_tab.trigger).click();
        cy.get(jobs.email_modal.schedule_tab.name).clear().type(generalText);
        cy.get(jobs.email_modal.schedule_tab.schedule_group_id).click().type('Daily{enter}');
        cy.get(jobs.email_modal.email_tab.trigger).click();
      }

      cy.get(jobs.email_modal.email_tab.emailTo)
      .click()
      .type(email_configs.email_settings.gmail.address)
      .type('{enter}');

      cy.get(jobs.email_modal.email_tab.emailCc)
      .click()
      .type(email_configs.email_settings.yahoo.username)
      .type('{enter}');

      cy.get(jobs.email_modal.schedule_tab.subject).clear().type(generalText);
      cy.get(jobs.email_modal.schedule_tab.message).clear().type(generalText);
      cy.get(jobs.email_modal.email_tab.attachCsv).check();
      cy.get(jobs.email_modal.email_tab.csvLimit).clear().type('10');

      if (type === JobType.Threshold) {
        cy.get(jobs.email_modal.is_threshold).check({force: true});
        cy.get(jobs.email_modal.threshold_tab.trigger).click();
        cy.get(jobs.email_modal.threshold_tab.create_visual).click();

        cy.get('[name="thresholdTarget.triggerConditions.0.equality"]').select('>');
        cy.get('[name="thresholdTarget.triggerConditions.0.value"]').clear().type('1');
        cy.get('[data-test-id="add-trigger-condition-button"]').click();
        cy.get('[name="thresholdTarget.triggerConditions.1.value"]').clear().type('3');
        cy.get('[data-test-id="remove-trigger-condition-button-1"]').click();
      }

    });

    cy.get(jobs.email_modal.submit).click();

    if (type === JobType.Adhoc) {
      checkSuccessfulNotification('Email will be sent in a few seconds...');
    } else if (type === JobType.Scheduled) {
      checkSuccessfulNotification('Scheduled email successfully created with job ID');
    } else {
      checkSuccessfulNotification('Threshold email successfully created with job ID');
    }
  });
}

export const createExtractJob = () => {
  createConnection(extractTargetDb);

  navigate_to_data();
  selectDataset(extractJob.srcConnName, extractJob.srcDsName);
  createExtract(extractJob);
  runExtract(extractJob.tgtDbName, extractJob.tgtTableName, 'Success');
};

export const configureEmailSettings = () => {
  goto_arcviz_settings();

  const needsSave1 = ensureCheckboxState('Enable "Download as Image/PDF" by default', true);
  const needsSave2 = ensureCheckboxState('Permit emails', true);

  siteSettingsSearch('email');
  cy.get(locators.test_email_btn).click();
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_content).should('contain.text', 'Email Status: 1 sent');

  needsSave1.then((b1) => {
    needsSave2.then((b2) => {
      if (b1 || b2) {
        saveSiteSettings();
      }
    });
  });
};

export const getRandomInt = (min: number, max: number) => {
  const minCeiled = Math.ceil(min);
  const maxFloored = Math.floor(max);
  return Math.floor(Math.random() * (maxFloored - minCeiled) + minCeiled);
};

export const editEmailJob = () => {
  cy.contains('Edit Job Details');

  cy.get(jobs.email_modal.form).within(() => {
    cy.log('Edit job fields');
    cy.get(jobs.email_modal.schedule_tab.trigger).click();
    cy.get(jobs.email_modal.schedule_tab.name).type(' EDITED');
    cy.get(jobs.email_modal.schedule_tab.schedule_group_id).click().type('Weekly{enter}');
    cy.get(jobs.email_modal.email_tab.trigger).click();

    cy.fixture('configs/email/emails.json').then((email_configs) => {
      cy.get(jobs.email_modal.email_tab.emailTo)
      .click()
      .type(email_configs.email_settings.yahoo.username)
      .type('{enter}');

      cy.get(jobs.email_modal.email_tab.emailCc)
      .click()
      .type(email_configs.email_settings.gmail.address)
      .type('{enter}');
    });

    cy.get(jobs.email_modal.schedule_tab.subject).type(' EDITED');
    cy.get(jobs.email_modal.schedule_tab.message).type(' EDITED');
  });

  cy.log('Save changes');
  cy.get(jobs.email_modal.submit).click();
};
