import * as apps from 'lib/apps';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import { silent } from 'lib/logins';
import { CSSselector } from 'locators/common/CSSselector';

function get_time() {
  const a = new Date();
  return a.getTime();
}

const tokenUrl = '/arc/trustedauth/getticket';
const dbname1 = 'TrustedAuth DB' + get_time();
const dbname2 = 'AdminUser DB' + get_time();
let token: string;
const invalidToken = 'Vu1JI9cCRRygWmQPDBrtKQ';
let dashId1: number;
let dashId2: number;
let appId: number;
const APP = 'testApp ' + get_time();
const TAB1 = 'Tab1' + get_time();
const accessDeniedMsg = 'Access denied.';

describe('Trusted Auth.', () => {
  context('ARC-10824: Automate trusted auth', () => {
    before(() => {
      const user = {
        username: 'tadmin',
        password: 'p@ssw0rd'
      };

      silent();

      BackendAPI.users.createOrRecreate(user, [DefaultRoleIds.Analyst]);
      functions.addNewDB(dbname2);
      functions.createNewVisual('samples', 'Cereals');
      dashboard.saveDashboard();
      functions.searchDashboard(dbname2);
      cy.get(`[data-test-id="${dbname2}"]`).then(($el) => {
        dashId1 = $el.data('test-report-id');
      });
      silent('tadmin', 'p@ssw0rd');
      functions.addNewDB(dbname1);
      functions.createNewVisual('samples', 'Cereals');
      dashboard.saveDashboard();
      functions.createNewVisual('samples', 'World Life Expectancy');
      dashboard.saveDashboard();
      functions.createNewVisual('samples', 'NYC Taxicab Rides');
      dashboard.saveDashboard();
      functions.searchDashboard(dbname1);
      cy.get(`[data-test-id="${dbname1}"]`).then(($el) => {
        dashId2 = $el.data('test-report-id');
      });
      functions.navigate_to_visuals();
      apps.createNewApp(APP, TAB1);
      apps.addDashboard('samples', 'Cereals', dbname1);
      apps.saveApp();
      functions.navigate_to_visuals();
      cy.get(`${CSSselector.Visuals_all}`).click();
      cy.get(apps.locators.apps_list).click({ force: true });
      cy.get(`[data-test-id="searchbox"`).type('"' + APP + '"');
      cy.get(`[data-test-id="${APP}"]`).then(($el) => {
        appId = $el.data('test-report-id');
      });
    });

    beforeEach(() => {
      cy.request({
        method: 'POST',
        url: tokenUrl, // baseUrl is prepended to url
        body: 'username=tadmin&trustedusername=tadmin&trustedpassword=p@ssw0rd',
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
        },
      }).then((response) => {
        token = response.body;
      });
    });

    it('1: Access dashboards with valid and invalid token', () => {
      // Accessing recently created dashboard with valid token
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/app/' + dashId2);
      functions.waitForVisualsLoaded();

      cy.get(dashboard.locators.dashboard_title).should('have.text', dbname1);
      cy.get(dashboard.locators.dashboard_subtitle).should('have.text', dbname1 + ' subtitle');
      cy.get('div.dash-viz-wrapper').should('have.length', 3);
      cy.get('div.navbar-collapse.collapse').should('be.visible');

      // Verify single use of a token
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/app/' + dashId2, {
        failOnStatusCode: false,
      });
      cy.get('body').should('contain', 'Forbidden');
    });

    it('2: Verify Session Expiry after the time mentioned in timeout setting', () => {
      // timeout of token is set to be 30 secs in settings_local.py file
      cy.wait(30000);
        cy.visit(`/arc/trustedauth/trusted/${token.trim()}/app/27`, { failOnStatusCode: false });
        cy.get('body').should('contain', 'Forbidden');
    });

    it('3: Accessing sample dashboard with valid token', () => {
      cy.visit(`/arc/trustedauth/trusted/${token.trim()}/app/27`);
      functions.waitForVisualsLoaded();
      cy.get(dashboard.locators.dashboard_title).should('have.text', 'Time & Industry Threat View');
      cy.get('div.navbar-collapse.collapse').should('be.visible');
    });

    it('4: Accesssing dashboard with valid token but invalid id as well as with invalid token and valid id', () => {
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/app/500', {
        failOnStatusCode: false,
      });
      cy.get('body').should('contain', 'Visual 500 does not exist.');

      // Accessing created dashboard with invalid token
      cy.visit('/arc/trustedauth/trusted/' + invalidToken + '/app/' + dashId2, {
        failOnStatusCode: false,
      });
      cy.get('body').should('contain', 'Forbidden');
    });

    it('5: Access user created app with valid and invalid token', () => {
      // Accessing recently created app with valid token
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/appgroup/' + appId);
      functions.waitForVisualsLoaded();

      cy.title().should('eq', APP);
      cy.get('div.navbar-collapse.collapse .arc-link').should('have.text', ' ' + TAB1 + ' ');
      cy.get('.navbar a.navbar-brand').click();
      cy.title().should('eq', APP);
      cy.get('div.navbar-collapse.collapse .arc-link').should('have.text', ' ' + TAB1 + ' ');

      // Accessing created app with invalid token
      cy.visit('/arc/trustedauth/trusted/' + invalidToken + '/appgroup/' + appId, {
        failOnStatusCode: false,
      });
      cy.get('body').should('contain', 'Forbidden');
    });

    it('6: Access sample app with valid token', () => {
      // Accessing sample app with valid token
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/appgroup/6');
      functions.waitForVisualsLoaded();
      cy.wait(2000);
      cy.get('div.navbar-collapse.collapse .arc-link').should('have.length', 3);
      cy.get('.navbar a.navbar-brand').click();
      cy.get('div.navbar-collapse.collapse .arc-link').should('have.length', 3);
    });

    it('7: Access home url with token', () => {
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/apps/home', { failOnStatusCode: false });
      cy.get('body').contains('Page not found', { matchCase: false });
    
      const redirectPath = encodeURIComponent('/arc/apps/home');
      cy.visit(`/arc/trustedauth/trusted/${token.trim()}/redirect?next=${redirectPath}`, { failOnStatusCode: false });
    
      cy.get('body').should('not.contain', 'Page not found', { matchCase: false });
      cy.get('body').should('not.contain', 'Forbidden', { matchCase: false });
      cy.get('body').should('not.contain', accessDeniedMsg, { matchCase: false });
    
      cy.get('div.navbar-collapse.collapse').should('be.visible');
    });

    it("8: Access dashboard of admin user's private workspace with token", () => {
      cy.visit('/arc/trustedauth/trusted/' + token.trim() + '/app/' + dashId1, {
        failOnStatusCode: false,
      });
      cy.get('#main .alert-more-link').click();
      cy.get('body')
        .should('contain', 'Visual ' + dashId1 + ' does not exist.')
        .should('contain', 'You do not have permissions to view dashboard ' + dashId1);
    });

    it('9: Access trustedauth urls', () => {
      cy.visit(`/arc/trustedauth/trusted/${token.trim()}/app/27`);
        cy.visit('/arc/jobs/adminlogs');
        cy.wait(2000);
        cy.get(`[data-test-id="jobsHeader"`).should('contain', 'Jobs');
        cy.get('a#runsTab').should('have.class', 'active');
        cy.get('body').should('not.contain', accessDeniedMsg);

        cy.visit('/arc/jobs/customemails', { failOnStatusCode: false });
        cy.get('body').should('contain', 'You do not have permission to update email templates');
    });
  });
});
