import { Interception } from 'cypress/types/net-stubbing';
import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import * as util from '../util';

/* 
  I keep this here, to able to reuse in case we have to reproduce the visual.
  This is a clear definition of the dashboard however manually creating it is slow.
  const data = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'Cereals',
  filterName1: 'cereal_name',
  filterName2: 'manufacturer_code',
  filterValue1: '100%_Bran',
  filterValue2: ['Q', 'N'],
  };
  util.createDBWithTwoSheetsAndFilters(data);
*/

let alias: string;

interface ShortUrlPostData {
  dashboardId: number;
  isCreatedInEmail: boolean;
  parameters: string;
  slug: string;
  type: 'dashboard' | 'app';
}

const waitForUrlAliasSectionToBeShownAndFilled = () => {
  cy.get('#url-shortener input.short-url-slug').should('be.visible');
  cy.get('#url-shortener input.short-url-slug')
    .invoke('val')
    .should('not.be.empty');
};

const pressGenerateAliasButton = () => {
  cy.get('#generate-url-alias').click();
};

const unCheckIncludeParamCheckbox = () => {
  cy.get('#includePageParametersShortUrl')
    .scrollIntoView()
    .uncheck();
};

const checkIncludeParamCheckbox = () => {
  cy.get('#includePageParametersShortUrl')
    .scrollIntoView()
    .check();
};

const waitUntilSaveIsEnabledAndPressIt = () => {
  cy.get('#save-url-alias').should('be.enabled');
  cy.get('#save-url-alias').click();
};

const testFiltersAreNotIncludedInTheRequest = (response: Interception) =>
  expect((response.request.body as ShortUrlPostData).parameters).to.equal('');
const testFiltersAreIncludedInTheRequest = (response: Interception) =>
  expect((response.request.body as ShortUrlPostData).parameters.length).to.greaterThan(0);

describe('Dashboard - URL shortener test', () => {
  before(() => {
    logins.silent();
  });

  it('Test 1 - Create dashboard with 2 sheet and filters', () => {
    BackendAPI.dashboards
      .import({
        fileName: 'dashboard_with_filter_prettified.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => cy.visit(`/arc/apps/builder/${dashboard.id}`));
  });

  it('Test 2 - Generate alias without parameters and delete it', () => {
    util.openURLModal();
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    unCheckIncludeParamCheckbox();
    cy.intercept('POST', '/arc/short/').as('postShortUrl');
    waitUntilSaveIsEnabledAndPressIt();
    cy.wait('@postShortUrl').then((response) => {
      // Test no filter param is sent
      testFiltersAreNotIncludedInTheRequest(response);
    });

    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });

    util.closeURLModal();
  });

  it('Test 3 - Generate alias with parameters and delete it', () => {
    util.openURLModal();
    cy.get('.nav-tab-url').click();
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    checkIncludeParamCheckbox();
    cy.intercept('POST', '/arc/short/').as('postShortUrl');
    waitUntilSaveIsEnabledAndPressIt();
    cy.wait('@postShortUrl').then((response) => {
      // Test no filter param is sent
      testFiltersAreIncludedInTheRequest(response);
    });
    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });

    util.closeURLModal();
  });

  it('Test 4 - Generate specific alias without parameters, check the Copy to Clipboard button and delete the alias', () => {
    util.openURLModal();

    util.checkDefaultCopyToURLButton();
    cy.intercept('POST', '/arc/short/').as('postShortUrl');
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    cy.get('.short-url-slug')
      .click()
      .clear()
      .type('viz-test-url-dashboard');
    unCheckIncludeParamCheckbox();
    cy.get('.short.url-copy-btn').should('have.class', 'disabled');
    waitUntilSaveIsEnabledAndPressIt();
    cy.wait('@postShortUrl').then((response) => {
      // Test no filter param is sent
      testFiltersAreNotIncludedInTheRequest(response);
    });
    cy.get('.short.url-copy-btn').should('not.have.class', 'disabled');

    util.checkGeneratedCopyToClipboard();

    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });

    util.closeURLModal();
  });
});

describe('App - URL shortener test', () => {
  before(() => {
    logins.silent();
  });

  it('Test 1 - Create app', () => {
    apps.createNewApp();
    apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
    apps.addMenuTab();
    apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
    apps.saveApp();

    apps.launchApp('Re-ordering App');
  });

  it('Test 2- Generate alias without parameters and delete it', () => {
    util.openURLModal();
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    unCheckIncludeParamCheckbox();
    cy.intercept('POST', '/arc/short/').as('postShortUrl');
    waitUntilSaveIsEnabledAndPressIt();
    cy.wait('@postShortUrl').then((response) => {
      // Test no filter param is sent
      testFiltersAreNotIncludedInTheRequest(response);
    });
    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });
  });

  it('Test 3 - Generate alias with parameters and delete it', () => {
    cy.get('.nav-tab-url').click();
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    checkIncludeParamCheckbox();
    cy.intercept('POST', '/arc/short/').as('postShortUrl');
    waitUntilSaveIsEnabledAndPressIt();

    cy.wait('@postShortUrl').then((response) => {
      // Test no filter param is sent
      testFiltersAreIncludedInTheRequest(response);
    });
    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });
  });

  it('Test 4 - Change the autogenerated alias and delete it', () => {
    cy.get('.nav-tab-url').click();
    util.checkDefaultCopyToURLButton();
    pressGenerateAliasButton();
    waitForUrlAliasSectionToBeShownAndFilled();
    cy.get('.short-url-slug')
      .click()
      .clear()
      .type('viz-test-url-app');
    checkIncludeParamCheckbox();
    cy.get('.short.url-copy-btn').should('have.class', 'disabled');
    waitUntilSaveIsEnabledAndPressIt();
    cy.get('.short.url-copy-btn').should('not.have.class', 'disabled');

    util.checkGeneratedCopyToClipboard();

    cy.get('.short-url-slug').then(($input) => {
      alias = $input.val() as string;
      util.existingURLListControlsCheck(alias);
    });

    util.closeURLModal();
  });
});
