import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';
import * as user_groups from 'lib/users_groups';
import { locators } from 'lib/users_groups/locators';

// there are intermittent fails caused because somehow the `viz_guest_group` role
// have been kept in the app from some previous test suits,
// so in the before hook we make sure to start with a clean environment
const groupList = ['grp1', 'grp2', 'grp3', 'viz_guest_group'];
const usersList = ['analyst', 'visualconsumer', 'user1', 'user2'];

describe('Users and Groups Management.', () => {
  before(() => {
    logins.silent();
    functions.bulkObjectDeleteApi('users', [], usersList, 'username');
    functions.bulkObjectDeleteApi('groups', [], groupList, 'name');
  });

  context('ARC-11314: Automate User/Group management - Smoke', () => {
    it('1: Create Users and Groups and assign roles to them ', () => {
      functions.navigate_to_home();
      cy.fixture('users.json').then((data) => {
        user_groups.createNewUser(data.analyst.username, data.analyst.password, ['Analyst']);
        user_groups.createNewUser(data.visualconsumer.username, data.visualconsumer.password, [
          'Visual Consumer',
        ]);
        user_groups.createNewUser(data.user1.username, data.user1.password, []);
        user_groups.createNewUser(data.user2.username, data.user2.password, []);
        user_groups.createNewGroup(groupList[0], [
          data.analyst.username,
          data.visualconsumer.username,
        ]);
        user_groups.createNewGroup(
          groupList[1],
          [data.user1.username, data.user2.username],
          ['Analyst', 'System Admin']
        );
        user_groups.createGrpFromUserModal(groupList[2]);
      });
    });

    it('2: Verify created group names, roles and assigned users', () => {
      cy.get(locators.group_tab).click();
      cy.fixture('users.json').then((data) => {
        // Verify group names
        for (let index = 0; index < 3; index++) {
          cy.get(locators.groups.name).should('contain', groupList[index]);
        }

        // Verify group members count
        const membersCount = ['2', '2', '0'];
        for (let index = 0; index < 3; index++) {
          cy.get(locators.groups.name)
            .contains(groupList[index])
            .parents('tr')
            .find('.badge')
            .should('have.text', membersCount[index]);
        }

        // Verify group member's names
        cy.get(locators.groups.name)
          .contains(groupList[0])
          .parents('tr')
          .find('.gp-users-list-wrapper')
          .should('contain', data.analyst.username);
        cy.get(locators.groups.name)
          .contains(groupList[1])
          .parents('tr')
          .find('.gp-users-list-wrapper')
          .should('contain', data.user1.username)
          .should('contain', data.user2.username);
        cy.get(locators.groups.name)
          .contains(groupList[2])
          .parents('tr')
          .find('.gp-users-list-wrapper > ul')
          .should('have.text', '\n                  \n              ');

        // Verify group member's roles
        for (let index = 0; index < 2; index++) {
          cy.get(locators.groups.name)
            .contains(groupList[1])
            .parents('tr')
            .find('.group-table-roles')
            .should('contain', 'System Admin,Analyst');
        }
        cy.get(locators.groups.name)
          .contains(groupList[2])
          .parents('tr')
          .find('.group-table-roles')
          .should('have.text', '');
      });
    });

    it('3: Verify created user names, roles and assigned groups', () => {
      cy.fixture('users.json').then((data) => {
        cy.get(locators.users_tab).click();
        // Verify user names
        for (let index = 0; index < 4; index++) {
          functions.searchForUser(usersList[index]);
          cy.get(locators.users.name).should('contain', usersList[index]);
        }

        // Verify user groups
        const grpList = ['grp1', 'grp1', 'grp2', 'grp2'];
        for (let index = 0; index < 4; index++) {
          functions.searchForUser(usersList[index]);
          cy.get(locators.users.name)
            .contains(usersList[index])
            .parents('tr')
            .find('.user-table-groups')
            .should('have.text', grpList[index]);
        }

        // Verify user roles
        const userRoles = [
          'Analyst',
          'Visual Consumer',
          'System Admin, Analyst',
          'System Admin, Analyst',
        ];
        for (let index = 0; index < 4; index++) {
          functions.searchForUser(usersList[index]);
          cy.get(locators.users.name)
            .contains(usersList[index])
            .parents('tr')
            .find('.user-table-roles')
            .should('have.text', userRoles[index]);
        }
      });
    });

    it('4: Make the user inactive and verify it', () => {
      cy.fixture('users.json').then((data) => {
        user_groups.inactiveUser(usersList[1]);
        // wait to change the status of user
        cy.wait(2000);
        functions.searchForUser(usersList[1]);
        cy.get(locators.users.name)
          .contains(usersList[1])
          .parents('tr')
          .find('.user-table-status')
          .should('have.text', 'I');

        logins.logout();
        cy.visit('/');
        cy.get('#username').type(data.visualconsumer.username);
        cy.get('#password').type(data.visualconsumer.password);
        cy.get(`[data-test-id='submit']`).click({ force: true });
        cy.get('div.invalid-feedback').should('have.text', 'Your account is disabled.');
      });
    });

    it('5: Delete the user and verify it', () => {
      logins.silent();
      cy.visit('/');
      cy.fixture('users.json').then((data) => {
        functions.deleteUser(data.visualconsumer.username);
      });
      cy.get(locators.users.name).should('not.contain', usersList[1]);
    });

    it('6: ARC-8582: Verify last login date for the user who never logged in', () => {
      functions.searchForUser('analyst');
      cy.get(locators.users.name)
        .contains('analyst')
        .parents('tr')
        .find('.fa-pencil')
        .click();
      cy.get(locators.users.last_login).should('have.text', 'Never');
      functions.modalClick(locators.users.cancel);
    });
  });
});
