import * as arctest from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import {
  Dashboard,
  ShelfTypes,
  SortOptions,
  VisualType,
} from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const data = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'Restaurant Inspection SF',
};

const tableBody = '.dataTables_scrollBody tbody tr';
const crosstabBody = '.viz-crosstab-table .slick-row';

const crosstabDashboard: Dashboard = {
  title: 'crosstab_visual_expansion_title',
  subTitle: 'crosstab_visual_expansion_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'crosstabvisual',
          connection_name: data.connectionName,
          dataset_name: data.datasetName,
          visual_type: '.chart-icon-crosstab',
          visual_title: 'simple_crosstab',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'inspection_type',
                field_shelf: ShelfTypes.X_DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'business_name',
                field_shelf: ShelfTypes.Y_DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'business_address',
                field_shelf: ShelfTypes.Y_DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'business_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [
                    '100% Dessert Cafe',
                    '1300 on Fillmore',
                    '17th & Noe Market',
                    '18 Reasons',
                    '18th Street Commissary',
                  ],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

const tableDashboard: Dashboard = {
  title: 'table_visual_expansion_title',
  subTitle: 'table_visual_expansion_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'tablevisual',
          connection_name: data.connectionName,
          dataset_name: data.datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'inspection_type',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'business_name',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'business_address',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'business_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [
                    '100% Dessert Cafe',
                    '1300 on Fillmore',
                    '17th & Noe Market',
                    '18 Reasons',
                    '18th Street Commissary',
                  ],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

function getCellSelector({
  visualType,
  rowNumber,
  columnNumber,
  iconType,
}: {
  visualType: 'table' | 'crosstab';
  rowNumber?: number;
  columnNumber?: number;
  iconType?: 'plus' | 'minus';
}) {
  let cell = '';
  let body = '';
  let row = '';
  let column = '';
  const icon = iconType ? `i.fa.fa-${iconType}-square-o` : '';
  if (visualType === 'table') {
    body = '.dataTables_scrollBody tbody';
    row = rowNumber ? `tr:nth-child(${rowNumber})` : 'tr';
    column = columnNumber ? `td:nth-child(${columnNumber})` : 'td';
  } else {
    body = '.viz-crosstab-table';
    row = rowNumber ? `.slick-row:nth-child(${rowNumber})` : '.slick-row';
    column = columnNumber ? `.slick-cell:nth-child(${columnNumber})` : '.slick-cell';
  }

  cell = `${body} ${row} ${column} ${icon}`;

  return cell;
}

describe('Cross tabulation visual expansion', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
    dashboards.createDashboard(crosstabDashboard);
  });

  it('Enable expansion', () => {
    arctest.screen.builder.modal.setExpansion('business_name');
    arctest.screen.builder.modal.refreshVisual();
    // once the expansion is enabled, only 4 rows should be visible
    cy.get(`${crosstabBody}`).should('have.length', 4);
    // click on the 4th row 1st column cell's + icon
    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 4, columnNumber: 1, iconType: 'plus' })
    ).click();
    // open the collapsed columned - should have 9 rows
    cy.get(`${crosstabBody}`).should('have.length', 9);
    // the + icon should be - after expansion
    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 4, columnNumber: 1, iconType: 'minus' })
    ).should('be.visible');
    // checking the displayed values are correct
    cy.get(getCellSelector({ visualType: 'crosstab', rowNumber: 5, columnNumber: 1 })).then(
      ($e) => {
        assert.equal($e.text(), '100% Dessert Cafe');
      }
    );

    cy.get(getCellSelector({ visualType: 'crosstab', rowNumber: 9, columnNumber: 1 })).then(
      ($e) => {
        assert.equal($e.text(), '18th Street Commissary');
      }
    );

    // click on the 4th row 2st column cell's + icon
    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 5, columnNumber: 2, iconType: 'plus' })
    ).click();
    // open the collapsed row - should have 10 rows
    cy.get(`${crosstabBody}`).should('have.length', 10);
    // checking the displayed values are correct
    cy.get(getCellSelector({ visualType: 'crosstab', rowNumber: 6, columnNumber: 2 })).then(
      ($e) => {
        assert.equal($e.text(), '2512 Clement St');
      }
    );

    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 10, columnNumber: 2, iconType: 'plus' })
    ).click();
    // open the collapsed row - should have 11 rows
    cy.get(`${crosstabBody}`).should('have.length', 11);

    cy.get(getCellSelector({ visualType: 'crosstab', rowNumber: 11, columnNumber: 2 })).then(
      ($e) => {
        assert.equal($e.text(), '3334 18th St');
      }
    );

    // collapse the 2nd column
    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 4, columnNumber: 2, iconType: 'minus' })
    ).click();
    // open the collapsed row - should have 9 rows
    cy.get(`${crosstabBody}`).should('have.length', 9);

    // collapse the 1st column
    cy.get(
      getCellSelector({ visualType: 'crosstab', rowNumber: 4, columnNumber: 1, iconType: 'minus' })
    ).click();
    // open the collapsed row - should have 9 rows
    cy.get(`${crosstabBody}`).should('have.length', 4);
  });
});

describe('Table visual expansion', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
    dashboards.createDashboard(tableDashboard);
  });

  it('Enable expansion', () => {
    arctest.screen.builder.modal.setExpansion('inspection_type');
    arctest.screen.builder.modal.refreshVisual();

    // once the expansion is enabled, only 1 row should be visible
    cy.get(`${tableBody}`).should('have.length', 1);

    // click on the 1st row 1st column cell's + icon
    cy.get(getCellSelector({ visualType: 'table', columnNumber: 1, iconType: 'plus' })).click();

    // once the expansion icon clicked, 4 rows should be visible
    cy.get(`${tableBody}`).should('have.length', 4);

    // the + icon should be - after expansion
    cy.get(
      getCellSelector({ visualType: 'table', rowNumber: 1, columnNumber: 1, iconType: 'minus' })
    ).should('be.visible');

    // checking the displayed values are correct
    cy.get(getCellSelector({ visualType: 'table', rowNumber: 2, columnNumber: 1 })).then(($e) => {
      assert.equal($e.text(), 'Complaint');
    });

    cy.get(getCellSelector({ visualType: 'table', rowNumber: 4, columnNumber: 1 })).then(($e) => {
      assert.equal($e.text(), 'Routine - Unscheduled');
    });

    // click on the 2nd row 2nd column cell's + icon
    cy.get(
      getCellSelector({ visualType: 'table', rowNumber: 2, columnNumber: 2, iconType: 'plus' })
    ).click();

    cy.get(`${tableBody}`).should('have.length', 5);

    // click on the 2nd row 5th column cell's + icon
    cy.get(
      getCellSelector({ visualType: 'table', rowNumber: 5, columnNumber: 2, iconType: 'plus' })
    ).click();

    cy.get(`${tableBody}`).should('have.length', 10);

    // checking the displayed values are correct
    cy.get(getCellSelector({ visualType: 'table', rowNumber: 3, columnNumber: 2 })).then(($e) => {
      assert.equal($e.text(), '1300 on Fillmore');
    });
    cy.get(getCellSelector({ visualType: 'table', rowNumber: 10, columnNumber: 2 })).then(($e) => {
      assert.equal($e.text(), '18th Street Commissary');
    });

    // collapse the 2nd column
    cy.get(
      getCellSelector({ visualType: 'table', rowNumber: 1, columnNumber: 2, iconType: 'minus' })
    ).click();

    cy.get(`${tableBody}`).should('have.length', 4);

    // collapse the 2nd column
    cy.get(
      getCellSelector({ visualType: 'table', rowNumber: 1, columnNumber: 1, iconType: 'minus' })
    ).click();

    cy.get(`${tableBody}`).should('have.length', 1);
  });
});
