import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';

const allReportCountSelector = '[data-test-id="app-group-all-report-count"]';
const favoritesCountSelector = '[data-test-id="favorites-app-group-report-count"]';
const publicCountSelector = '[data-test-id="Public-report-count"]';

let numberOfReportsOnFirstLoad: number;
let numberOfFavoritesOnFirstLoad: number;
let numberOfPublicReportsOnFirstLoad: number;

describe('Visuals page delete report', () => {
  before(() => {
    logins.silent();

    cy.visit('/arc/apps/apps');

    cy.get(allReportCountSelector).should('exist');
    cy.get(`${allReportCountSelector} svg`).should('not.exist');

    cy.get(allReportCountSelector)
      .invoke('text')
      .then((count) => {
        numberOfReportsOnFirstLoad = +count;
      });

    cy.get(favoritesCountSelector)
      .invoke('text')
      .then((count) => {
        numberOfFavoritesOnFirstLoad = +count;
      });

    cy.get(publicCountSelector)
      .invoke('text')
      .then((count) => {
        numberOfPublicReportsOnFirstLoad = +count;
      });
  });

  it('should delete a dashboard', () => {
    BackendAPI.dashboards
      .import({
        fileName: 'basic_dashboard.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        cy.visit('/arc/apps/apps');

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 1);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 1);

        cy.get('[data-test-id="Public"]').click();
        cy.get('[data-test-id="workspace-title"]').click();
        cy.get('[data-test-id="reports-loading-indicator"]').should('not.exist');

        cy.get(`[data-test-report-id="dashboard-${dashboard.id}"]`).scrollIntoView();
        cy.get(
          `[data-test-report-id="dashboard-${dashboard.id}"] [data-test-id="report-tile-more-actions-icon"]`
        ).click();
        cy.get(`[data-test-id="delete-report-${dashboard.id}"]`).click();

        cy.get('[data-test-id="delete-confirmation-text"]').type('DELETE');
        cy.get('[data-test-id="delete-confirmation-yes"]').click();

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad);
      });
  });

  it('should delete a favorite dashboard', () => {
    BackendAPI.dashboards
      .import({
        fileName: 'basic_dashboard.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        cy.visit(`/arc/apps/app/${dashboard.id}`);

        cy.get('[data-test-id="more-btn"]').click();
        cy.contains('Add to Favorites').click();

        cy.visit('/arc/apps/apps');
        cy.wait(5000);
        cy.reload(true);

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 1);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad + 1);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 1);

        cy.get('[data-test-id="Public"]').click();
        cy.get('[data-test-id="workspace-title"]').click();
        cy.get('[data-test-id="reports-loading-indicator"]').should('not.exist');

        cy.get(`[data-test-report-id="dashboard-${dashboard.id}"]`).scrollIntoView();
        cy.get(
          `[data-test-report-id="dashboard-${dashboard.id}"] [data-test-id="report-tile-more-actions-icon"]`
        ).click();
        cy.get(`[data-test-id="delete-report-${dashboard.id}"]`).click();

        cy.get('[data-test-id="delete-confirmation-text"]').type('DELETE');
        cy.get('[data-test-id="delete-confirmation-yes"]').click();

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad);
      });
  });

  it('should handle error when deleting a dashboard', () => {
    BackendAPI.dashboards
      .import({
        fileName: 'basic_dashboard.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        cy.visit('/arc/apps/apps');
        cy.wait(5000);
        cy.reload(true);

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 1);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 1);

        cy.get('[data-test-id="Public"]').click();
        cy.get('[data-test-id="workspace-title"]').click();
        cy.get('[data-test-id="reports-loading-indicator"]').should('not.exist');

        cy.get(`[data-test-report-id="dashboard-${dashboard.id}"]`).scrollIntoView();
        cy.get(
          `[data-test-report-id="dashboard-${dashboard.id}"] [data-test-id="report-tile-more-actions-icon"]`
        ).click();
        cy.get(`[data-test-id="delete-report-${dashboard.id}"]`).click();

        cy.get('[data-test-id="delete-confirmation-text"]').type('DELETE');
        cy.intercept(
          { url: `/arc/reports/report/${dashboard.id}`, method: 'DELETE', times: 1 },
          (req) => {
            req.reply({
              statusCode: 500,
            });
          }
        ).as('deleteReport');
        cy.get('[data-test-id="delete-confirmation-yes"]').click();

        cy.wait('@deleteReport');

        cy.get('[data-test-id="notification-content"]').contains('Failed to delete dashboard');

        cy.get('[data-test-id="delete-confirmation-yes"]').should('be.visible');

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 1);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 1);
      });
  });

  it('should delete an app', () => {
    BackendAPI.apps
      .create({
        name: 'testing this app',
        workspaceId: 1,
      })
      .then((id) => {
        cy.visit('/arc/apps/apps');
        cy.wait(5000);
        cy.reload(true);

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 2);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 2);

        cy.get('[data-test-id="Public"]').click();
        cy.get('[data-test-id="workspace-title"]').click();
        cy.get('[data-test-id="reports-loading-indicator"]').should('not.exist');

        cy.get(`[data-test-report-id="app-${id}"]`).scrollIntoView();
        cy.get(
          `[data-test-report-id="app-${id}"] [data-test-id="report-tile-more-actions-icon"]`
        ).click();
        cy.get(`[data-test-id="delete-report-${id}"]`).click();

        cy.get('[data-test-id="delete-confirmation-text"]').type('DELETE');
        cy.get('[data-test-id="delete-confirmation-yes"]').click();

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 1);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 1);
      });
  });

  it('should handle error when deleting an app', () => {
    BackendAPI.apps
      .create({
        name: 'testing this app',
        workspaceId: 1,
      })
      .then((id) => {
        cy.visit('/arc/apps/apps');
        cy.wait(5000);
        cy.reload(true);

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 2);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 2);

        cy.get('[data-test-id="Public"]').click();
        cy.get('[data-test-id="workspace-title"]').click();
        cy.get('[data-test-id="reports-loading-indicator"]').should('not.exist');

        cy.get(`[data-test-report-id="app-${id}"]`).scrollIntoView();
        cy.get(
          `[data-test-report-id="app-${id}"] [data-test-id="report-tile-more-actions-icon"]`
        ).click();
        cy.get(`[data-test-id="delete-report-${id}"]`).click();

        cy.get('[data-test-id="delete-confirmation-text"]').type('DELETE');
        cy.intercept({ url: `/arc/reports/appgroup/${id}`, method: 'DELETE', times: 1 }, (req) => {
          req.reply({
            statusCode: 500,
          });
        }).as('deleteReport');
        cy.get('[data-test-id="delete-confirmation-yes"]').click();

        cy.wait('@deleteReport');

        cy.get('[data-test-id="notification-content"]').contains('Failed to delete app');

        cy.get('[data-test-id="delete-confirmation-yes"]').should('be.visible');

        cy.get(allReportCountSelector).contains(numberOfReportsOnFirstLoad + 2);
        cy.get(favoritesCountSelector).contains(numberOfFavoritesOnFirstLoad);
        cy.get(publicCountSelector).contains(numberOfPublicReportsOnFirstLoad + 2);
      });
  });
});
