import * as dashboards from 'lib/dashboard';
import * as visual from 'lib/visual';
import { customColors } from '../../../../fixtures/locators/customColors';

const {
  reverseColorPaletteCheckbox,
  opacitySliderInput,
  fixDimensionColorsCheckbox,
} = visual.locators.style.colors;

export const testFixedDimensionColorsAreFromTheCorrectPalette = () => {
  cy.get(customColors.stylePanel.colorSettingSection.activeColorPaletteColorItems).then(
    ($colorItems) => {
      const colors: string[] = [];

      for (const color of $colorItems) {
        colors.push(color.style.backgroundColor);
      }

      cy.get(customColors.stylePanel.colorSettingSection.fixedDimensionColorMapItem).then(
        ($fixedMap) => {
          for (const color of $fixedMap) {
            expect(colors).to.includes(color.style.backgroundColor);
          }
        }
      );
    }
  );
};

export const testTheDesiredColorpaletteIsSelected = (desiredColorpaletteIndex: number) => {
  cy.get(
    customColors.stylePanel.colorSettingSection.colorpaletteSelectionNthItem(
      desiredColorpaletteIndex
    )
  ).should('have.class', 'settings-color-selected');
};

export const testMeasureRangePresence = (isVisible: boolean) => {
  cy.get(customColors.stylePanel.colorSettingSection.measureRangeColorSetting).should(
    isVisible ? 'be.visible' : 'not.exist'
  );
  cy.get(customColors.stylePanel.colorSettingSection.addRangeColorIntervalButton).should(
    isVisible ? 'be.visible' : 'not.exist'
  );
};

export const testFixedDimensionMapSettingPresence = (
  isVisible: boolean,
  isChecked: boolean = false
) => {
  cy.get(fixDimensionColorsCheckbox).should(isVisible ? 'be.visible' : 'not.exist');
  cy.get(customColors.stylePanel.colorSettingSection.fixedDimensionToColorMapList).should(
    isVisible ? `be.${isChecked ? '' : 'not.'}visible` : 'not.exist'
  );
  if (isVisible) {
    cy.get(fixDimensionColorsCheckbox).should(`be${isChecked ? '' : '.not'}.checked`);
  }

  if (isChecked) {
    testFixedDimensionColorsAreFromTheCorrectPalette();
  }
};

export const testColorSettings = ({
  opacityValue = 80,
  fixedDimensionCheckboxSelected,
  activeColorPaletteIndex = 1,
  reversedCheckboxValue = false,
  aggregate = false,
}: {
  aggregate: boolean;
  opacityValue?: number;
  fixedDimensionCheckboxSelected?: boolean;
  activeColorPaletteIndex?: number;
  reversedCheckboxValue?: boolean;
}) => {
  cy.get(reverseColorPaletteCheckbox).should(`be${reversedCheckboxValue ? '' : '.not'}.checked`);

  testTheDesiredColorpaletteIsSelected(activeColorPaletteIndex);
  cy.get(opacitySliderInput).should('have.value', opacityValue);

  if (aggregate) {
    testMeasureRangePresence(true);
    testFixedDimensionMapSettingPresence(false);
  } else {
    testFixedDimensionMapSettingPresence(true, fixedDimensionCheckboxSelected);
    testMeasureRangePresence(false);
  }
};

export const openBuildTab = () => {
  dashboards.chooseVisualTab('data');
};

export const openPropertiesFieldOfItem = (fieldItemName: string, shelfId: string) => {
  cy.get(`#app-shelves ${shelfId} .selection_field[title='${fieldItemName}']`).click();
};

export const enableBackgroundColorPropertiesOfField = (fieldItemName: string, shelfId: string) => {
  // Pre steps
  openBuildTab();
  openPropertiesFieldOfItem(fieldItemName, shelfId);
  // Main steps
  cy.get(customColors.fieldPropertiesPanel.backgroundColorSubmenu).click();
  cy.get(customColors.fieldPropertiesPanel.enableColorButton).click();
  cy.get(customColors.fieldPropertiesPanel.backgroundColorSubmenu).should('be.visible');
};

export const openColorSetting = () => {
  cy.get(customColors.fieldPropertiesPanel.backgroundColorPalette).click();
  cy.get(customColors.fieldPropertiesPanel.colorSettingPopOver.panel).should('be.visible');
};

export const setCheckboxState = ({
  checkboxSelector,
  enabled,
}: {
  checkboxSelector: string;
  enabled: boolean;
}) => {
  if (enabled) {
    cy.get(checkboxSelector).check();
  } else {
    cy.get(checkboxSelector).uncheck();
  }
};

export const reloadFixedColors = () => {
  setCheckboxState({ checkboxSelector: fixDimensionColorsCheckbox, enabled: false });
  setCheckboxState({ checkboxSelector: fixDimensionColorsCheckbox, enabled: true });
};

export const getAggregateTitle = (originalMeasureFieldName: string) => {
  return `sum(${originalMeasureFieldName})`;
};

interface Boundary {
  lower?: string;
  upper?: string;
}
export const setBoundaryOnLastAddedRange = ({
  lower: lowerBoundary,
  upper: upperBoundary,
}: Boundary) => {
  if (lowerBoundary) {
    cy.get(
      `${customColors.stylePanel.colorSettingSection.measureRangeColorSetting} .measure-value-list .measure-value:last input.range-left`
    ).type(lowerBoundary);
  }

  if (upperBoundary) {
    cy.get(
      `${customColors.stylePanel.colorSettingSection.measureRangeColorSetting} .measure-value-list:last .measure-value:last input.range-right`
    ).type(upperBoundary);
  }
};

export const setColorOfMeasureRangeOnLastAddedRange = (nthSingleColorOfPalette: number) => {
  cy.get(`${customColors.stylePanel.colorSettingSection.measureRangeColorSelectorButton}`).click();
  cy.get(customColors.stylePanel.colorSettingSection.rangeSettingColorPalettePicker).should(
    'be.visible'
  );
  cy.get(
    `${customColors.stylePanel.colorSettingSection.rangeSettingColorPalettePicker} .settings-swatch:nth-child(${nthSingleColorOfPalette})`
  ).click();
};

export const addNewColorRage = (boundary: Boundary, colorIndex: number) => {
  cy.get(customColors.stylePanel.colorSettingSection.addRangeColorIntervalButton).click();
  setBoundaryOnLastAddedRange(boundary);
  setColorOfMeasureRangeOnLastAddedRange(colorIndex);
};

// During development the correct mapping was ruined
// -> test that after colorpalette change the mapping is changing
// (Note: dimension maps recolored on refreshing manually the fixed setting (uncheck and recheck it))
export const testFixedDimensionMapHasColorsFromActivePalette = () => {
  reloadFixedColors();
  cy.get(customColors.stylePanel.colorSettingSection.fixedDimensionToColorMapList).should(
    'be.visible'
  );
  testFixedDimensionColorsAreFromTheCorrectPalette();
};

export const selectTheBlackGradedColorPalette = () => {
  const indexOfBlackGradedPalette = 4;
  cy.get(
    customColors.stylePanel.colorSettingSection.colorpaletteSelectionNthItem(
      indexOfBlackGradedPalette
    )
  ).click();
  testTheDesiredColorpaletteIsSelected(indexOfBlackGradedPalette);
};
