/* This script contains verification of following tests
  https://arcadiadata.atlassian.net/browse/ARC-10083
*/
import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as dsPermissions from 'lib/permissions';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

const connectionName = 'samples';
const databaseName = 'main';
const sourceTable = 'trips';
const datasetName = 'e2eTestTripsJoinSet';
const targetTable = 'trips_detail';

const dashboard: Dashboard = {
  title: 'simple_boxplot_visual_dashboard',
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Cereal sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplot visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'pickup_hour',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'trip_time_in_secs',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};
const dashboardJoin: Dashboard = {
  title: 'boxplot_join_dashboard',
  subTitle: 'boxplot_join_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Boxplot sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplot join visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'join_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'pickup_hour',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'trip_time_in_secs',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  before(() => {
    logins.silent();
    functions.navigate_to_home();
  });

  beforeEach(() => {
    functions.deleteDatasetFromConnection(connectionName, datasetName);
  });

  it('ARC-10083 1: Create boxplot visual', () => {
    functions.createDatasetFromConnection(connectionName, databaseName, sourceTable, datasetName);
    // create dashboard with box plot visual
    dashboards.createDashboard(dashboard);
    cy.reload();
    // verify that x-axis and y-axis of visual contains desired labels and data in visual edit view
    visual.editSpecificVisual(1);
    cy.wait('@visualGet');
    cy.get(CSSselector.xAxis_field_name).should('have.text', 'pickup_hour');
    cy.get(CSSselector.yAxis_field_name).should('have.text', 'trip_time_in_secs');
    cy.get(visual.locators.x_axis_values)
      .should('have.length.greaterThan', 0)
      .and('be.visible');
    cy.get(visual.locators.y_axis_values)
      .should('have.length.greaterThan', 0)
      .and('be.visible');
    dashboards.closeVisual(false);
    cy.get(CSSselector.dashboard.visual.x_axis)
      .should('have.length.greaterThan', 0)
      .and('be.visible');

    // this test will fail because values of y-axis on visual are not visible in dashboard view.
    // cy.get(selector.dashboard.visual.y_axis).should('have.length.greaterThan', 0).and('be.visible');

    // verify that after converting box-plot visual into horizontal, x-axis and y-axis of visual contains desired labels and data in visual edit view in reversed order
    cy.reload();
    cy.wait('@visualGet');
    cy.get('.dash-viz-wrapper table').should('be.visible');
    visual.editSpecificVisual(1);
    cy.get(CSSselector.visual_settings_mark)
      .click()
      .should('have.class', 'open');
    cy.get(visual.locators.horizontal_box_plot)
      .find('input')
      .check();
    cy.get(CSSselector.xAxis_field_name).should('have.text', 'trip_time_in_secs');
    cy.get(CSSselector.yAxis_field_name).should('have.text', 'pickup_hour');
    cy.get(visual.locators.x_axis_values).should('have.length.greaterThan', 0);
    // .and('be.visible');
    // Currently cypress do not support visibility check of element which parent overflow properties hidden
    // which in this case the SVG element itself:(
    // https://docs.cypress.io/guides/core-concepts/interacting-with-elements#Visibility
    cy.get(visual.locators.y_axis_values).should('have.length.greaterThan', 0);
    // .and('be.visible');
    dashboards.closeVisual(false);
  });

  it('ARC-10083 2: Create boxplot visual with dataset that contains inner join', () => {
    // Given
    functions.createDatasetFromConnection(connectionName, databaseName, sourceTable, datasetName);
    cy.reload();
    dsPermissions.selectDataset(connectionName, datasetName);
    const joinColumns = {
      pickup_neighborhood: 'pickup_neighborhood',
    };

    // When
    dsManage.addJoin(databaseName, sourceTable, databaseName, targetTable, joinColumns, 1);
    dsManage.selectJoin('inner join');

    // Then
    dashboards.createDashboard(dashboardJoin);
  });
});
