import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as scatter_box_functions from 'lib/visual/visual_scatter_boxplot';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'E2E test dataset Cereals';
const dashboardName = 'E2E Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplotvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer_code_15',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['110', '120', '130', '140', '150', '160'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('boxplot_visual_settings_tests_risk1', () => {
    before(() => {
      logins.silent();
      cy.visit('/');

      functions.deleteDatasetFromConnection(connectionName, datasetName);
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      dashboards.createDashboard(dashboard);
    });

    // General
    it('1: Set Download as CSV locale', () => {
      combo_functions.enableDisableCSVDownload();
    });

    // Data
    it('2: Show data with null values', () => {
      dashboards.editVisual();
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('3: Number of rows to download in CSV', () => {
      dashboards.editVisual();
      combo_functions.setNumberofRowsinCSV('125');
    });

    it(' 4: Maximum number of rows to fetch', () => {
      dashboards.editVisual();
      combo_functions.setNumberofRowsToFetch('200');
    });

    it('5: Disable incremental queries', () => {
      dashboards.editVisual();
      combo_functions.setDisableIncrementalQueries();
    });

    it('6: Distinct dimensions to visualize', () => {
      dashboards.editVisual();
      scatter_box_functions.setNumberofDistinctDimensions('3');
      dashboards.refreshVisual();
      cy.get('.dash-viz-wrapper .viz-trellis-table tr > td > div').should('have.length', 3);
      dashboards.closeVisual(false);
    });

    // Trellis
    it('7: Display trellis borders', () => {
      dashboards.editVisual();
      combo_functions.displayTrelisBorder();
    });

    it('8: Independent dimension scale', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Trellis');
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='scale_dim_independent']")
        .scrollIntoView()
        .check();
      dashboards.refreshVisual();
      cy.get(visual.locators.visual.x_axis + ' > g.tick').should('have.length', 3);
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='scale_dim_independent']")
        .scrollIntoView()
        .uncheck();
      dashboards.refreshVisual();
      cy.get(visual.locators.visual.x_axis + ' > g.tick').should('have.length', 9);
      dashboards.closeVisual(false);
    });

    it('9: Independent aggregate scale', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Trellis');
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='scale_agg_independent']")
        .scrollIntoView()
        .check({ force: true });
      dashboards.refreshVisual();
      cy.get(visual.locators.visual.y_axis + ' > g.tick').should('have.length.greaterThan', 9);
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='scale_agg_independent']")
        .scrollIntoView()
        .uncheck({ force: true });
      dashboards.refreshVisual();
      dashboards.chooseVisualTab('settings');
      cy.get(visual.locators.visual.y_axis + ' > g.tick').should('have.length.greaterThan', 11);
      dashboards.closeVisual(false);
    });

    // Marks
    it('10: Display tooltips', () => {
      dashboards.editVisual();
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='show_tooltip']")
        .scrollIntoView()
        .check({ force: true });
      cy.get(visual.locators.boxplot.boxes)
        .last()
        .trigger('mouseover');
      cy.get('#tooltip').should('be.visible');
      cy.get(visual.locators.boxplot.boxes)
        .last()
        .trigger('mouseout');
      cy.get('#tooltip').should('not.be.visible');
      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='show_tooltip']")
        .scrollIntoView()
        .uncheck({ force: true });
      cy.get(visual.locators.boxplot.boxes)
        .last()
        .trigger('mouseover');
      cy.get('#tooltip').should('not.be.visible');
      visual.closeVisual(false);
    });

    it('11: Show detailed data button', () => {
      dashboards.editVisual();
      // getting the context menu is not working
      combo_functions.enableDisableDetailData('checked');
      dashboards.editVisual();
      combo_functions.enableDisableDetailData('unchecked');
    });

    // Skipped like in bar visual because setting the bar size range not figured out in tests
    it.skip('12: Bar size range', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Marks');
      cy.get("input[name='BAR_MARK_SIZE_RANGE']") // outdated!
        .scrollIntoView()
        .clear()
        .type('10-40');
      dashboards.refreshVisual();
      dashboards.saveVisual();
      cy.get(visual.locators.boxplot.boxes)
        .last()
        .should('have.attr', 'width', '40');
      visual.closeVisual(false);
    });

    it('13: Horizontal boxplot', () => {
      // TODO check how it should! work!
      dashboards.editVisual();
      visual.openSection('settings', 'Marks');

      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='horizontalBoxplots']")
        .scrollIntoView()
        .check();

      cy.get(visual.locators.boxplot.intervalLines).each(($line) => {
        const y1 = $line.attr('y1');
        const y2 = $line.attr('y2');
        expect(y1).to.equal(y2);
      });

      dashboards.chooseVisualTab('settings');
      cy.get("#dashboard-visual-settings-sidebar input[name='horizontalBoxplots']")
        .scrollIntoView()
        .uncheck();

      cy.get(visual.locators.boxplot.intervalLines).each(($line) => {
        const x1 = $line.attr('x1');
        const x2 = $line.attr('x2');
        expect(x1).to.equal(x2);
      });
      visual.closeVisual(false);
    });

    it('14: Outline width', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Marks');
      cy.get(
        "#dashboard-visual-settings-sidebar div[data_setting_id='lineWidth'] input[class='settings-slider-input']"
      ).trigger('click', 'right');
      visual.closeVisual(false);
    });

    // Axes
    it('15: Positive Aggregate Scale', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      cy.get('#dashboard-visual-settings-sidebar input[name="scale_agg_zero"]')
        .scrollIntoView()
        .uncheck();
      cy.get(visual.locators.visual.y_axis + ' > g.tick:nth-child(1)').each(($el) => {
        cy.wrap($el).should('not.have.text', '0');
      });
    });

    it('16: Include dimension axis label', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      cy.get(`${visual.locators.x_axis_labels}`).should('be.exist');
      cy.get('#dashboard-visual-settings-sidebar input[name="x_axis_label"]')
        .scrollIntoView()
        .uncheck();
      dashboards.refreshVisual();
      dashboards.saveVisual();
      cy.get(`${visual.locators.x_axis_labels}`).should('not.have.text');
    });

    it('17: Include measure axis label', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      cy.get(`${visual.locators.x_axis_labels}`).should('be.exist');
      cy.get('#dashboard-visual-settings-sidebar [data-test-id="checkbox-y_axis_label"]')
        .scrollIntoView()
        .uncheck();
      dashboards.refreshVisual();
      dashboards.saveVisual();
      cy.get(`${visual.locators.y_axis_labels}`).should('not.exist');
    });

    it('18: Show dimension axis', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      cy.get('#dashboard-visual-settings-sidebar [data-test-id="checkbox-SHOW_X_AXIS"]')
        .scrollIntoView()
        .uncheck();
      dashboards.refreshVisual();
      dashboards.saveVisual();
      cy.get(visual.locators.visual.x_axis).should('not.exist');
    });

    it('19: Show measure axis', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      cy.get('#dashboard-visual-settings-sidebar [data-test-id="checkbox-SHOW_Y_AXIS"]')
        .scrollIntoView()
        .uncheck();
      dashboards.refreshVisual();
      dashboards.saveVisual();
      cy.get(visual.locators.visual.y_axis).should('not.exist');
    });

    it('20: Grid lines enabled', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Axes');
      // cy.get(visual.locators.visual.grid_lines).each(($el) => {
      //     cy.wrap($el).should('not.be.visible');
      // });
      cy.get('#dashboard-visual-settings-sidebar input[name="gridlines_enabled"]')
        .scrollIntoView()
        .check();
      cy.get(visual.locators.visual.grid_lines).should('be.visible');
    });

    // it('Test 26: Min Max Y Axis Scale', () => {
    //     dashboards.editVisual();
    //     dashboards.editVisual();
    // });

    // it('Test 27: Gridlines enabled', () => {
    //     dashboards.editVisual();
    //     combo_functions.setGridLines();
    // });

    // it('Test 28: Include X-Axis label', () => {
    //     cloneAndOpenVisual(dashboardName, cloneDashboardName);
    //     combo_functions.setIncludeXAxisLabel();
    // });

    // it('Test 29: Include Y-Axis label', () => {
    //     cloneAndOpenVisual(dashboardName, cloneDashboardName);
    //     combo_functions.setIncludeYAxisLabel();
    // });

    // it('Test 30: Reverse X-Axis label', () => {
    //     cloneAndOpenVisual(dashboardName, cloneDashboardName);
    //     combo_functions.setIncludeXAxisLabel();
    // });

    // it('Test 31: Reverse Y-Axis label', () => {
    //     cloneAndOpenVisual(dashboardName, cloneDashboardName);
    //     combo_functions.setIncludeYAxisLabel();
    // });
  });
});
