import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { waitForVisualsLoaded } from 'lib/functions';
import * as logins from 'lib/logins';
import * as visual from 'lib/visual';
import { FontSizeSettingTestOptions } from 'lib/visual/fontsizeSetting';
import {
  testBasicFontStyleChangeToImpact,
  testSettingFontSizeHasTheDesiredEffect,
  testVisualTitleAlignmentSetToCenter,
  testVisualTitleFontWeightSetToBold,
  testVisualtitleIsHiddenInViewMode,
} from 'lib/visual/visual_common_tests';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'E2E test dataset Cereals';
const dashboardName = 'E2E Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplotvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          visual_subtitle: 'subtitle',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer_code_15',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['110', '120', '130', '140', '150', '160'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};
// Because of the intermittent timeout errors tracked on VIZ-1517,
// we set these particular tests to retry on fails
describe('Boxplot visual tests.', { retries: 2 }, () => {
  context('Style tab tests', () => {
    before(() => {
      logins.silent();
      cy.visit('/');
      functions.deleteDatasetFromConnection(connectionName, datasetName);
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      dashboards.createDashboard(dashboard);
    });

    beforeEach(() => {
      // visit is done in the cloneDashboard call
      // dashboards.createDashboard(dashboard);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // To setup the style tab
      cy.wait(2000);
    });

    function saveChangesReloadPageAndOpenVisualSetting() {
      // save close and reopen the visual
      dashboards.saveVisual();

      cy.reload();
      waitForVisualsLoaded();

      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
    }
    // Visual Basics
    it('1: Visual basic font type', () => {
      // GIVEN
      // WHEN
      testBasicFontStyleChangeToImpact();
    });

    it('2: Visual basic font size', () => {
      testSettingFontSizeHasTheDesiredEffect(FontSizeSettingTestOptions.SET_VISUAL_FONTSIZE, 22);
    });

    //   Visual Title
    it('3: Visual title font type', () => {
      // set the visual title font to be Impact
      visual.openSection('style', 'Visual Title');
      cy.get(visual.locators.style.titleSelector).within(() => {
        cy.get(visual.locators.style.titleFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).click();
          cy.get("a[data-value='arc-font-arial']").click();
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.titleSelector).within(() => {
        cy.get(visual.locators.style.titleFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).should(
            'have.attr',
            'data-value',
            'arc-font-arial'
          );
        });
      });
      cy.get(dashboards.locators.viz_title_class_span).should(
        'have.css',
        'font-family',
        'Arial, Helvetica, sans-serif'
      );
    });

    it('4: Visual title font size', () => {
      const visualTitleFontSizeSelector = "div[data-setting-id='title_font_size']";
      // set the visual title font size
      visual.openSection('style', 'Visual Title');
      cy.get(visual.locators.style.titleSelector).within(() => {
        cy.get(visualTitleFontSizeSelector).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('40');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font size is set right
      cy.get(visual.locators.style.titleSelector).within(() => {
        cy.get(visualTitleFontSizeSelector).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '40');
        });
      });
      cy.get(dashboards.locators.viz_title_class_span).should(
        'have.attr',
        'style',
        'font-size: 40px;'
      );
    });

    it('5: Visual title font weight', () => {
      testVisualTitleFontWeightSetToBold();
    });

    it('6: Visual title alignment', () => {
      testVisualTitleAlignmentSetToCenter();
    });

    it('7: Visual title hide in view mode', () => {
      testVisualtitleIsHiddenInViewMode();
    });

    //   Visual Subtitle
    it('8: Visual subtitle font', () => {
      visual.openSection('style', 'Visual Subtitle');
      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).click();
          cy.get("a[data-value='arc-font-helvetica']").click();
        });
      });

      saveChangesReloadPageAndOpenVisualSetting();

      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).should(
            'have.attr',
            'data-value',
            'arc-font-helvetica'
          );
        });
      });
      cy.get(dashboards.locators.viz_subtitle_class_span).should(
        'have.css',
        'font-family',
        '"Helvetica Neue", Helvetica, Arial, sans-serif'
      );
    });

    it('9: Visual subtitle font size', () => {
      visual.openSection('style', 'Visual Subtitle');
      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('30');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // cy.reload();
      // editVisual();
      // verify the font is set right
      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '30');
        });
      });
      cy.get(dashboards.locators.viz_subtitle_class_span).should(
        'have.attr',
        'style',
        'font-size: 30px;'
      );
    });

    it('10: Visual subtitle alignment', () => {
      visual.openSection('style', 'Visual Subtitle');
      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontAlignment).within(() => {
          cy.get("input[value='Right']").click();
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.subtitleSelector).within(() => {
        cy.get(visual.locators.style.subtitleFontAlignment).within(() => {
          cy.get("input[value='Right']").then(($el) => {
            const isChecked = $el.attr('checked');
            assert.isTrue(typeof isChecked !== typeof undefined);
            // assert.isTrue($el.attr("checked"));
          });
        });
      });

      cy.get(dashboards.locators.viz_subtitle_container).should(
        'have.attr',
        'style',
        'text-align: right;'
      );
    });

    //   Tooltip
    it('11: Visual style tooltip font', () => {
      visual.openSection('style', 'Tooltip');
      cy.get(visual.locators.style.tooltipSelector).within(() => {
        cy.get(visual.locators.style.tooltipFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).click();
          cy.get("a[data-value='arc-font-tahoma']").click();
        });
      });
      saveChangesReloadPageAndOpenVisualSetting();
      cy.get(visual.locators.style.tooltipSelector).within(() => {
        cy.get(visual.locators.style.tooltipFontSelector).within(() => {
          cy.get(visual.locators.style.dropDownSelector).should(
            'have.attr',
            'data-value',
            'arc-font-tahoma'
          );
        });
      });

      cy.get(visual.locators.boxplot.boxes)
        .first()
        .trigger('mouseover');
      cy.get('#tooltip .popover-table').should(
        'have.css',
        'font-family',
        'Tahoma, Geneva, sans-serif'
      );
    });

    it('12: Visual style tooltip size ', () => {
      visual.openSection('style', 'Tooltip');
      cy.get(visual.locators.style.tooltipSelector).within(() => {
        cy.get(visual.locators.style.tooltipFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('20');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.tooltipSelector).within(() => {
        cy.get(visual.locators.style.tooltipFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '20');
        });
      });

      cy.get(visual.locators.boxplot.boxes)
        .last()
        .trigger('mouseover');
      cy.get('#tooltip .popover-table')
        .should('have.attr', 'style')
        .and('match', /font-size:\s*20px/);
    });

    // Visual Axes and Labels
    it('13: Axis lable font size ', () => {
      visual.openSection('style', 'Visual Axes and Labels');
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.axisLableFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('25');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.axisLableFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '25');
        });
      });
      cy.get(visual.locators.visual.y_axis)
        .next()
        .should('have.attr', 'style', 'font-size: 25px;');
    });

    it('14: Axis ticks font size ', () => {
      visual.openSection('style', 'Visual Axes and Labels');
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.axisTicksFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('35');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.axisTicksFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '35');
        });
      });
      cy.get(visual.locators.visual.y_axis + ' text').should(
        'have.attr',
        'style',
        'text-anchor: end; font-size: 35px;'
      );
    });

    it('15: Trellis Font Size ', () => {
      visual.openSection('style', 'Visual Axes and Labels');
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.trellisFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('30');
        });
      });
      // save close and reopen the visual
      saveChangesReloadPageAndOpenVisualSetting();
      // verify the font is set right
      cy.get(visual.locators.style.axisSelector).within(() => {
        cy.get(visual.locators.style.trellisFontSize).within(() => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '30');
        });
      });
      cy.get(visual.locators.visual.trellis).should('have.attr', 'style', 'font-size: 30px;');
    });
  });
});
