import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const colors = ['#397b21', '#ce0000'];
const colorsCheck = ['rgb(57, 123, 33)', 'rgb(206, 0, 0)'];

const dashboard: Dashboard = {
  title: 'simple_extension_visual_dashboard',
  subTitle: 'simple_extension_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'ext visuals',
      sheet_number: 1,
      visuals: [
        {
          name: 'extensionvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-extension',
          visual_title: 'simple_extension_visual',
          linked: 'false',
          param_extension_text_field: `<h1 style="text-align: center;"><font color="${colors[0]}">This is a Header</font></h1><h1 style="text-align: left;"><span style="font-size: 20px;"><font color="${colors[1]}">This is first line in color red</font></span></h1><h1 style="text-align: left;"><span style="font-size: 20px;"><font color="${colors[1]}"><br></font></span></h1><table class="table table-bordered"><tbody><tr><td>Index</td><td>Links</td><td>Comments</td></tr><tr><td><p>1</p></td><td><a href="https://www.wikipedia.org/" style="background-color: rgb(255, 255, 255);">Link to wikipedia</a><br></td><td>wikipedia</td></tr></tbody></table><h1 style="text-align: left;"><span style="font-size: 20px;"><font color="${colors[1]}"><br></font></span></h1>`,
        },
      ],
    },
  ],
};

describe('Arcviz extension visuals tests.', () => {
  before(() => {
    logins.silent();
    dashboards.createDashboard(dashboard);
  });

  beforeEach(() => {
    dashboards.editVisual();
    dashboards.chooseVisualTab('data', false);
  });

  context('Extension visual tests', () => {
    it('Test 1: Verify content of extension visual', () => {
      cy.get('[data-test-id="visual-list-item"] h1:nth-child(1)')
        .should('have.attr', 'style', 'text-align: center;')
        .find('font')
        .should('have.text', 'This is a Header')
        .should('have.css', 'color', colorsCheck[0]);

      cy.get('[data-test-id="visual-list-item"] h1:nth-child(2)')
        .should('have.attr', 'style', 'text-align: left;')
        .find('span')
        .should('have.attr', 'style', 'font-size: 20px;')
        .find('font')
        .should('have.text', 'This is first line in color red')
        .should('have.css', 'color', colorsCheck[1]);

      cy.get('[data-test-id="visual-list-item"] table tbody tr:nth-child(2)')
        .find('td:nth-child(2)')
        .find('a')
        .should('have.attr', 'href', 'https://www.wikipedia.org/')
        .should('have.attr', 'style', 'background-color: rgb(255, 255, 255);')
        .should('have.text', 'Link to wikipedia');
    });

    it('Test 2: Verify "Cancel" button on the edit modal', () => {
      editTheVisual();
      cy.get('[data-test-id="super-modal"] [data-test-id="modal-cancel-btn"]').click();
      cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
      verifyTheVisualNotUpdated();
    });

    it('Test 3: Verify "X" button on the edit modal', () => {
      editTheVisual();
      cy.get('[data-test-id="super-modal"] [data-test-id="close-btn"]').click();
      cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
      verifyTheVisualNotUpdated();
    });

    it('Test 4: Verify autocompletion on the edit modal', () => {
      cy.get('[data-test-id="app-shelves"] [data-test-id="viz-html-view"]').click();

      cy.get('[data-test-id="super-modal"] [data-test-id="chart-container"]')
        .find('.ace_editor textarea')
        .type('h', { force: true });

      // select autocomplete option "h1"
      cy.get('.ace_autocomplete .ace_scroller')
        .find('.ace_line')
        .contains('h1')
        .click({ force: true });

      // verify that "h" autocompleted to "h1"
      cy.get(
        '[data-test-id="super-modal"] [data-test-id="chart-container"] .ace_editor .ace_text-layer .ace_line:nth-last-child(1)'
      )
        .find('.ace_text:nth-last-child(1)')
        .should('have.text', 'h1');

      // update autocomplete options for 'k' from 'h'
      cy.get('[data-test-id="super-modal"] [data-test-id="chart-container"]')
        .find('.ace_editor textarea')
        .type('{backspace}{backspace}k', { force: true });

      cy.get('[data-test-id="viz-html-editor-autocomplete"] input').uncheck();

      cy.get('[data-test-id="super-modal"] [data-test-id="chart-container"]')
        .find('.ace_editor textarea')
        .type('{backspace}h', { force: true });

      // verify autocomplete options are not updated despite typing "h"
      cy.get('.ace_autocomplete .ace_scroller').should('not.include.text', 'h1');

      cy.get(
        '[data-test-id="super-modal"] [data-test-id="chart-container"] .ace_editor .ace_text-layer .ace_line:nth-last-child(1)'
      )
        .find('.ace_text:nth-last-child(1)')
        .should('have.text', 'h');

      cy.get('[data-test-id="super-modal"] [data-test-id="close-btn"]').click();
      cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
    });

    it('Test 5: Verify "Refresh Preview" button on the edit modal', () => {
      editTheVisual();

      cy.get('[data-test-id="super-modal"] [data-test-id="refresh-html-preview"]').click();

      cy.get('[data-test-id="super-modal"] [data-test-id="viz-html-preview"]')
        .find('table tr:nth-child(1) td:nth-child(1)')
        .should('have.text', 'Changed index');

      cy.get('[data-test-id="super-modal"] [data-test-id="modal-cancel-btn"]').click();
      cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);

      verifyTheVisualNotUpdated();
    });
  });
});

function editTheVisual() {
  cy.get('[data-test-id="app-shelves"] [data-test-id="viz-html-view"]').click();

  const changedExtensionTextField = dashboard.sheets[0].visuals[0].param_extension_text_field?.replace(
    'Index',
    'Changed index'
  );
  // Change to now use ace editor
  cy.get('[data-test-id="super-modal"] [data-test-id="chart-container"]')
    .find('.ace_editor')
    .then(($elem) => {
      cy.window()
        .its('ace' as any)
        .invoke('edit', $elem.attr('id'))
        .invoke('setValue', changedExtensionTextField);
    });

  // Give ace.js some time to finish processing, otherwise we get a getTokens exception
  // from that library
  cy.wait(2000);
}

function verifyTheVisualNotUpdated() {
  cy.get('[data-test-id="visual-list-item"] table tbody tr:nth-child(1)')
    .find('td:nth-child(1)')
    .should('have.text', 'Index')
    .should('not.have.text', 'Changed index');
}
