import { logins } from 'lib/arctest';
import { refreshVisual } from 'lib/arctest/screen/builder/modal';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { waitForVisualsLoaded } from 'lib/functions';
import * as visual from 'lib/visual';
import { locators } from 'lib/visual/locators';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_tests from 'lib/visual/visual_common_tests';
import * as scatter_box_functions from 'lib/visual/visual_scatter_boxplot';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const dashboardName = 'Cereal Scatters DB e2e';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.SIZE,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['50', '70', '80', '90', '100', '110'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Scatter Visual tests.', () => {
  context('scatter_visual_settings_tests_risk1', () => {
    before(() => {
      logins.silent();
      dashboards.createDashboard(dashboard);
    });

    // General
    it('1: Set Download as CSV locale', () => {
      const testName = 'testEnableDisableCSVDownload';
      console.log('Starting test ' + testName);
      combo_functions.enableDisableCSVDownload();
    });

    it('2: Enable Animation', () => {
      dashboards.editVisual();
      const testName = 'enableAnimation';
      console.log('Starting test ' + testName);
      combo_functions.enableDisableAnimation();
    });

    // Data
    it('3: Show data with null values', () => {
      const testName = 'showNullValues';
      console.log('Starting test ' + testName);
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('4: Number of rows to download in CSV', () => {
      dashboards.editVisual();
      const testName = 'csvDownloadRows';
      console.log('Starting test ' + testName);
      combo_functions.setNumberofRowsinCSV('125');
    });

    it('5: Maximum number of rows to fetch', () => {
      dashboards.editVisual();
      const testName = 'maxRowCount';
      visual.openSection('settings', 'Data');
      console.log('Starting test ' + testName);
      cy.get('#dashboard-visual-settings-sidebar input[name="MAX_ROWCOUNT"]')
        .scrollIntoView()
        .clear()
        .type('10{enter}');

      refreshVisual();
      waitForVisualsLoaded();

      cy.get('.dash-viz-wrapper').scrollIntoView();
      // cy.get('.dash-viz-wrapper').matchImage();
      cy.get('.dash-viz-wrapper .viz-chart table.viz-trellis-outertable').matchImage();
    });

    it('6: Disable incremental queries', () => {
      dashboards.editVisual();
      const testName = 'disableIncrementalQuery';
      console.log('Starting test ' + testName);
      combo_functions.setDisableIncrementalQueries();
    });

    // Trellis
    it('7: Display trellis borders', () => {
      dashboards.editVisual();
      visual.openSection('settings', 'Trellis');
      const testName = 'displayTrelisBorder';
      console.log('Starting test ' + testName);
      cy.get("input[name='TRELLIS_BORDERS']")
        .scrollIntoView()
        .uncheck();
      cy.get(locators.visual.table).should(
        'have.css',
        'border-bottom',
        '0px none rgb(102, 102, 102)'
      );
      cy.get("input[name='TRELLIS_BORDERS']")
        .scrollIntoView()
        .check();
      cy.get(locators.visual.table).should(
        'have.css',
        'border-bottom',
        '1px solid rgb(221, 221, 221)'
      );
    });

    it('8: Min Cell Width', () => {
      dashboards.editVisual();
      const testName = 'inpdDimScale';
      console.log('Starting test ' + testName);
      visual.openSection('settings', 'Trellis');
      cy.get('div[data_setting_id="trellis_cell_minimum_width"]')
        .scrollIntoView()
        .find('.settings-slider-input')
        .invoke('val', 650)
        .trigger('input')
        .trigger('change');
      cy.get(locators.visual.svg).each(($el) => {
        cy.wrap($el).should('have.attr', 'width', '650px');
      });
    });

    it('9: Min Cell Height', () => {
      dashboards.editVisual();
      const testName = 'inpdAggrScale';
      console.log('Starting test ' + testName);
      visual.openSection('settings', 'Trellis');
      cy.get('div[data_setting_id="trellis_cell_minimum_height"]')
        .scrollIntoView()
        .find('.settings-slider-input')
        .invoke('val', 600)
        .trigger('input')
        .trigger('change');
      cy.get(locators.visual.svg).each(($el) => {
        cy.wrap($el).should('have.attr', 'height', '600px');
      });
    });

    it('10: Independent dimension scale', () => {
      dashboards.editVisual();
      const testName = 'inpdDimScale';
      console.log('Starting test ' + testName);
      combo_functions.setinpdDimScaleTrellis();
    });

    it('11: Independent aggregate scale', () => {
      dashboards.editVisual();
      const testName = 'inpdAggrScale';
      console.log('Starting test ' + testName);
      combo_functions.setinpdAggrScaleTrellis();
    });

    // Marks
    it('12: Display tooltips', () => {
      dashboards.editVisual();
      // getting the tooltip is not working
      const testName = 'showTooltips';
      console.log('Starting test ' + testName);
      combo_functions.setEnableTooltipsSetting(dashboardName, 'unchecked');
      combo_functions.setEnableTooltipsSetting(dashboardName, 'checked');
    });

    it('13: Show detailed data button', () => {
      dashboards.editVisual();
      // getting the context menu is not working
      const testName = 'editDataSet';
      console.log('Starting test ' + testName);
      combo_functions.enableDisableDetailData('checked');
      dashboards.editVisual();
      combo_functions.enableDisableDetailData('unchecked');
    });

    it('14: Bar size range', () => {
      dashboards.editVisual();
      // slider not working
      const testName = 'barSizeRange';
      console.log('Starting test ' + testName);
    });

    it('15: Legend style', () => {
      dashboards.editVisual();
      const testName = 'legendStyle';
      console.log('Starting test ' + testName);
      visual_common_tests.testLegendPositionChangeBySwitchingTo('none');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('right');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('bottom');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('above');
    });

    it('16: Legend item width', () => {
      dashboards.editVisual();
      const testName = 'legendWidthOverride';
      console.log('Starting test ' + testName);
      combo_functions.setLegendWidth(dashboardName, '100');
    });

    it('17: Reverse legend order', () => {
      dashboards.editVisual();
      const testName = 'reverseLegendOrder';
      console.log('Starting test ' + testName);
      combo_functions.reverseLegendOrder();
    });

    it('18: Transition Delay', () => {
      dashboards.editVisual();
      const testName = 'transitionDelay';
      console.log('Starting test ' + testName);
      scatter_box_functions.setTransitionDelay('500');
    });

    it('19: Autoplay Transition Animation', () => {
      dashboards.editVisual();
      const testName = 'reverseLegendOrder';
      console.log('Starting test ' + testName);
      scatter_box_functions.setTransitionDelay('500');
    });

    it('20: Display linear trendline', () => {
      const testName = 'lineraTrendline';
      console.log('Starting ' + testName);
      visual.openSection('settings', 'Marks');
      cy.get("input[name='trendline_linear']")
        .scrollIntoView()
        .check();
      cy.get("input[name='trendline_color']")
        .scrollIntoView()
        .check();
      dashboards.saveVisual();
      cy.get(locators.style.trendlines).should('be.visible');
      dashboards.closeVisual(false);
    });

    it('21: Trendline decimals', () => {
      // not able to find trendline tooltip class
      const testName = 'editDataSet';
      console.log('Starting test ' + testName);
      visual.openSection('settings', 'Marks');
      cy.get('div[data_setting_id="trendline_decimal"]')
        .scrollIntoView()
        .find('.settings-slider-input')
        .invoke('val', 3)
        .trigger('change');
    });

    // Axes
    it('22: Auto detect time columns', () => {
      dashboards.editVisual();
      const testName = 'autoDetectTimeColumn';
      console.log('Starting test ' + testName);
      combo_functions.setAutoDetectTimeColumn();
    });

    it('23: X Axis Scale', () => {
      dashboards.editVisual();
      const testName = 'xAsixScale';
      console.log('Starting test ' + testName);
      scatter_box_functions.AxisScale('x', 'log10');
      dashboards.editVisual();
      scatter_box_functions.AxisScale('x', 'linear');
    });

    it('24: Y Axis Scale', () => {
      dashboards.editVisual();
      const testName = 'yAsixScale';
      console.log('Starting test ' + testName);
      scatter_box_functions.AxisScale('y', 'log10');
      dashboards.editVisual();

      refreshVisual();
      waitForVisualsLoaded();
      // sometimes assetrion is not working because visual is not loading at all
      // cy.wait(2000);
      // cy.get(locators.visual.y_axis + ' > g.tick').should('have.length.greaterThan', '28');
      scatter_box_functions.AxisScale('y', 'linear');
    });

    it('25: X Axis Continuity', () => {
      dashboards.editVisual();
      const testName = 'xAsixContinuity';
      console.log('Starting test ' + testName);
      scatter_box_functions.AxisContinuity('x', 'checked');
      dashboards.editVisual();
      scatter_box_functions.AxisContinuity('x', 'unchecked');
    });

    it('26: Y Axis Continuity', () => {
      dashboards.editVisual();
      const testName = 'yAsixContinuity';
      console.log('Starting test ' + testName);
      scatter_box_functions.AxisContinuity('y', 'checked');
      dashboards.editVisual();
      scatter_box_functions.AxisContinuity('y', 'unchecked');
    });

    it.skip('27: Min Max X Axis Scale', () => {
      // skipping this because, it will be disabel in the dashbaord we are creating
      dashboards.editVisual();
      const testName = 'minMaxXAsixScale';
      console.log('Starting ' + testName);
      scatter_box_functions.setMinMaxScale('x', 'min', '10');
      dashboards.editVisual();
      scatter_box_functions.setMinMaxScale('x', 'max', '20');
    });

    it.skip('28: Min Max Y Axis Scale', () => {
      // skipping this because, it will be disabel in the dashbaord we are creating
      dashboards.editVisual();
      const testName = 'minMaxYAsixScale';
      console.log('Starting test ' + testName);
      scatter_box_functions.setMinMaxScale('y', 'min', '15');
      dashboards.editVisual();
      scatter_box_functions.setMinMaxScale('y', 'max', '25');
    });

    it('29: Gridlines enabled', () => {
      // dashboards.createDashboard(dashboard);
      dashboards.editVisual();
      const testName = 'gridLinesEnabled';
      console.log('Starting test ' + testName);
      visual.openSection('settings', 'Axes');
      cy.get('#dashboard-visual-settings-sidebar input[name="gridlines_enabled"]')
        .scrollIntoView()
        .check();

      refreshVisual();
      waitForVisualsLoaded();

      cy.get(locators.visual.grid_line).should('be.visible');
      dashboards.saveVisual();
    });

    it('30: Include X-Axis label', () => {
      visual.openSection('settings', 'Axes');
      const testName = 'includeXAxisLabel';
      console.log('Starting test ' + testName);
      cy.get("#dashboard-visual-settings-sidebar input[name='x_axis_label']")
        .scrollIntoView()
        .uncheck();
      dashboards.saveVisual();
      cy.wait(1000);
      cy.get(visual.locators.x_axis_labels).should('be.empty');
    });

    it('31: Include Y-Axis label', () => {
      visual.openSection('settings', 'Axes');
      const testName = 'includeYAxisLabel';
      console.log('Starting test ' + testName);
      cy.get("#dashboard-visual-settings-sidebar input[name='y_axis_label']")
        .scrollIntoView()
        .uncheck();
      dashboards.saveVisual();
      cy.get(visual.locators.y_axis_labels).should('not.exist'); // it is might empty on version 7.0.0!
    });

    it('32: Reverse X-Axis label', () => {
      dashboards.editVisual();
      const testName = 'reverseXAxisLabel';
      console.log('Starting test ' + testName);
      scatter_box_functions.testReverseXAxisScale();
    });

    it('33: Reverse Y-Axis label', () => {
      dashboards.editVisual();
      const testName = 'reverseYAxisLabel';
      console.log('Starting test ' + testName);
      scatter_box_functions.testReverseYAxisScale();
    });
  });
});
