import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as logins from 'lib/logins';
import * as visual from 'lib/visual';
import { FontSizeSettingTestOptions } from 'lib/visual/fontsizeSetting';
import * as visual_common_tests from 'lib/visual/visual_common_tests';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const dashboardName = 'Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          visual_subtitle: 'simple_scatter_visual1_subtitle',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.SIZE,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['50', '70', '80', '90', '100', '110'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};
// Because of the intermittent timeout errors tracked on VIZ-1517,
// we set these particular tests to retry on fails
describe('Scatter Visual tests.', { retries: 2 }, () => {
  context('Style tab tests', () => {
    before(() => {
      logins.silent();
      cy.visit('/');
      dashboards.createDashboard(dashboard);
    });

    beforeEach(() => {
      // visit is done in the cloneDashboard call
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // To setup the style tab
      cy.wait(2000);
    });

    // Visual Basics
    it('1: Visual basic font type', () => {
      visual_common_tests.testBasicFontStyleChangeToImpact();
      dashboards.closeVisual(false);
    });

    it('2: Visual basic font size', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_VISUAL_FONTSIZE,
        22
      );
      dashboards.closeVisual(false);
    });

    //   Visual Title
    it('3: Visual title font type', () => {
      visual_common_tests.testVisualTitleFontWeightSetToBold();
      dashboards.closeVisual(false);
    });

    it('4: Visual title font size', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_VISUAL_TITLE_FONTSIZE,
        40
      );
      dashboards.closeVisual(false);
    });

    it('5: Visual title font weight', () => {
      visual_common_tests.testVisualTitleFontWeightSetToBold();
      dashboards.closeVisual(false);
    });

    it('6: Visual title alignment', () => {
      visual_common_tests.testVisualTitleAlignmentSetToCenter();
      dashboards.closeVisual(false);
    });

    it('7: Visual title hide in view mode', () => {
      visual_common_tests.testVisualtitleIsHiddenInViewMode();
    });

    //   Visual Subtitle
    it('8: Visual subtitle font', () => {
      visual_common_tests.testVisualSubtitleFontTypeSetToHelvetica();
      dashboards.closeVisual(false);
    });

    it('9: Visual subtitle font size', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_VISUAL_SUBTITLE_FONTSIZE,
        30
      );
      dashboards.closeVisual(false);
    });

    it('10: Visual subtitle alignment', () => {
      visual_common_tests.testVisualSubtitleFontAlignmentSetToRight();
      dashboards.closeVisual(false);
    });

    //   Tooltip
    it('11: Visual style tooltip font', () => {
      visual.openSection('style', 'Tooltip');
      cy.get(visual.locators.style.tooltipSelector).within(($visualBasicSection) => {
        cy.get(visual.locators.style.tooltipFontSelector).within(($fontTypeSection) => {
          cy.get(visual.locators.style.dropDownSelector).click();
          cy.get("a[data-value='arc-font-tahoma']").click();
        });
      });
      dashboards.closeVisual(false);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      cy.get(visual.locators.style.tooltipSelector).within(($visualBasicSection) => {
        cy.get(visual.locators.style.tooltipFontSelector).within(($fontTypeSection) => {
          cy.get(visual.locators.style.dropDownSelector).should(
            'have.attr',
            'data-value',
            'arc-font-tahoma'
          );
        });
      });
      dashboards.closeVisual(false);
    });

    it('12: Visual style tooltip size ', () => {
      visual.openSection('style', 'Tooltip');
      cy.get(visual.locators.style.tooltipSelector).within(($visualTitleSection) => {
        cy.get(visual.locators.style.tooltipFontSize).within(($titleFontSizeSection) => {
          cy.get("input[name='TITLE_FONT_SIZE']").clear();
          cy.get("input[name='TITLE_FONT_SIZE']").type('20');
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      cy.reload();
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visual.locators.style.tooltipSelector).within(($visualTitleSection) => {
        cy.get(visual.locators.style.tooltipFontSize).within(($titleFontSizeSection) => {
          cy.get("input[name='TITLE_FONT_SIZE']").should('have.attr', 'value', '20');
        });
      });
      dashboards.closeVisual(false);
    });

    // Visual Axes and Labels
    it('13: Axis label font size ', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_AXIS_LABEL_FONTSIZE,
        25
      );
      dashboards.closeVisual(false);
    });

    it('14: Axis ticks font size ', () => {
      visual.openSection('style', 'Visual Axes and Labels');
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_AXIS_TICKS_FONTSIZE,
        35
      );
      dashboards.closeVisual(false);
    });

    it('15: Trellis Font Size ', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_TRELLIS_LABEL_FONTSIZE,
        30
      );
      dashboards.closeVisual(false);
    });

    it('16: Legend Font Size ', () => {
      visual_common_tests.testSettingFontSizeHasTheDesiredEffect(
        FontSizeSettingTestOptions.SET_LEGEND_FONTSIZE,
        30
      );
      dashboards.closeVisual(false);
    });
  });
});
