'use strict';

const puppeteer = require('puppeteer');
const querystring = require('querystring');

// const PAGE = 'http://192.168.1.150:9000/arc/apps/';
const CDV_URL = process.env.CDV_URL;
const URL_LABEL = process.env.URL_LABEL;

let r = new RegExp("(.*://.*?)/");
let m = CDV_URL.match(r);
const BASE_URL = m[1];
const LOGIN_URL = `${BASE_URL}/arc/apps/login`;
const LOGIN_REDIRECT_URL = `${BASE_URL}/arc/apps/home`;

const LOGIN_FIRST = true;
const LOGIN_USERNAME = 'vizapps_admin';
const LOGIN_PASSWORD = 'vizapps_admin';
const headless = ('0'!=process.env.HEADLESS) ;

let firstRedirect = true;

(async () => {
  const browser = await puppeteer.launch({
    headless: headless,
    ignoreHTTPSErrors: true
  });
  const page = await browser.newPage();
  let history = [];
  const startTimes = {};
  let lastRequest = new Date().getTime();
  let totalRuns = 0;
  let concurrentRuns = 0;
  let DEBUG = false;

  function debug(msg) {
    if (DEBUG) {
      console.log(msg);
    }
  }

  function info(msg) {
    console.log(msg);
  }

  function logHistory(header) {
    info(`<START ${header}>`);
    history.forEach(h => {
      info(JSON.stringify({
        method: h.request_method,
        url: h.request_url,
        headers: h.request_headers,
        post_data: h.request_post_data,
      }));
    });
    info(`<STOP ${header}>`);
  }

  function onRequestStarted(request) {
    if (request.url().startsWith('data')) {
      request.continue();
      return;
    }

    // do not record URLs pointing to other domains
    if (false == request.url().startsWith(BASE_URL)) {
      request.continue();
      return;
    }

    lastRequest = new Date().getTime();
    const request_id = request._requestId;
    if (startTimes[request_id]) {
      debug(`DUPLICATE ${request_id}`);
      request.continue();
      return;
    }

    if (request.url() == LOGIN_REDIRECT_URL && firstRedirect) {
      firstRedirect = false;
      request.abort();
      logHistory("LOGIN");
      history = [];
      return;
    }

    startTimes[request_id] = new Date().getTime();
    totalRuns = totalRuns + 1;
    concurrentRuns = concurrentRuns + 1;
    request.continue();
  }

  function dir(object) {
    const stuff = [];
    for (const s in object) {
        stuff.push(s);
    }
    stuff.sort();
    return stuff;
  }

  const tokenRE = new RegExp('.csrf_token\\("(.*?)"');
  const tokenRE2 = new RegExp('"csrfmiddlewaretoken".*?value="(.*?)"');

  function csrfmiddlewaretoken(responseText) {
    let m = responseText.match(tokenRE);
    if (!m) {
      m = responseText.match(tokenRE2);
    }

    if (m) {
      const token = m[1];
      info(`CSRF TOKEN ${token}`);
      return token;
    }

    return undefined;
  }

  function onRequestFinished(request) {
    if (request.url().startsWith('data')) {
      return;
    }

    // do not record URLs pointing to other domains
    if (false == request.url().startsWith(BASE_URL)) {
      return;
    }

    const response = request.response();
    concurrentRuns = concurrentRuns - 1;
    if (concurrentRuns === 0) {
      debug(concurrentRuns);
    }

    const request_id = request._requestId;
    const request_method = request.method();
    const request_url = request.url();
    const request_headers = request.headers();

    const parsedPostData = querystring.parse(request.postData());
    const request_post_keys = Object.keys(querystring.parse(request.postData()));
    const request_post_data = {};
    Object.keys(parsedPostData).forEach(k => {
      request_post_data[k] = parsedPostData[k];
    });

    const response_status = response.status();
    const request_runtime = (new Date().getTime() - startTimes[request_id])/100.0;
//    const response_headers = response.headers;
//    const response_size = response_headers['content-length'];
//    const response_body = response.body;

    delete startTimes[request_id];

    let response_body = '';
    if (response_status == 200) {
      // Look for csrfmiddlewaretoken
      // console.log(dir(request));
      // console.log(dir(response));

      response
            .buffer()
            .then(buffer => {
              const responseText = buffer.toString('utf8');
              const token = csrfmiddlewaretoken(responseText);
              if (token) {
                info(`token ${token}`);
              }
            });
    }

    const l = {
      request_id,
      request_method,
      request_url,
      request_headers,
      request_post_data,
      response_status,
      response_body,
    };

    info(`${request_id} ${request_method} ${response_status} ${request_runtime} ${request_url}`);

    if (request_method == 'POST') {
      info(l);
    }
    history.push(l);
  }

  async function waitForOngoingWork() {
    let currTime = new Date().getTime();
    let diff = currTime - lastRequest;
    let timeout = 5000;

    debug(`${diff} ${concurrentRuns}`);
    while (diff < timeout || concurrentRuns > 0) {
      await page.waitForTimeout(1000);
      currTime = new Date().getTime();
      diff = currTime - lastRequest;
      debug(`${diff} ${concurrentRuns}`);
    }
  }

  await page.setRequestInterception(true);
  page.on("request", onRequestStarted);
  page.on("requestfinished", onRequestFinished);

  if (LOGIN_FIRST) {
    await page.goto(LOGIN_URL, {
      waitUntil: 'networkidle0',
    });

    await page.type('#username', LOGIN_USERNAME);
    await page.type('#password', LOGIN_PASSWORD);

    await page.click('[data-test-id=\'submit\']');
    await page.waitForTimeout(2000);
  }

  await page.goto(CDV_URL, {
    waitUntil: 'networkidle0',
  });

  let currTime = new Date().getTime();
  let diff = currTime - lastRequest;
  let timeout = 5000;

  waitForOngoingWork();
  debug(startTimes);

  logHistory(URL_LABEL);
  history = [];

  await browser.close();
  process.exit();
})();
